           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                July 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_Ext_choice = Csp_semantics + Csp_Prefix + Csp_STOP:

(*  The following simplification rules are deleted in this theory file *)
(*  because they unexpectly rewrite UnionT and InterT.                 *)
(*                  Union (B ` A) = (UN x:A. B x)                      *)
(*                  Inter (B ` A) = (INT x:A. B x)                     *)

declare Union_image_eq [simp del]
declare Inter_image_eq [simp del]

(*****************************************************************

         1. Union Ts     : dom_T
         2. EXTCHf Ts Fs : dom_F
         3. EXTCHsf SFs  : dom_SF
         4. 

 *****************************************************************)

(*********************************************************
               Lemmas (Ext_choicet in Dom_T)
 *********************************************************)

(*** Ext_choicet in dom_T ***)

lemma Ext_choicet_in : 
  "ALL Ts. Ts ~= {} & (ALL T:Ts. T:dom_T) --> Union Ts: dom_T"
apply (intro allI impI)

apply (simp add: dom_T_def CT1_def)
apply (simp (no_asm) add: prefix_closed_def)
apply (intro allI impI)

apply (erule exE)
apply (erule conjE)+
apply (erule bexE)
apply (rename_tac Ts s t T)
apply (rule_tac x="T" in bexI)

apply (drule_tac x="T" in bspec)
apply (simp)
apply (blast intro: prefix_closed_propI)
apply (simp)
done

(*** Ext_choicet in dom_T ***)

lemma Ext_choicet_in_rep : 
  "ALL Ts. Ts ~= {} --> Union (Rep_dom_T ` Ts): dom_T"
apply (intro allI)
apply (insert Ext_choicet_in)
apply (drule_tac x="Rep_dom_T ` Ts" in spec)
apply (simp add: Rep_dom_T)
done

(*********************************************************
               Lemmas (Ext_choicef in Dom_F)
 *********************************************************)

(*** Ext_choicef in dom_F ***)

lemma Ext_choicef_in :
  "ALL Ts Fs. (ALL F:Fs. F:dom_F) --> EXTCHf Ts Fs: dom_F"
apply (intro allI impI)

apply (simp add: Ext_choicef_def)
apply (simp add: dom_F_def)
apply (simp (no_asm) add: CF1_def)
apply (intro allI impI)

  (* s = []t & Ball Fs (op : ([]t, X)) *)

apply (erule conjE)+
apply (erule disjE)
apply (rule disjI1)
apply (simp)
apply (rule ballI)
apply (rename_tac Fs s X Y F)

apply (drule_tac x="F" in bspec)
apply (simp)
apply (erule conjE)
apply (drule_tac x="F" in bspec)
apply (simp)
apply (blast intro: CF1_propI)

  (*  ~= []t & Bex Fs (op : (s, X)) *)

apply (erule disjE)
apply (rule disjI2)
apply (rule disjI1)
apply (erule conjE)+
apply (simp)
apply (erule bexE)
apply (rename_tac Fs s X Y F)

apply (drule_tac x="F" in bspec)
apply (simp)
apply (rule_tac x="F" in bexI)
apply (blast intro: CF1_propI)
apply (simp)

  (* s = []t & X <= Evset & Bex Ts (op : [Tick]t) *)

apply (rule disjI2)
apply (rule disjI2)
apply (auto)
done

(*** Ext_choicef in dom_F ***)

lemma Ext_choicef_in_rep :
  "ALL Ts Fs. EXTCHf Ts (Rep_dom_F ` Fs): dom_F"
apply (intro allI)
apply (insert Ext_choicef_in)
apply (drule_tac x="Ts" in spec)
apply (drule_tac x="Rep_dom_F ` Fs" in spec)
apply (drule mp)
apply (simp_all add: Rep_dom_F)
done

(*********************************************************
               Lemmas (Ext_choicesf in Dom_SF)
 *********************************************************)

(* T2 *)

lemma Ext_choicesf_in_T2 :
  "ALL TFs. TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> CT2 (EXTCHsf TFs)"
apply (intro allI impI)
apply (simp add: Ext_choicesf_def CT2_def)
apply (intro allI impI)
apply (simp add: Ext_choicet_in Rep_dom_T Abs_dom_T_inverse 
                 pair_fst_set_nonemp)
apply (simp add: Ext_choicef_in Rep_dom_F Abs_dom_F_inverse)

apply (erule conjE)+
apply (erule bexE)
apply (rename_tac TFs s TF)

apply (drule_tac x="TF" in bspec)
apply (simp)

apply (simp add: dom_SF_def CT2_def)
apply (erule conjE)
apply (drule_tac x="s" in spec)

  (* Evset *)

apply (rule conjI)
apply (simp add: Ext_choicef_def)

apply (case_tac "s = []t")

    (* s=[]t *)
apply (rule disjI2)
apply (rule disjI2)
apply (simp)
apply (rule_tac x="TF" in bexI)
apply (simp add: nil_in_trace event_in_trace Rep_trace Abs_trace_inverse)
apply (simp)

    (* s~=[]t *)
apply (rule disjI2)
apply (rule disjI1)
apply (simp)
apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)

  (* EvsetTick *)

apply (simp add: Ext_choicef_def)
apply (rule disjI2)
apply (rule disjI1)

apply (rule conjI)

apply (simp add: app_t_nil)
apply (simp add: nil_in_trace event_in_trace Abs_trace_inject)

apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)
done

(* F2 *)

lemma Ext_choicesf_in_F2 :
  "ALL TFs. TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> CF2 (EXTCHsf TFs)"
apply (intro allI impI)
apply (simp add: Ext_choicesf_def CF2_def)
apply (intro allI impI)
apply (simp add: Ext_choicet_in Rep_dom_T Abs_dom_T_inverse 
                 pair_fst_set_nonemp)
apply (simp add: Ext_choicef_in Rep_dom_F Abs_dom_F_inverse)

apply (simp add: Ext_choicef_def)
apply (erule conjE)+
apply (erule disjE)

    (* s=[]t & (ALL x:snd ` TFs. ([]t, X) :f x) *)

apply (rule disjI1)
apply (simp)
apply (rule ballI)
apply (rename_tac TFs s X Y TF)

apply (erule conjE)
apply (drule_tac x="TF" in bspec)
apply (simp)
apply (drule_tac x="TF" in bspec)
apply (simp)
apply (simp add: dom_SF_def CF2_def)

    (* s ~= []t & (EX x: snd ` TFs. (s, X) :f x) *)
apply (erule disjE)
apply (rule disjI2)
apply (rule disjI1)
apply (erule conjE)

apply (simp)
apply (erule bexE)
apply (rename_tac TFs s X Y TF)

apply (simp add: dom_SF_def CF2_def)

apply (drule_tac x="TF" in bspec)
apply (simp)

apply (erule conjE)+
apply (drule_tac x="s" in spec)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Y" in spec)
apply (simp)

apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)

    (* s = []t & X <= Evset & (EX x:snd ` TFs. [Tick]t :t x) *)
apply (rule disjI2)
apply (rule disjI2)
apply (simp)
apply (case_tac "Tick : Y")

      (* Tick:Y --> contradiction *)
apply (drule_tac x="Tick" in spec)
apply (simp)
apply (erule conjE)+
apply (erule bexE)
apply (rename_tac TFs s X Y TF)
apply (rotate_tac 4)

apply (drule_tac x="TF" in bspec)
apply (simp)
apply (simp add: nil_in_trace Abs_trace_inverse)
apply (simp add: Rep_trace_inverse)

      (* Therefore Tick~:Y *)

apply (auto simp add: Evset_def)
done

(* F3 *)

lemma Ext_choicesf_in_F3 :
  "ALL TFs. TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> CF3 (EXTCHsf TFs)"
apply (intro allI impI)
apply (simp add: Ext_choicesf_def CF3_def)
apply (intro allI impI)
apply (simp add: Ext_choicet_in Rep_dom_T Abs_dom_T_inverse 
                 pair_fst_set_nonemp)
apply (simp add: Ext_choicef_in Rep_dom_F Abs_dom_F_inverse)

apply (simp add: Ext_choicef_def)
apply (erule exE)
apply (erule conjE)+

apply (erule nonempty_exE)
apply (erule exE)

    (* case 1: s=[]t *)
apply (erule disjE)
apply (rule_tac x="x" in bexI)
apply (simp add: nil_t_in_all_dom Rep_dom_T)
apply (simp)

    (* case 2: s~=[]t *)
apply (erule disjE)
apply (erule conjE)
apply (erule bexE)

apply (rename_tac TFs s X x TF)

apply (simp add: dom_SF_def CF3_def)
apply (drule_tac x="TF" in bspec)
apply (simp)

apply (erule conjE)+
apply (drule_tac x="s" in spec)
apply (drule mp)
apply (rule_tac x="X" in exI)
apply (simp)

apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)

    (* case 3: s=[]t *)

apply (simp add: nil_t_in_all_dom Rep_dom_T)
apply (rule_tac x="fst x" in exI)
apply (rule_tac x="snd x" in exI)
apply (simp)
done

(*** Ext_choicesf in dom_SF ***)

lemma Ext_choicesf_in :
  "ALL TFs. TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> (EXTCHsf TFs): dom_SF"
apply (intro allI impI)
apply (simp (no_asm) add: dom_SF_def)
apply (simp add: Ext_choicesf_in_T2)
apply (simp add: Ext_choicesf_in_F2)
apply (simp add: Ext_choicesf_in_F3)
done

(*********************************************************
                     Lemmas (mono)
 *********************************************************)

(*** T ***)

lemma Ext_choicet_mono: "ALL Ss Ts. (ALL S : Ss. EX T : Ts. S <= T)
  --> Union Ss <= Union Ts"
apply (auto)
done

(*** F ***)

lemma Ext_choicef_mono: 
  "ALL Ss Ts Es Fs. 
     (ALL S : Ss. EX T : Ts. S <= T) & 
     (ALL E : Es. EX F : Fs. E <= F) &
     (ALL F : Fs. EX E : Es. E <= F)
                    --> EXTCHf Ss Es <= EXTCHf Ts Fs"
apply (intro allI impI)
apply (simp add: Ext_choicef_def)

apply (simp add: subset_def)
apply (rule ballI)
apply (simp)

      (* Int F *)
  
apply (erule disjE)
apply (rule disjI1)
apply (elim conjE exE)
apply (simp)
apply (rule ballI)

apply (rotate_tac 2)
apply (drule_tac x="xa" in bspec)
apply (simp)

apply (erule bexE)
apply (drule_tac x="E" in bspec)
apply (simp)

apply (drule_tac x="([]t, X)" in bspec)
apply (simp)
apply (simp)

      (* Un F *)

apply (erule disjE)
apply (rule disjI2)
apply (rule disjI1)
apply (simp add: pair_fun_apply)
apply (elim conjE bexE)

apply (rotate_tac 2)
apply (drule_tac x="xa" in bspec)
apply (simp)
apply (erule bexE)
apply (drule_tac x="F" in bspec)
apply (simp)
apply (erule bexE)

apply (rule_tac x="F" in bexI)
apply (simp)
apply (simp)

      (* Un T *)

apply (rule disjI2)
apply (rule disjI2)
apply (elim conjE exE)

apply (rule_tac x="X" in exI)
apply (simp)
apply (erule bexE)

apply (drule_tac x="xa" in bspec)
apply (simp)
apply (erule bexE)
apply (drule_tac x="[Tick]t" in bspec)
apply (simp)

apply (rule_tac x="T" in bexI)
apply (simp)
apply (simp)
done

(*** SF ***)

lemma Ext_choicesf_mono: 
  "ALL SEs TFs. 
     SEs ~= {} & TFs ~= {} &
     (ALL SE : SEs. EX TF : TFs. SE <= TF) & 
     (ALL TF : TFs. EX SE : SEs. SE <= TF)
                --> EXTCHsf SEs <= EXTCHsf TFs"
apply (intro allI impI)
apply (simp add: Ext_choicesf_def)
apply (simp add: order_pair_def)

      (* fst SEs : dom_T *)

apply (insert Ext_choicet_in)
apply (drule_tac x="Rep_dom_T ` fst ` SEs " in spec)
apply (drule mp)
apply (simp add: image_iff)
apply (simp add: Rep_dom_T)

      (* fst TFs : dom_T *)
apply (insert Ext_choicet_in)
apply (drule_tac x="Rep_dom_T ` fst ` TFs " in spec)
apply (drule mp)
apply (simp add: image_iff)
apply (simp add: Rep_dom_T)

      (* snd SEs : dom_T *)
apply (insert Ext_choicef_in)
apply (drule_tac x="Rep_dom_T ` fst ` SEs" in spec)
apply (drule_tac x="Rep_dom_F ` snd ` SEs" in spec)
apply (drule mp)
apply (simp add: image_iff)
apply (simp add: Rep_dom_F)

      (* right TFs : dom_T *)
apply (insert Ext_choicef_in)
apply (drule_tac x="Rep_dom_T ` fst ` TFs" in spec)
apply (drule_tac x="Rep_dom_F ` snd ` TFs" in spec)
apply (drule mp)
apply (simp add: image_iff)
apply (simp add: Rep_dom_F)

apply (simp add: subset_T_def subset_F_def)
apply (simp add: Abs_dom_T_inverse Abs_dom_F_inverse)

      (* T *)

apply (insert "Union_mono_all_ex")
apply (drule_tac x="Rep_dom_T ` fst ` SEs" in spec)
apply (drule_tac x="Rep_dom_T ` fst ` TFs" in spec)
apply (drule mp)

apply (rule ballI)
apply (elim conjE exE)

apply (simp add: image_iff)
apply (erule bexE)

apply (drule_tac x="xa" in bspec)
apply (simp)
apply (erule bexE)
apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)

      (* F *)
apply (insert Ext_choicef_mono)
apply (drule_tac x="Rep_dom_T ` fst ` SEs" in spec)
apply (drule_tac x="Rep_dom_T ` fst ` TFs" in spec)
apply (drule_tac x="Rep_dom_F ` snd ` SEs" in spec)
apply (drule_tac x="Rep_dom_F ` snd ` TFs" in spec)
apply (drule mp)

apply (simp)

         (* T1 *)
apply (rule conjI)
apply (rule ballI)
apply (elim conjE exE)
apply (drule_tac x="S" in bspec)
apply (simp)
apply (erule bexE)
apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)

         (* F1 *)
apply (rule conjI)
apply (rule ballI)
apply (elim conjE exE)
apply (drule_tac x="E" in bspec)
apply (simp)
apply (erule bexE)
apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)

        (* F2 *)
apply (rule ballI)
apply (elim conjE exE)
apply (rotate_tac -1)
apply (drule_tac x="F" in bspec)
apply (simp)
apply (erule bexE)
apply (rule_tac x="SE" in bexI)
apply (simp)
apply (simp)

  (* *)
apply (simp)
done

(*********************************************************
                  Ext choice trace eval
 *********************************************************)

lemma Ext_choicet_eval:
  "ALL P Q ev. [[P [+] Q]]T ev = [[P]]T ev Un [[Q]]T ev"
apply (intro allI)
apply (simp add: eval_T_def)
apply (simp add: eval_TF_decompo)
apply (simp add: Ext_choicesf_def)

apply (insert Ext_choicet_in_rep)
apply (drule_tac x="{Abs_dom_T ([[P]]T ev), Abs_dom_T ([[Q]]T ev)}" in spec)
apply (simp add: Abs_dom_T_inverse)
done

(*********************************************************
                  Ext choice failure eval
 *********************************************************)

lemma Ext_choicef_eval:
  "ALL P Q ev. [[P [+] Q]]F ev = 
           {([]t,X) |X. ([]t,X) : [[P]]F ev Int [[Q]]F ev} Un
           {(s,X) |s X. s ~= []t & (s,X) : [[P]]F ev Un [[Q]]F ev} Un
           {([]t,X) |X. X <= Evset & [Tick]t : [[P]]T ev Un [[Q]]T ev}"

apply (intro allI)
apply (simp add: eval_F_def)
apply (simp add: eval_TF_decompo)
apply (simp add: Ext_choicesf_def)
apply (insert Ext_choicef_in_rep)
apply (drule_tac x="{Rep_dom_T (Abs_dom_T ([[P]]T ev)),
                     Rep_dom_T (Abs_dom_T ([[Q]]T ev))}" in spec)
apply (drule_tac x="{Abs_dom_F ([[P]]F ev), Abs_dom_F ([[Q]]F ev)}" in spec)
apply (simp add: Abs_dom_F_inverse)

apply (simp add: Ext_choicef_def)
apply (insert Ext_choicet_in_rep)
apply (drule_tac x="{Abs_dom_T ([[P]]T ev), Abs_dom_T ([[Q]]T ev)}" in spec)
apply (simp add: eval_T_def Rep_dom_T Abs_dom_T_inverse)
done

(*********************************************************
                 Prefix choice trace eval
 *********************************************************)

lemma Pre_choicet_eval:
  "ALL X Pf ev. [[? :X -> Pf]]T ev = 
        insert []t {[Ev a]t @t s |a s. a:X & s: [[Pf a]]T ev}"
apply (intro allI)
apply (simp add: eval_T_def Ext_choicesf_def)

apply (insert Ext_choicet_in)
apply (drule_tac x=
  "Rep_dom_T ` fst ` (insert STOPsf {a ->sf [[Pf a]]TF ev |a. a : X})"
  in spec)

apply (drule mp)
apply (simp add: STOPsf_def Abs_dom_T_inverse STOPt_in)
apply (simp add: Rep_dom_T)

apply (simp add: Abs_dom_T_inverse)
apply (simp add: Prefixsf_def STOPsf_def)

apply (simp only: image_Collect_fun)
apply (simp add: STOPt_in Abs_dom_T_inverse)
apply (simp add: Prefixt_in Abs_dom_T_inverse Rep_dom_T)
apply (simp add: STOPt_def)
apply (simp add: Prefixt_def)
apply (rule equalityI)

      (* <= *)
apply (simp add: subset_def)
apply (intro allI impI ballI)

apply (elim exE conjE)
apply (simp)
apply (erule disjE)
apply (simp)

apply (erule exE)
apply (rule disjI2)
apply (rule_tac x="a" in exI)
apply (rule_tac x="s" in exI)
apply (simp)

      (* => *)

apply (simp add: subset_def)
apply (intro allI impI ballI)

apply (elim exE conjE)
apply (rule disjI2)
apply (rule_tac x=
   "insert []t {[Ev a]t @t s |s. s :t fst ([[Pf a]]TF ev)}" in exI)
apply (rule conjI)
apply (rule_tac x="a" in exI)
apply (simp)

apply (simp)
apply (rule disjI2)
apply (rule_tac x="s" in exI)
apply (simp)
done

(*********************************************************
                 Prefix choice failure eval
 *********************************************************)

(*** notick in Union T ***)

lemma Pre_choicef_eval_no_tick:
  "ALL X Pf ev. 
       {([]t, Y) |Y.
            Y <= Evset &
            ([Tick]t : STOPt |
             (EX x. (EX a. x = a ->t Rep_dom_T (fst ([[Pf a]]TF ev)) &
                           a : X) &
                    [Tick]t : x))} = {}"
apply (auto)
apply (simp add: STOPt_def)
apply (simp add: event_in_trace nil_in_trace Abs_trace_inject)
apply (simp add: Prefixt_def)
apply (simp add: event_in_trace nil_in_trace Abs_trace_inject)
apply (erule exE)
apply (simp add: event_in_trace Abs_trace_inverse)
apply (simp add: event_in_trace Abs_trace_inject event_app_in_trace Rep_trace)
done

(*** Union Fi ***)

lemma Pre_choicef_eval_Un:
  "ALL X Pf ev. 
       {(s, Y). s ~= []t &
               ((s, Y) : STOPf |
             (EX x. (EX a. x = a ->f Rep_dom_F (snd ([[Pf a]]TF ev)) &
                           a : X) & (s, Y) : x))}
      = {([Ev a]t @t s, Y) |a s Y. a:X & (s,Y):f (snd ([[Pf a]]TF ev))}"
apply (auto simp add: Prefixf_def STOPf_def)
apply (simp add: event_in_trace nil_in_trace Abs_trace_inverse)
apply (simp add: nil_in_trace Abs_trace_inject event_app_in_trace Rep_trace)
done

(*** Inter Fi ***)

lemma Pre_choicef_eval_Int:
  "ALL X Pf ev. 
     {([]t, Y) |Y.
            ([]t, Y) : STOPf &
            (ALL x.
                (EX a. x = a ->f Rep_dom_F (snd ([[Pf a]]TF ev)) &
                       a : X) -->
                ([]t, Y) : x)}
      = {([]t,Y) |Y. Ev ` X Int Y = {}}"
apply (intro allI)
apply (simp add: Prefixf_def STOPf_def)
apply (rule equalityI)

      (* <= *)

apply (simp add: subset_def EvsetTick_def)
apply (intro allI impI)

apply (case_tac "Ev ` X Int b = {}")
apply (simp)

apply (erule nonempty_exE)
apply (erule exE)

apply (insert event_tick_or_Ev)
apply (drule_tac x="x" in spec)
apply (erule disjE)
apply (simp add: image_def)

apply (erule exE)
apply (drule_tac x="{([]t, X) |X. Ev a ~: X} Un
                    {([Ev a]t @t s, X) |s X.
                     (s, X) :f snd ([[Pf a]]TF ev)}" in spec)
apply (drule mp)
apply (rule_tac x="a" in exI)
apply (simp add: image_def)

apply (simp)
apply (elim conjE exE)
apply (simp add: event_in_trace nil_in_trace Abs_trace_inverse)
apply (simp add: nil_in_trace Abs_trace_inject event_app_in_trace Rep_trace)

      (* => *)

apply (simp add: subset_def EvsetTick_def)
apply (intro allI impI)
apply (erule exE)
apply (simp)
apply (intro impI)

apply (insert Int_no_empty)
apply (rotate_tac -1)
apply (drule_tac x="Ev a" in spec)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="b" in spec)
apply (simp)
done

(*******************************
         Prefix choice F
 *******************************)

lemma Pre_choicef_eval:
  "ALL X Pf ev. [[? :X -> Pf]]F ev = 
        {([]t,Y) |Y. Ev ` X Int Y = {}} Un
        {([Ev a]t @t s, Y) |a s Y. a:X & (s,Y): [[Pf a]]F ev}"
apply (intro allI)
apply (simp add: eval_F_def Ext_choicesf_def)
apply (simp only: image_Collect_fun)

apply (simp add: Prefixsf_def STOPsf_def)

apply (simp add: STOPt_in Abs_dom_T_inverse)
apply (simp add: STOPf_in Abs_dom_F_inverse)
apply (simp add: Prefixt_in Rep_dom_T Abs_dom_T_inverse)
apply (simp add: Prefixf_in Rep_dom_F Abs_dom_F_inverse)

apply (insert Ext_choicef_in)
apply (drule_tac x=
  "insert STOPt {a ->t Rep_dom_T (fst ([[Pf a]]TF ev)) |a. a : X}" in spec)
apply (drule_tac x=
  "insert STOPf {a ->f Rep_dom_F (snd ([[Pf a]]TF ev)) |a. a : X}" in spec)

apply (drule mp)
apply (simp add: Ball_def)
apply (intro allI)
apply (simp add: STOPf_in)

apply (intro impI)
apply (erule exE)
apply (simp add: Prefixf_in Rep_dom_F)
apply (simp add: Abs_dom_F_inverse)

apply (simp add: Ext_choicef_def)

apply (simp add: Pre_choicef_eval_no_tick)
apply (simp add: Pre_choicef_eval_Un)
apply (simp add: Pre_choicef_eval_Int)
done

(*********************************************************
                 Tick and Prefix choice
 *********************************************************)

(* Tick notin Prefix choicet *)

lemma tick_notin_Pre_choicet:
    "ALL X Pf ev. [Tick]t ~: [[? :X -> Pf]]T ev"
apply (simp add: Pre_choicet_eval)
apply (simp add: event_in_trace Abs_trace_inverse)
apply (simp add: event_in_trace event_app_in_trace Rep_trace Abs_trace_inject)
apply (simp add: event_in_trace nil_in_trace Abs_trace_inject)
done

(* Tick notin Prefix choicef *)

lemma tick_notin_Pre_choicef:
    "ALL X Y Pf ev. ([Tick]t,Y) ~: [[? :X -> Pf]]F ev"
apply (simp add: Pre_choicef_eval)
apply (simp add: event_in_trace Abs_trace_inverse)
apply (simp add: event_in_trace event_app_in_trace Rep_trace Abs_trace_inject)
apply (simp add: event_in_trace nil_in_trace Abs_trace_inject)
done

(****************** to add them again ******************)

declare Union_image_eq [simp]
declare Inter_image_eq [simp]

end
