           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                July 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_Parallel = Csp_semantics:

(*  The following simplification rules are deleted in this theory file *)
(*  because they unexpectly rewrite (notick | t = []t)                 *)
(*                                                                     *)
(*                  disj_not1: (~ P | Q) = (P --> Q)                   *)

declare disj_not1 [simp del]

(*****************************************************************

         1. S |[X]|t T : dom_T
         2. E |[X]|f F : dom_F
         3. SE |[X]|sf TF : dom_SF
         4. 

 *****************************************************************)

(*********************************************************
               Lemma (Parallelt in Dom_T)
 *********************************************************)

lemma Parallelt_in : 
  "ALL X S T. S:dom_T & T:dom_T --> S |[X]|t T: dom_T"
apply (intro allI impI)

apply (simp (no_asm) add: dom_T_def CT1_def)

  (* non-empty *)

apply (rule conjI)
apply (simp add: Parallelt_def)
apply (rule_tac x="{[]t}" in exI)
apply (simp)
apply (rule_tac x="[]t" in exI)
apply (rule_tac x="[]t" in exI)
apply (simp add: nil_t_in_all_dom)
apply (simp add: par_rep_def)
apply (simp add: nil_in_trace Abs_trace_inverse)
apply (simp add: par_list_def)

  (* prefix closed *)

apply (simp (no_asm) add: prefix_closed_def)
apply (intro allI impI)
apply (simp add: Parallelt_def)
apply (elim conjE exE)

apply (insert par_trace_prefix)
apply (drule_tac x="X" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule_tac x="sa" in spec)
apply (drule_tac x="ta" in spec)
apply (simp)
apply (elim conjE exE)

apply (rule_tac x="s' |[X]|set t'" in exI)
apply (simp)
apply (rule_tac x="s'" in exI)
apply (rule_tac x="t'" in exI)

  (* T1 *)

apply (simp add: dom_T_def CT1_def)
apply (blast intro: prefix_closed_propI)
done

(*********************************************************
               Lemma (Parallelf in Dom_F)
 *********************************************************)

(*
   Y12 = Y1 - (insert Tick (Ev ` X));
   Z12 = Z Int Y12;
   Y1' = Y1 - Y12;
   Y2' = Y2 - Y12;
   Z1' = Z Int Y1';
   Z2' = Z Int Y2';
   Z12 = Z Int Y12;
   Z1  = Z1' Un Z12';
   Z2  = Z2' Un Z12'

   Hence,

       Y12 = Y2 - insert Tick (Ev ` X);
       Z12 = Z Int (Y2 - insert Tick (Ev ` X));
       Y1' = Y1 - (Y2 - insert Tick (Ev ` X));
       Y2' = Y2 - (Y2 - insert Tick (Ev ` X));
       Z1' = Z Int (Y1 - (Y2 - insert Tick (Ev ` X)));
       Z2' = Z Int (Y2 - (Y2 - insert Tick (Ev ` X)));
       Z12 = Z Int (Y2 - insert Tick (Ev ` X));
       Z1  = Z Int (Y1 - (Y2 - insert Tick (Ev ` X))) Un
             Z Int (Y2 - insert Tick (Ev ` X));
       Z2  = Z Int (Y2 - (Y2 - insert Tick (Ev ` X))) Un
             Z Int (Y2 - insert Tick (Ev ` X))

   Finally,

       (Z1 - (insert Tick (Ev ` X))) = Z12
       (Z2 - (insert Tick (Ev ` X))) = Z12
                            Z1 Un Z2 = Z12
*)

(* Z1 Un Z2 = Z *)

lemma Parallelf_in_Z:
  "ALL X Y Z Y1 Y2.
       (Z <= Y1 Un Y2 &
       Y1 - insert Tick (Ev ` X) = Y2 - insert Tick (Ev ` X))
          --> Z = Z Int (Y1 - (Y2 - insert Tick (Ev ` X))) Un
                  Z Int (Y2 - insert Tick (Ev ` X)) Un
                 (Z Int (Y2 - (Y2 - insert Tick (Ev ` X))) Un
                  Z Int (Y2 - insert Tick (Ev ` X)))"
apply (auto)
done

(* Z1-(insert Tick (Ev ` X)) = Z2-(insert Tick (Ev ` X)) *)

lemma Parallelf_in_Z1_Z2:
  "ALL X Y Z Y1 Y2.
       (Z <= Y1 Un Y2 &
       Y1 - insert Tick (Ev ` X) = Y2 - insert Tick (Ev ` X))
          --> Z Int (Y1 - (Y2 - insert Tick (Ev ` X))) Un
              Z Int (Y2 - insert Tick (Ev ` X)) -
              insert Tick (Ev ` X) =
              Z Int (Y2 - (Y2 - insert Tick (Ev ` X))) Un
              Z Int (Y2 - insert Tick (Ev ` X)) -
              insert Tick (Ev ` X)"
apply (auto)
done

(* Parallelf in dom_F *)

lemma Parallelf_in : 
  "ALL X F1 F2. F1:dom_F & F2:dom_F --> F1 |[X]|f F2: dom_F"
apply (intro allI impI)

apply (simp (no_asm) add: dom_F_def CF1_def)
apply (intro allI impI)

apply (simp add: Parallelf_def)
apply (elim conjE exE)

apply (rename_tac X F1 F2 u Y Z Y1 Y2 s t)

      (* Z1 *)
apply (rule_tac x="Z Int (Y1 - (Y2 - insert Tick (Ev ` X))) Un
                   Z Int (Y2 - insert Tick (Ev ` X))" in exI)
      (* Z2 *)
apply (rule_tac x="Z Int (Y2 - (Y2 - insert Tick (Ev ` X))) Un
                   Z Int (Y2 - insert Tick (Ev ` X))" in exI)

apply (simp add: Parallelf_in_Z)
apply (simp add: Parallelf_in_Z1_Z2)

apply (rule_tac x="s" in exI)
apply (rule_tac x="t" in exI)

apply (simp add: dom_F_def)
apply (unfold CF1_def)

apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule_tac x="Y1" in spec)
apply (drule_tac x="Y2" in spec)

      (* Z1 *)
apply (drule_tac x="Z Int (Y1 - (Y2 - insert Tick (Ev ` X))) Un
                    Z Int (Y2 - insert Tick (Ev ` X))" in spec)
      (* Z2 *)
apply (drule_tac x="Z Int (Y2 - (Y2 - insert Tick (Ev ` X))) Un
                    Z Int (Y2 - insert Tick (Ev ` X))" in spec)
apply (auto)
done

(*********************************************************
               Lemma (Parallelsf in Dom_SF)
 *********************************************************)

(*** T2 ***)

lemma Parallelsf_in_T2 :
  "ALL X TF1 TF2. TF1:dom_SF & TF2:dom_SF --> CT2 (TF1 |[X]|sf TF2)"
apply (intro allI impI)
apply (simp add: Parallelsf_def CT2_def)
apply (intro allI impI)
apply (simp add: Parallelt_in Rep_dom_T Abs_dom_T_inverse)
apply (simp add: Parallelf_in Rep_dom_F Abs_dom_F_inverse)

apply (simp add: Parallelt_def)
apply (elim conjE exE)
apply (rename_tac X TF1 TF2 u st s t)

apply (insert par_trace_last_only_if)
apply (drule_tac x="X" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="Tick" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (simp)

apply (elim conjE exE)
apply (simp add: dom_SF_def CT2_def)

apply (erule conjE)+
apply (drule_tac x="s'" in spec)
apply (drule_tac x="t'" in spec)
apply (simp)

apply (simp add: Parallelf_def)

   (* Evset *) 

apply (rule conjI)
apply (rule_tac x="Evset" in exI)
apply (rule_tac x="Evset" in exI)
apply (simp)
apply (rule_tac x="s'" in exI)
apply (rule_tac x="t'" in exI)
apply (simp)

   (* EvsetTick *) 

apply (rule_tac x="EvsetTick" in exI)
apply (rule_tac x="EvsetTick" in exI)
apply (simp)
apply (rule_tac x="s" in exI)
apply (rule_tac x="t" in exI)
apply (simp)
done

(****** F2 ******)

(*
   Y1  = {a. a :Y & a: insert Tick (Ev ` X) & s @t [a]t ~:t left TF1}
   Y2  = {a. a :Y & a: insert Tick (Ev ` X) & s @t [a]t ~:t left TF2}
   Y12 = Y - insert Tick (Ev ` X)

   Z1  = Y1 Un Y12
   Z2  = Y2 Un Y12
*)

(*** sync ***)

lemma Parallelsf_F2_in:
  "ALL X TF1 TF2 u Y X1 X2 s t.
       ((ALL a. a : Y -->
          u @t [a]t ~: Rep_dom_T (fst TF1) |[X]|t Rep_dom_T (fst TF2)) &
        X1 - insert Tick (Ev ` X) = X2 - insert Tick (Ev ` X) &
        u : s |[X]|set t & s :t fst TF1 & t :t fst TF2)
        --> (ALL a. a : Y & (a = Tick | a : Ev ` X)
             --> s @t [a]t ~:t (fst TF1) |
                 t @t [a]t ~:t (fst TF2) |
                 tick s | tick t)"
apply (intro allI impI)
apply (case_tac "s @t [a]t ~:t fst TF1 | t @t [a]t ~:t fst TF2 |
                 tick s | tick t")
apply (simp)

   (* s @t [a]t :t fst TF1 & t @t [a]t :t fst TF2 --> contradiction *)

apply (simp)
apply (erule conjE)+
apply (drule_tac x="a" in spec)
apply (simp)

apply (insert par_trace_last_if)
apply (drule_tac x="X" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="a" in spec)
apply (drule_tac x="s @t [a]t" in spec)
apply (drule_tac x="t @t [a]t" in spec)
apply (simp)

apply (drule mp)
apply (rule_tac x="s" in exI)
apply (rule_tac x="t" in exI)
apply (simp)

apply (simp add: Parallelt_def)
apply (drule_tac x="(s @t [a]t) |[X]|set (t @t [a]t)" in spec)
apply (rotate_tac -1)
apply (erule disjE)

apply (drule_tac x="s @t [a]t" in spec)
apply (drule_tac x="t @t [a]t" in spec)
apply (simp)
apply (simp)
done

(*** interleave ***)

lemma Parallelsf_F2_notin:
  "ALL X TF1 TF2 u Y X1 X2 s t.
       ((ALL a. a : Y -->
          u @t [a]t ~: Rep_dom_T (fst TF1) |[X]|t Rep_dom_T (fst TF2)) &
        X1 - insert Tick (Ev ` X) = X2 - insert Tick (Ev ` X) &
        u : s |[X]|set t & s :t fst TF1 & t :t fst TF2)
        --> (ALL a. a : Y & a ~= Tick & a ~: Ev ` X
             --> (s @t [a]t ~:t (fst TF1) &
                  t @t [a]t ~:t (fst TF2) ) |
                 tick s | tick t)"
apply (intro allI impI)
apply (case_tac "(s @t [a]t ~: Rep_dom_T (fst TF1) &
                  t @t [a]t ~: Rep_dom_T (fst TF2) ) |
                  tick s | tick t")
apply (erule disjE)
apply (simp)

   (* s @t [a]t :t fst TF1 | t @t [a]t :t fst TF2 --> contradiction *)

apply (simp)

apply (simp)
apply (erule conjE)+
apply (drule_tac x="a" in spec)
apply (simp)
apply (erule disjE)

      (* s @t [a]t :t fst TF1 *)

apply (insert par_trace_last_if)
apply (drule_tac x="X" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="a" in spec)
apply (drule_tac x="s @t [a]t" in spec)
apply (drule_tac x="t" in spec)
apply (simp)

apply (erule conjE)+

apply (drule mp)
apply (rule_tac x="s" in exI)
apply (simp)

apply (simp add: Parallelt_def)
apply (drule_tac x="(s @t [a]t) |[X]|set t" in spec)
apply (erule disjE)

apply (drule_tac x="s @t [a]t" in spec)
apply (drule_tac x="t" in spec)
apply (simp)
apply (simp)

      (* t @t [a]t :t fst TF2 *)

      (* insert par_trace_last_if *)
apply (drule_tac x="X" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="a" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t @t [a]t" in spec)
apply (simp)

apply (erule conjE)+

apply (rotate_tac -1)
apply (drule mp)
apply (rule_tac x="t" in exI)
apply (simp)

apply (simp add: Parallelt_def)
apply (drule_tac x="s |[X]|set (t @t [a]t)" in spec)
apply (erule disjE)

apply (drule_tac x="s" in spec)
apply (drule_tac x="t @t [a]t" in spec)
apply (simp)
apply (simp)
done

(*** Z1 ***)

lemma Parallelsf_F2_Z1:
  "ALL X TF1 TF2 u Y X1 X2 s t.
       ((ALL a. a : Y -->
          u @t [a]t ~: Rep_dom_T (fst TF1) |[X]|t Rep_dom_T (fst TF2)) &
        X1 - insert Tick (Ev ` X) = X2 - insert Tick (Ev ` X) &
        u : s |[X]|set t & s :t fst TF1 & t :t fst TF2 &
        notick s & notick t)
        --> (ALL a. a : Y & a ~= Tick & a ~: Ev ` X
             --> s @t [a]t ~:t (fst TF1))"
apply (intro allI impI)
apply (insert Parallelsf_F2_notin)
apply (drule_tac x="X" in spec)
apply (drule_tac x="TF1" in spec)
apply (drule_tac x="TF2" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="Y" in spec)
apply (drule_tac x="X1" in spec)
apply (drule_tac x="X2" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (auto)
done

(*** Z2 ***)

lemma Parallelsf_F2_Z2:
  "ALL X TF1 TF2 u Y X1 X2 s t.
       ((ALL a. a : Y -->
          u @t [a]t ~: Rep_dom_T (fst TF1) |[X]|t Rep_dom_T (fst TF2)) &
        X1 - insert Tick (Ev ` X) = X2 - insert Tick (Ev ` X) &
        u : s |[X]|set t & s :t fst TF1 & t :t fst TF2 &
        notick s & notick t)
        --> (ALL a. a : Y & a ~= Tick & a ~: Ev ` X
             --> t @t [a]t ~:t (fst TF2))"
apply (intro allI impI)
apply (insert Parallelsf_F2_notin)
apply (drule_tac x="X" in spec)
apply (drule_tac x="TF1" in spec)
apply (drule_tac x="TF2" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="Y" in spec)
apply (drule_tac x="X1" in spec)
apply (drule_tac x="X2" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (auto)
done

(*** Y = Z1 Un Z2 ***)

lemma Parallelsf_F2_Y:
  "ALL X TF1 TF2 u Y X1 X2 s t.
       ((ALL a. a : Y -->
          u @t [a]t ~: Rep_dom_T (fst TF1) |[X]|t Rep_dom_T (fst TF2)) &
        X1 - insert Tick (Ev ` X) = X2 - insert Tick (Ev ` X) &
        u : s |[X]|set t & s :t fst TF1 & t :t fst TF2 &
        notick s & notick t)
        --> X1 Un X2 Un Y =
           X1 Un
           ({a. a : Y & (a = Tick | a : Ev ` X) & s @t [a]t ~:t fst TF1} Un
            {a. a : Y & a ~= Tick & a ~: Ev ` X}) Un
           (X2 Un
            ({a. a : Y & (a = Tick | a : Ev ` X) & t @t [a]t ~:t fst TF2} Un
             {a. a : Y & a ~= Tick & a ~: Ev ` X}))"
apply (intro allI impI)
apply (rule equalityI)
apply (simp add: subset_def Ball_def)
apply (intro allI impI)

apply (case_tac "x ~= Tick & x ~: Ev ` X")
apply (simp)
apply (simp)

apply (insert Parallelsf_F2_in)
apply (drule_tac x="X" in spec)
apply (drule_tac x="TF1" in spec)
apply (drule_tac x="TF2" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="Y" in spec)
apply (drule_tac x="X1" in spec)
apply (drule_tac x="X2" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (auto)
done

(* X1 Un Z1 - insert Tick (Ev ` X) = X2 Un Z2 - insert Tick (Ev ` X) *)

lemma Parallelsf_F2_eq:
  "ALL X TF1 TF2 Y X1 X2 s t.
       (X1 - insert Tick (Ev ` X) = X2 - insert Tick (Ev ` X))
        -->
          (X1 Un
           ({a. a : Y & (a = Tick | a : Ev ` X) & s @t [a]t ~:t fst TF1} Un
            {a. a : Y & a ~= Tick & a ~: Ev ` X})) - insert Tick (Ev ` X)
          =
          (X2 Un
           ({a. a : Y & (a = Tick | a : Ev ` X) & t @t [a]t ~:t fst TF2} Un
            {a. a : Y & a ~= Tick & a ~: Ev ` X})) - insert Tick (Ev ` X)"

(* by "apply (auto)" *)

apply (intro allI impI)
apply (rule equalityI)

  (* <= *)

apply (simp add: subset_def)
apply (rule ballI)
apply (simp)

apply (erule conjE)+
apply (erule disjE)

apply (erule equalityE)
apply (unfold subset_def)
apply (drule_tac x="x" in bspec)
apply (blast)
apply (blast)

   (* | *)

apply (erule disjE)
apply (simp)
apply (simp)

  (* => *)

apply (simp)
apply (rule ballI)
apply (simp)
apply (erule conjE)+
apply (erule disjE)

apply (erule equalityE)
apply (unfold subset_def)
apply (rotate_tac -1)
apply (drule_tac x="x" in bspec)
apply (blast)
apply (blast)

   (* | *)

apply (blast)
done

(*** F2 ***)

lemma Parallelsf_in_F2 :
  "ALL X TF1 TF2. TF1:dom_SF & TF2:dom_SF --> CF2 (TF1 |[X]|sf TF2)"
apply (intro allI impI)
apply (simp add: Parallelsf_def CF2_def)
apply (intro allI impI)
apply (simp add: Parallelt_in Rep_dom_T Abs_dom_T_inverse)
apply (simp add: Parallelf_in Rep_dom_F Abs_dom_F_inverse)
apply (rename_tac X TF1 TF2 u X12 Y)

apply (simp add: Parallelf_def)
apply (elim conjE exE)
apply (rename_tac X TF1 TF2 u X12 Y X1 X2 s t)

apply (simp add: dom_SF_def CF3_def)
apply (erule conjE)+
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule mp)
apply (rule_tac x="X1" in exI)
apply (simp)
apply (drule mp)
apply (rule_tac x="X2" in exI)
apply (simp)

apply (insert contra_par_set_in_sync_event_inv)
apply (drule_tac x="X" in spec)
apply (rotate_tac -1)
apply (drule_tac x="Tick" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)

apply (rule_tac x=
        "X1 Un
         ({a. a : Y & (a = Tick | a : Ev ` X) & s @t [a]t ~:t fst TF1} Un
          {a. a : Y & a ~= Tick & a ~: Ev ` X})" in exI)
apply (rule_tac x=
        "X2 Un
         ({a. a : Y & (a = Tick | a : Ev ` X) & t @t [a]t ~:t fst TF2} Un
          {a. a : Y & a ~= Tick & a ~: Ev ` X})" in exI)
apply (rule conjI)

   (* X1 Un X2 Un Y = ... *)

apply (insert Parallelsf_F2_Y)
apply (drule_tac x="X" in spec)
apply (drule_tac x="TF1" in spec)
apply (drule_tac x="TF2" in spec)
apply (drule_tac x="u" in spec)
apply (drule_tac x="Y" in spec)
apply (drule_tac x="X1" in spec)
apply (drule_tac x="X2" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (simp)

   (* X1 Un Z1 - insert Tick (Ev ` X) = X2 Un Z2 - insert Tick (Ev ` X) *)

apply (simp  add: Parallelsf_F2_eq)

   (* F2 check *)

apply (rule_tac x="s" in exI)
apply (rule_tac x="t" in exI)
apply (simp)

apply (simp add: dom_SF_def CF2_def)
apply (drule_tac x="s" in spec)
apply (rotate_tac -1)
apply (drule_tac x="X1" in spec)
apply (rotate_tac -1)
apply (drule_tac x=
         "{a. a : Y & (a = Tick | a : Ev ` X) & s @t [a]t ~:t fst TF1} Un
          {a. a : Y & a ~= Tick & a ~: Ev ` X}" in spec)
apply (drule_tac x="t" in spec)
apply (rotate_tac -1)
apply (drule_tac x="X2" in spec)
apply (rotate_tac -1)
apply (drule_tac x=
         "{a. a : Y & (a = Tick | a : Ev ` X) & t @t [a]t ~:t fst TF2} Un
          {a. a : Y & a ~= Tick & a ~: Ev ` X}" in spec)

apply (simp)

         (* s @t [a]t ~:t (fst TF1) *)

apply (drule mp)
apply (insert Parallelsf_F2_Z1)
apply (rotate_tac -1)
apply (drule_tac x="X" in spec)
apply (drule_tac x="TF1" in spec)
apply (drule_tac x="TF2" in spec)
apply (drule_tac x="u" in spec)
apply (rotate_tac -1)
apply (drule_tac x="Y" in spec)
apply (rotate_tac -1)
apply (drule_tac x="X1" in spec)
apply (rotate_tac -1)
apply (drule_tac x="X2" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (simp)

         (* t @t [a]t ~:t fst TF2 *)

apply (drule mp)
apply (insert Parallelsf_F2_Z2)
apply (rotate_tac -1)
apply (drule_tac x="X" in spec)
apply (drule_tac x="TF1" in spec)
apply (drule_tac x="TF2" in spec)
apply (drule_tac x="u" in spec)
apply (rotate_tac -1)
apply (drule_tac x="Y" in spec)
apply (rotate_tac -1)
apply (drule_tac x="X1" in spec)
apply (rotate_tac -1)
apply (drule_tac x="X2" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (simp)

apply (simp)
done

(*** F3 ***)

lemma Parallelsf_in_F3 :
  "ALL X TF1 TF2. TF1:dom_SF & TF2:dom_SF --> CF3 (TF1 |[X]|sf TF2)"
apply (intro allI impI)
apply (simp add: Parallelsf_def CF3_def)
apply (intro allI impI)
apply (simp add: Parallelt_in Rep_dom_T Abs_dom_T_inverse)
apply (simp add: Parallelf_in Rep_dom_F Abs_dom_F_inverse)
apply (rename_tac X12 TF1 TF2 u)

apply (simp add: Parallelf_def)
apply (elim conjE exE)
apply (rename_tac X12 TF1 TF2 u X1 X2 s t)

apply (simp add: dom_SF_def CF3_def)
apply (erule conjE)+
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule mp)
apply (rule_tac x="X1" in exI)
apply (simp)
apply (drule mp)
apply (rule_tac x="X2" in exI)
apply (simp)

apply (simp add: Parallelt_def)
apply (rule_tac x="s |[X12]|set t" in exI)
apply (simp)
apply (rule_tac x="s" in exI)
apply (rule_tac x="t" in exI)
apply (simp)
done

(*** Parallelsf in dom_SF ***)

lemma Parallelsf_in :
  "ALL X TF1 TF2. TF1:dom_SF & TF2:dom_SF --> (TF1 |[X]|sf TF2): dom_SF"
apply (intro allI impI)
apply (simp (no_asm) add: dom_SF_def)
apply (simp add: Parallelsf_in_T2)
apply (simp add: Parallelsf_in_F2)
apply (simp add: Parallelsf_in_F3)
done

(*********************************************************
                     Lemmas (mono)
 *********************************************************)

(*** T ***)

lemma Parallelt_mono: 
  "ALL X S1 T1 S2 T2. S1 <= T1 & S2 <= T2 --> S1 |[X]|t S2 <= T1 |[X]|t T2"
apply (intro allI impI)
apply (simp add: Parallelt_def)
apply (simp add: subset_def)
apply (intro allI impI ballI)
apply (elim conjE exE)
apply (drule_tac x="s" in bspec)
apply (simp)
apply (drule_tac x="t" in bspec)
apply (simp)

apply (rule_tac x="s |[X]|set t" in exI)
apply (simp)
apply (rule_tac x="s" in exI)
apply (rule_tac x="t" in exI)
apply (simp)
done

(*** F ***)

lemma Parallelf_mono: 
  "ALL X E1 F1 E2 F2. E1 <= F1 & E2 <= F2 --> E1 |[X]|f E2 <= F1 |[X]|f F2"
apply (intro allI impI)
apply (simp add: Parallelf_def)
apply (simp add: subset_def)
apply (intro allI impI)
apply (elim conjE exE)
apply (drule_tac x="(s, Y)" in bspec)
apply (simp)
apply (drule_tac x="(t, Z)" in bspec)
apply (simp)
apply (rule_tac x="Y" in exI)
apply (rule_tac x="Z" in exI)
apply (simp)
apply (rule_tac x="s" in exI)
apply (rule_tac x="t" in exI)
apply (simp)
done

(*** SF ***)

lemma Parallelsf_mono: 
  "ALL X SE1 TF1 SE2 TF2. SE1 <= TF1 & SE2 <= TF2 
      --> SE1 |[X]|sf SE2 <= TF1 |[X]|sf TF2"
apply (auto simp add: Parallelsf_def)
apply (simp add: order_pair_def)
apply (simp add: subset_T_def subset_F_def)
apply (simp add: Parallelt_in Parallelf_in Rep_dom_T Rep_dom_F
                 Abs_dom_T_inverse Abs_dom_F_inverse)
apply (simp add: Parallelt_mono Parallelf_mono)
done

(*********************************************************
                   Parallel trace eval
 *********************************************************)

lemma Parallelt_eval:
  "ALL X P Q ev. [[P |[X]| Q]]T ev = [[P]]T ev |[X]|t [[Q]]T ev"
apply (intro allI)
apply (simp add: eval_T_def)
apply (simp add: eval_TF_decompo)
apply (simp add: Parallelsf_def)
apply (simp add: Parallelt_in Rep_dom_T Abs_dom_T_inverse)
done

(*********************************************************
                   Parallel failure eval
 *********************************************************)

lemma Parallelf_eval:
  "ALL X P Q ev. [[P |[X]| Q]]F ev = [[P]]F ev |[X]|f [[Q]]F ev"
apply (intro allI)
apply (simp add: eval_F_def)
apply (simp add: eval_TF_decompo)
apply (simp add: Parallelsf_def)
apply (simp add: Parallelf_in Rep_dom_F Abs_dom_F_inverse)
done

(****************** to add it again ******************)

declare disj_not1 [simp] 

end
