           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                August 2004                |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_law_dist = Csp_law_basic:

(*****************************************************************

      distribution of internal choice over ...

         1. (P1 |~| P2) [+] Q
         2. Q [+] (P1 |~| P2)
         3. (P1 |~| P2) |[X]| Q
         4. Q |[X]| (P1 |~| P2)
         5. (P1 |~| P2) -- X
         6. (P1 |~| P2) [[r]]

 *****************************************************************)

(*********************************************************
                dist law for Ext_choice (l)
 *********************************************************)

lemma csp_Ext_choice_dist_l: 
  "LET df IN (P1 |~| P2) [+] Q =F[af]
   LET df IN (P1 [+] Q) |~| (P2 [+] Q)"
apply (simp add: eqsf_iff eq_Ext_choice_dist_l)
done

(*********************************************************
                dist law for Ext_choice (r)
 *********************************************************)

lemma csp_Ext_choice_dist_r: 
  "LET df IN P [+] (Q1 |~| Q2) =F[af]
   LET df IN (P [+] Q1) |~| (P [+] Q2)"
apply (simp add: eqsf_iff eq_Ext_choice_dist_r)
done

(*********************************************************
                dist law for Parallel (l)
 *********************************************************)

lemma csp_Parallel_dist_l: 
  "LET df IN (P1 |~| P2) |[X]| Q =F[af]
   LET df IN (P1 |[X]| Q) |~| (P2 |[X]| Q)"
apply (simp add: eqsf_iff eq_Parallel_dist_l)
done

(*********************************************************
                dist law for Parallel (r)
 *********************************************************)

lemma csp_Parallel_dist_r: 
  "LET df IN P |[X]| (Q1 |~| Q2) =F[af]
   LET df IN (P |[X]| Q1) |~| (P |[X]| Q2)"
apply (simp add: eqsf_iff eq_Parallel_dist_r)
done

(*********************************************************
                dist law for Hide
 *********************************************************)

lemma csp_Hide_dist: 
  "LET df IN (P1 |~| P2) -- X =F[af]
   LET df IN (P1 -- X) |~| (P2 -- X)"
apply (simp add: eqsf_iff eq_Hide_dist)
done

(*********************************************************
                dist law for Rename
 *********************************************************)

lemma csp_Rename_dist: 
  "LET df IN (P1 |~| P2) [[r]] =F[af]
   LET df IN (P1 [[r]]) |~| (P2 [[r]])"
apply (simp add: eqsf_iff eq_Rename_dist)
done

(*********************************************************
                dist law for Sequence
 *********************************************************)

lemma csp_Sequence_dist: 
  "LET df IN (P1 |~| P2) ;; Q =F[af]
   LET df IN (P1 ;; Q) |~| (P2 ;; Q)"
apply (simp add: eqsf_iff eq_Sequence_dist)
done

(*********************************************************
                     dist laws
 *********************************************************)

lemmas csp_dist = csp_Ext_choice_dist_l csp_Ext_choice_dist_r
                      csp_Parallel_dist_l   csp_Parallel_dist_r
                      csp_Hide_dist         csp_Rename_dist
                      csp_Sequence_dist

lemmas csp_dist_rw_left_eq   = csp_dist[THEN csp_rule_rw_left_eq]
lemmas csp_dist_rw_right_eq  = csp_dist[THEN csp_rule_rw_right_eq]
lemmas csp_dist_rw_left_ref  = csp_dist[THEN csp_rule_rw_left_ref]
lemmas csp_dist_rw_right_ref = csp_dist[THEN csp_rule_rw_right_ref]

lemmas csp_dist_rw_left      = csp_dist_rw_left_eq csp_dist_rw_left_ref
lemmas csp_dist_rw_right     = csp_dist_rw_right_eq csp_dist_rw_right_ref

lemmas csp_dist_rw           = csp_dist_rw_left csp_dist_rw_right

(*****************************************************************

      distribution of internal bind over ...

         1. (! :X .. Pf) [+] Q
         2. Q [+] (! :X .. Pf)
         3. (! :X .. Pf) |[X]| Q
         4. Q |[X]| (! :X .. Pf)
         5. (! :X .. Pf) -- X
         6. (! :X .. Pf) [[r]]

 *****************************************************************)

(*********************************************************
                Dist law for Ext_choice (l)
 *********************************************************)

lemma csp_Ext_choice_Dist_l: 
  "X ~= {} ==>
     LET df IN (! :X .. Pf) [+] Q =F[af]
     LET df IN ! x:X .. (Pf x [+] Q)"
apply (simp add: eqsf_iff eq_Ext_choice_Dist_l)
done

(*********************************************************
                Dist law for Ext_choice (r)
 *********************************************************)

lemma csp_Ext_choice_Dist_r: 
  "X ~= {} ==>
     LET df IN P [+] (! :X .. Qf) =F[af]
     LET df IN ! x:X .. (P [+] Qf x)"
apply (simp add: eqsf_iff eq_Ext_choice_Dist_r)
done

(*********************************************************
                Dist law for Parallel (l)
 *********************************************************)

lemma csp_Parallel_Dist_l: 
  "Y ~= {} ==>
     LET df IN (! :Y .. Pf) |[X]| Q =F[af]
     LET df IN ! x:Y .. (Pf x |[X]| Q)"
apply (simp add: eqsf_iff eq_Parallel_Dist_l)
done

(*********************************************************
                Dist law for Parallel (r)
 *********************************************************)

lemma csp_Parallel_Dist_r: 
  "Y ~= {} ==>
     LET df IN P |[X]| (! :Y .. Qf) =F[af]
     LET df IN ! x:Y .. (P |[X]| Qf x)"
apply (simp add: eqsf_iff eq_Parallel_Dist_r)
done

(*********************************************************
                Dist law for Hide
 *********************************************************)

lemma csp_Hide_Dist: 
  "Y ~= {} ==>
     LET df IN (! :Y .. Pf) -- X =F[af]
     LET df IN ! x:Y .. (Pf x -- X)"
apply (simp add: eqsf_iff eq_Hide_Dist)
done

(*********************************************************
                Dist law for Rename
 *********************************************************)

lemma csp_Rename_Dist: 
  "X ~= {} ==>
     LET df IN (! :X .. Pf) [[r]] =F[af]
     LET df IN ! x:X .. (Pf x [[r]])"
apply (simp add: eqsf_iff eq_Rename_Dist)
done

(*********************************************************
                Dist law for Sequence
 *********************************************************)

lemma csp_Sequence_Dist: 
  "X ~= {} ==>
     LET df IN (! :X .. Pf) ;; Q =F[af]
     LET df IN ! x:X .. (Pf x ;; Q)"
apply (simp add: eqsf_iff eq_Sequence_Dist)
done

(*********************************************************
                     Dist laws
 *********************************************************)

lemmas csp_Dist = csp_Ext_choice_Dist_l csp_Ext_choice_Dist_r
                      csp_Parallel_Dist_l   csp_Parallel_Dist_r
                      csp_Hide_Dist         csp_Rename_Dist
                      csp_Sequence_Dist

lemmas csp_Dist_rw_left_eq   = csp_Dist[THEN csp_rule_rw_left_eq]
lemmas csp_Dist_rw_right_eq  = csp_Dist[THEN csp_rule_rw_right_eq]
lemmas csp_Dist_rw_left_ref  = csp_Dist[THEN csp_rule_rw_left_ref]
lemmas csp_Dist_rw_right_ref = csp_Dist[THEN csp_rule_rw_right_ref]

lemmas csp_Dist_rw_left      = csp_Dist_rw_left_eq csp_Dist_rw_left_ref
lemmas csp_Dist_rw_right     = csp_Dist_rw_right_eq csp_Dist_rw_right_ref

lemmas csp_Dist_rw           = csp_Dist_rw_left csp_Dist_rw_right


end
