           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                August 2004                |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_law_ref = Csp_law_decompo:

(*****************************************************************

         1. rules for refinement

 *****************************************************************)

(*-------------------------------------------------------*
 |                                                       |
 |     decompose internal choice (Csp-Prover rule)       |
 |                                                       |
 *-------------------------------------------------------*)

(*** or <= ***)                                         (* unsafe *)

lemma csp_Int_choice_left1:
  "LET df IN P1 <=F[af] LET df IN Q
   ==> LET df IN P1 |~| P2 <=F[af] LET df IN Q"
apply (simp add: refsf_iff ref_Int_choice_left1)
done

lemma csp_Int_choice_left2:
  "LET df IN P2 <=F[af] LET df IN Q
   ==> LET df IN P1 |~| P2 <=F[af] LET df IN Q"
apply (simp add: refsf_iff ref_Int_choice_left2)
done

lemma csp_Int_choice_left3:
  "LET df IN P1 [+] P2 <=F[af] LET df IN Q
   ==> LET df IN P1 |~| P2 <=F[af] LET df IN Q"
apply (rule csp_trans[of _ _ "LET df IN P1 [+] P2"])
apply (simp add: refsf_iff ref_Int_choice_Ext_choice)
apply (simp)
done

(*** <= and ***)                                          (* safe *)

lemma csp_Int_choice_right:
  "[| LET df IN P <=F[af] LET df IN Q1 ;
      LET df IN P <=F[af] LET df IN Q2 |]
      ==> LET df IN P <=F[af] LET df IN Q1 |~| Q2"
apply (simp add: refsf_iff ref_Int_choice_right)
done

(*-------------------------------------------------------*
 |                                                       |
 |      decompose internal bind (Csp-Prover rule)        |
 |                                                       |
 *-------------------------------------------------------*)

(*** EX <= ***)                                           (* unsafe *)

lemma csp_Int_bind_left:
  "(EX a. a:X & LET df IN Pf a <=F[af] LET df IN Q)
     ==> LET df IN ! :X .. Pf <=F[af] LET df IN Q"
apply (auto simp add: refsf_iff ref_Int_bind_left)
done

lemma csp_Int_bind_left_x:
  "[| x:X ; LET df IN Pf x <=F[af] LET df IN Q |]
     ==> LET df IN ! :X .. Pf <=F[af] LET df IN Q"
apply (auto simp add: refsf_iff ref_Int_bind_left)
done

(*** <= ALL ***)                                         (* safe *)

lemma csp_Int_bind_right:
  "[| !!a. a:X ==> LET df IN P <=F[af] LET df IN Qf a |]
        ==> LET df IN P <=F[af] LET df IN ! :X .. Qf"
apply (simp add: refsf_iff ref_Int_bind_right)
done

(*-------------------------------------------------------*
 |                                                       |
 |     decomposition with subset (Csp-Prover rule)       |
 |                                                       |
 *-------------------------------------------------------*)

(*** Int_bind ***)                                        (* unsafe *)

lemma csp_Int_bind_subset:
  "[|  Y <= X  ; !!a. a:Y ==> LET df IN Pf a <=F[af] LET df IN Qf a |]
      ==> LET df IN ! :X .. Pf <=F[af] LET df IN ! :Y .. Qf"
apply (rule csp_trans[of _ _ "LET df IN ! :Y .. Pf"])
apply (simp add: refsf_iff ref_Int_bind_subset)
apply (simp add: csp_mono)
done

(*** ! x:X .. and ? -> ***)

lemma csp_Int_bind_Pre_choice_subset:
  "[| Y ~={} ; Y <= X ; !!a. a:Y ==> LET df IN Pf a <=F[af] LET df IN Qf a |]
        ==> LET df IN ! x:X .. (x -> Pf x) <=F[af] 
            LET df IN ? :Y -> Qf"
apply (rule csp_trans[of _ _ "LET df IN ! x:Y .. (x -> Qf x)"])
apply (rule csp_Int_bind_subset, simp)
apply (simp add: csp_mono)
apply (simp add: refsf_iff ref_Int_bind_Pre_choice)
done

lemmas csp_decompo_subset = csp_Int_bind_subset 
                            csp_Int_bind_Pre_choice_subset

(*-------------------------------------------------------*
 |                                                       |
 |     decompose external choice (Csp-Prover rule)       |
 |                                                       |
 *-------------------------------------------------------*)

lemma csp_Ext_choice_right:
  "[| LET df IN P <=F[af] LET df IN Q1 ;
      LET df IN P <=F[af] LET df IN Q2 |]
      ==> LET df IN P <=F[af] LET df IN Q1 [+] Q2"
apply (rule csp_trans[of _ _ "LET df IN Q1 |~| Q2"])
apply (simp add: csp_Int_choice_right)
apply (simp add: refsf_iff ref_Int_choice_Ext_choice)
done

end
