           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                August 2004                |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_laws = Csp_law_ref  + Csp_law_SKIP + 
                  Csp_law_dist + Csp_law_fp +
                  Csp_law_step + Csp_law_IF:

(*****************************************************************

         1. Internal ... simp
         2. External ... simp
         3. Mix      ... simp
         4. Extended reflex


 *****************************************************************)



(*****************************************************************
                       Internal ... simp                         
 *****************************************************************)

(*-------------------------------------------------------*
 |                                                       |
 |                 DIV (Csp-Prover rule)                 |
 |                                                       |
 *-------------------------------------------------------*)

lemma csp_DIV_unwind:
   "LET df IN DIV =F[af] LET df IN ! x:{} .. DIV"
apply (rule csp_sym, simp add: eqsf_iff eq_Int_bind_empty)
done

(*-------------------------------------------------------*
 |                                                       |
 |       ! :{a} .. Pf =F[af] Pf a (Csp-Prover rule)          |
 |                                                       |
 *-------------------------------------------------------*)

lemma csp_Int_bind_unique:
  "LET df IN ! :{a} .. Pf =F[af] LET df IN Pf a"
apply (simp add: eqsf_iff eq_Int_bind_unique)
done

(*-------------------------------------------------------*
 |                                                       |
 |               P |~| P (Csp-Prover rule)               |
 |                Int choice idempotence                 |
 *-------------------------------------------------------*)

lemma csp_Int_choice_idem: 
  "LET df IN P =F[af] LET df IN Q ==> LET df IN P |~| Q =F[af] LET df IN P"
apply (rule csp_trans[of _ _ "LET df IN P |~| P"])
apply (rule csp_decompo)
apply (simp add: csp_reflex)
apply (rule csp_sym, simp)
apply (simp add: eqsf_iff eq_Int_choice_idem)
done

(*=======================================================*
 |                                                       |
 |              Int_simp (Csp-Prover rule)               |
 |                                                       |
 *=======================================================*)

lemmas csp_Int_simp = csp_DIV_unwind 
                      csp_Int_bind_unique 
                      csp_Int_choice_idem

lemmas csp_Int_simp_rw_left_eq   = csp_Int_simp[THEN csp_rule_rw_left_eq]
lemmas csp_Int_simp_rw_left_ref  = csp_Int_simp[THEN csp_rule_rw_left_ref]
lemmas csp_Int_simp_rw_right_eq  = csp_Int_simp[THEN csp_rule_rw_right_eq]
lemmas csp_Int_simp_rw_right_ref = csp_Int_simp[THEN csp_rule_rw_right_ref]

lemmas csp_Int_simp_rw_left      = csp_Int_simp_rw_left_eq csp_Int_simp_rw_left_ref
lemmas csp_Int_simp_rw_right     = csp_Int_simp_rw_right_eq csp_Int_simp_rw_right_ref

lemmas csp_Int_simp_rw           = csp_Int_simp_rw_left csp_Int_simp_rw_right

(*****************************************************************
                      External ... simp
 *****************************************************************)

(*-------------------------------------------------------*
 |                                                       |
 |       unit for Ext_choice (Csp-Prover rule)           |
 |                                                       |
 *-------------------------------------------------------*)

(*** unit l ***)

lemma csp_Ext_choice_unit_l: 
  "LET df IN Q =F[af] LET df IN STOP 
   ==> LET df IN Q [+] P =F[af] LET df IN P"
apply (rule csp_trans[of _ _ "LET df IN STOP [+] P"])
apply (rule csp_cong)
apply (simp)
apply (simp add: csp_reflex)

apply (rule csp_free_decompo_left)
apply (rule csp_step)
apply (rule csp_reflex)
apply (simp add: eqsf_iff eq_Ext_choice_unit_l)
done

(*** unit r ***)

lemma csp_Ext_choice_unit_r: 
  "LET df IN Q =F[af] LET df IN STOP 
   ==> LET df IN P [+] Q =F[af] LET df IN P"
apply (rule csp_trans[of _ _ "LET df IN Q [+] P"])
apply (rule csp_Ext_choice_commut)
apply (simp add: csp_Ext_choice_unit_l)
done

(*=======================================================*
 |                                                       |
 |              Ext_simp (Csp-Prover rule)               |
 |                                                       |
 *=======================================================*)

lemmas csp_Ext_simp = csp_Ext_choice_unit_l
                      csp_Ext_choice_unit_r

lemmas csp_Ext_simp_rw_left_eq   = csp_Ext_simp[THEN csp_rule_rw_left_eq]
lemmas csp_Ext_simp_rw_left_ref  = csp_Ext_simp[THEN csp_rule_rw_left_ref]
lemmas csp_Ext_simp_rw_right_eq  = csp_Ext_simp[THEN csp_rule_rw_right_eq]
lemmas csp_Ext_simp_rw_right_ref = csp_Ext_simp[THEN csp_rule_rw_right_ref]

lemmas csp_Ext_simp_rw_left      = csp_Ext_simp_rw_left_eq csp_Ext_simp_rw_left_ref
lemmas csp_Ext_simp_rw_right     = csp_Ext_simp_rw_right_eq csp_Ext_simp_rw_right_ref

lemmas csp_Ext_simp_rw           = csp_Ext_simp_rw_left csp_Ext_simp_rw_right

(*****************************************************************
                      Mix ... simp
 *****************************************************************)

(*-------------------------------------------------------*
 |                                                       |
 |         decompose Timeout (Csp-Prover rule)           |
 |                                                       |
 *-------------------------------------------------------*)

(*** <= Timeout ***)

lemma csp_Timeout_right:
  "[| LET df IN P <=F[af] LET df IN Q1 ;
      LET df IN P <=F[af] LET df IN Q2 |]
      ==> LET df IN P <=F[af] LET df IN Q1 [> Q2"
apply (simp add: csp_dist_rw)
apply (rule csp_Int_choice_right)
apply (simp add: csp_Ext_choice_right)
apply (simp add: csp_Ext_simp_rw csp_reflex)
done

(*****************************************
           STOP [> P  =  P
 *****************************************)

lemma csp_STOP_Timeout:
  "[| LET df IN P1 =F[af] LET df IN STOP ;
      LET df IN P2 =F[af] LET df IN STOP |]
   ==> LET df IN (P1 |~| P2) [+] Q =F[af] LET df IN Q"
apply (rule csp_trans[of _ _ "LET df IN STOP [+] Q"])
apply (rule csp_cong)
apply (rule csp_trans[of _ _ "LET df IN STOP |~| STOP"])
apply (rule csp_cong)
apply (simp add: csp_Int_simp_rw csp_Ext_simp_rw csp_reflex)+
done

(*=======================================================*
 |                                                       |
 |            Timeout_simp (Csp-Prover rule)             |
 |                                                       |
 *=======================================================*)

lemmas csp_Mix_simp = csp_STOP_Timeout

lemmas csp_Mix_simp_rw_left_eq   = csp_Mix_simp[THEN csp_rule_rw_left_eq]
lemmas csp_Mix_simp_rw_left_ref  = csp_Mix_simp[THEN csp_rule_rw_left_ref]
lemmas csp_Mix_simp_rw_right_eq  = csp_Mix_simp[THEN csp_rule_rw_right_eq]
lemmas csp_Mix_simp_rw_right_ref = csp_Mix_simp[THEN csp_rule_rw_right_ref]

lemmas csp_Mix_simp_rw_left      = csp_Mix_simp_rw_left_eq csp_Mix_simp_rw_left_ref
lemmas csp_Mix_simp_rw_right     = csp_Mix_simp_rw_right_eq csp_Mix_simp_rw_right_ref

lemmas csp_Mix_simp_rw           = csp_Mix_simp_rw_left csp_Mix_simp_rw_right

(*=======================================================*
 |                                                       |
 |              csp_simp (Csp-Prover rule)               |
 |                                                       |
 *=======================================================*)

lemmas csp_simp  = csp_Int_simp csp_Ext_simp csp_Mix_simp

lemmas csp_simp_rw_left  = csp_Int_simp_rw_left csp_Ext_simp_rw_left
                           csp_Mix_simp_rw_left
lemmas csp_simp_rw_right = csp_Int_simp_rw_right csp_Ext_simp_rw_right
                           csp_Mix_simp_rw_right
lemmas csp_simp_rw       = csp_simp_rw_left csp_simp_rw_right

(*****************************************************************
                Extened reflex by empty sets
 *****************************************************************)

lemma csp_Pre_choice_STOP:
   "LET df IN ? :{} -> Pf =F[af] LET df IN STOP"
apply (rule csp_rw_right[of _ _ "LET df IN ? x:{} -> DIV"])
apply (simp add: eqsf_iff eq_STOP_step)
apply (simp add: eqsf_iff eq_Pre_choice_empty)
done

lemmas csp_Pre_choice_STOP_sym = csp_Pre_choice_STOP[THEN csp_sym]

lemma csp_Int_bind_DIV:
   "LET df IN ! :{} .. Pf =F[af] LET df IN DIV"
apply (simp add: eqsf_iff eq_Int_bind_empty)
done

lemmas csp_Int_bind_DIV_sym = csp_Int_bind_DIV[THEN csp_sym]

lemma csp_Pre_choice_empty_reflex:
   "LET df IN ? :{} -> Pf =F[af] LET df IN ? :{} -> Qf"
apply (rule csp_trans[of _ _ "LET df IN STOP"])
apply (simp add: csp_Pre_choice_STOP)
apply (simp add: csp_Pre_choice_STOP_sym)
done

lemma csp_Int_bind_empty_reflex:
   "LET df IN ! :{} .. Pf =F[af] LET df IN ! :{} .. Qf"
apply (rule csp_trans[of _ _ "LET df IN DIV"])
apply (simp add: csp_Int_bind_DIV)
apply (rule csp_sym, simp add: csp_Int_bind_DIV)
done

lemmas csp_reflex_ex = csp_reflex 
                       csp_Pre_choice_STOP
                       csp_Pre_choice_STOP_sym
                       csp_Int_bind_DIV
                       csp_Int_bind_DIV_sym
                       csp_Pre_choice_empty_reflex 
                       csp_Int_bind_empty_reflex

end
