           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                July 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_proof_decompo = Csp_relation_def:

   (* note: split_paired_All: (ALL x. ?P x) = (ALL a b. ?P (a, b)) *)
   (*       split_paired_Ex : (EX x. ?P x) = (EX a b. ?P (a, b))   *)

declare split_paired_All [simp del]
declare split_paired_Ex  [simp del]

(*****************************************************************

         1. mono
         2. congruence
         3. 
         4. 

 *****************************************************************)

(*********************************************************
                        Prefix mono
 *********************************************************)

lemma ref_Prefix_mono:
  "P <=F<ev> Q ==> a -> P <=F<ev> a -> Q"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp add: Prefixsf_mono)
done

(*********************************************************
                    Prefix choice mono 
 *********************************************************)

lemma ref_Pre_choice_mono:
  "(!! a. a:X ==> Pf a <=F<ev> Qf a)
          ==> ? :X -> Pf <=F<ev> ? :X -> Qf"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp)

apply (insert Ext_choicesf_mono)
apply (drule_tac x="insert STOPsf {a ->sf [[Qf a]]TF ev |a. a : X}" in spec)
apply (drule_tac x="insert STOPsf {a ->sf [[Pf a]]TF ev |a. a : X}" in spec)
apply (simp)

apply (drule mp)
apply (rule conjI)

apply (intro allI impI)
apply (rule disjI2)
apply (elim conjE exE)
apply (rule_tac x="a ->sf [[Pf a]]TF ev" in exI)
apply (simp add: Prefixsf_mono)
apply (rule_tac x="a" in exI)
apply (simp)

apply (intro allI impI)
apply (rule disjI2)
apply (elim conjE exE)
apply (rule_tac x="a ->sf [[Qf a]]TF ev" in exI)
apply (simp add: Prefixsf_mono)
apply (rule_tac x="a" in exI)
apply (simp)
apply (simp)
done

(*********************************************************
                      Ext choice mono
 *********************************************************)

lemma ref_Ext_choice_mono:
  "[| P1 <=F<ev> Q1 ; P2 <=F<ev> Q2 |]
           ==> P1 [+] P2 <=F<ev> Q1 [+] Q2"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp add: Ext_choicesf_mono)
done

(*********************************************************
                      Int choice mono
 *********************************************************)

lemma ref_Int_choice_mono:
  "[| P1 <=F<ev> Q1 ; P2 <=F<ev> Q2 |]
          ==> P1 |~| P2 <=F<ev> Q1 |~| Q2"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp add: Int_choicesf_mono)
done

(*********************************************************
                  Internal bind mono
 *********************************************************)

lemma ref_Int_bind_mono:
  "(!! a. a:X ==> Pf a <=F<ev> Qf a)
        ==> ! :X .. Pf <=F<ev> ! :X .. Qf"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp)

apply (insert Int_choicesf_mono)
apply (drule_tac x="insert DIVsf {[[Qf a]]TF ev |a. a : X}" in spec)
apply (drule_tac x="insert DIVsf {[[Pf a]]TF ev |a. a : X}" in spec)
apply (simp)

apply (drule mp)
apply (intro allI impI)
apply (rule disjI2)
apply (elim conjE exE)
apply (rule_tac x="[[Pf a]]TF ev" in exI)
apply (simp)
apply (rule_tac x="a" in exI)
apply (simp_all)
done

(*********************************************************
                     Condition mono
 *********************************************************)

lemma ref_IF_mono:
  "[| P1 <=F<ev> Q1 ; P2 <=F<ev> Q2 |]
         ==> IF b THEN P1 ELSE P2 <=F<ev> IF b THEN Q1 ELSE Q2"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp)
done

(*********************************************************
                       Parallel mono
 *********************************************************)

lemma ref_Parallel_mono:
  "[| P1 <=F<ev> Q1 ; P2 <=F<ev> Q2 |]
         ==> P1 |[X]| P2 <=F<ev> Q1 |[X]| Q2"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp add: Parallelsf_mono)
done

(*********************************************************
                        Hide mono
 *********************************************************)

lemma ref_Hide_mono:
  "P <=F<ev> Q ==> P -- X <=F<ev> Q -- X"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp add: Hidesf_mono)
done

(*********************************************************
                        Rename mono
 *********************************************************)

lemma ref_Rename_mono:
  "P <=F<ev> Q ==> P [[r]] <=F<ev> Q [[r]]"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp add: Renamesf_mono)
done

(*********************************************************
                       Sequence mono
 *********************************************************)

lemma ref_Sequence_mono:
  "[| P1 <=F<ev> Q1 ; P2 <=F<ev> Q2 |]
         ==> P1 ;; P2 <=F<ev> Q1 ;; Q2"
apply (simp add: refsf_def)
apply (simp only: subset_SF_def eval_SF_def procsf_in Abs_dom_SF_inverse)
apply (simp add: Sequencesf_mono)
done

(*-------------------------------------------------------*
 |                                                       |
 |                      Congruence                       |
 |                                                       |
 *-------------------------------------------------------*)

(*********************************************************
                        Prefix cong
 *********************************************************)

lemma eq_Prefix_cong:
  "P =F<ev> Q ==> a -> P =F<ev> a -> Q"
apply (simp add: eqsf_refsq ref_Prefix_mono)
done

(*********************************************************
                    Prefix choice cong
 *********************************************************)

lemma eq_Pre_choice_cong:
  "(!! a. a:X ==> Pf a =F<ev> Qf a)
          ==> ? :X -> Pf =F<ev> ? :X -> Qf"
apply (simp add: eqsf_refsq ref_Pre_choice_mono)
done

(*********************************************************
                      Ext choice cong
 *********************************************************)

lemma eq_Ext_choice_cong:
  "[| P1 =F<ev> Q1 ; P2 =F<ev> Q2 |]
         ==> P1 [+] P2 =F<ev> Q1 [+] Q2"
apply (simp add: eqsf_refsq ref_Ext_choice_mono)
done

(*********************************************************
                      Int choice cong
 *********************************************************)

lemma eq_Int_choice_cong:
  "[| P1 =F<ev> Q1 ; P2 =F<ev> Q2 |]
         ==> P1 |~| P2 =F<ev> Q1 |~| Q2"
apply (simp add: eqsf_refsq ref_Int_choice_mono)
done

(*********************************************************
                   internal bind cong
 *********************************************************)

lemma eq_Int_bind_cong:
  "(!! a. a:X ==> Pf a =F<ev> Qf a) 
          ==> ! :X .. Pf =F<ev> ! :X .. Qf"
apply (simp add: eqsf_refsq ref_Int_bind_mono)
done

(*********************************************************
                     Condition cong
 *********************************************************)

lemma eq_IF_cong:
  "[| P1 =F<ev> Q1 ; P2 =F<ev> Q2 |]
         ==> IF b THEN P1 ELSE P2 =F<ev> IF b THEN Q1 ELSE Q2"
apply (simp add: eqsf_refsq ref_IF_mono)
done

(*********************************************************
                       Parallel cong
 *********************************************************)

lemma eq_Parallel_cong:
  "[| P1 =F<ev> Q1 ; P2 =F<ev> Q2 |]
         ==> P1 |[X]| P2 =F<ev> Q1 |[X]| Q2"
apply (simp add: eqsf_refsq ref_Parallel_mono)
done

(*********************************************************
                        Hide cong
 *********************************************************)

lemma eq_Hide_cong:
  "P =F<ev> Q ==> P -- X =F<ev> Q -- X"
apply (simp add: eqsf_refsq ref_Hide_mono)
done

(*********************************************************
                      Rename cong
 *********************************************************)

lemma eq_Rename_cong:
  "P =F<ev> Q ==> P [[r]] =F<ev> Q [[r]]"
apply (simp add: eqsf_refsq ref_Rename_mono)
done

(*********************************************************
                      Sequence cong
 *********************************************************)

lemma eq_Sequence_cong:
  "[| P1 =F<ev> Q1 ; P2 =F<ev> Q2 |]
         ==> P1 ;; P2 =F<ev> Q1 ;; Q2"
apply (simp add: eqsf_refsq ref_Sequence_mono)
done

(****************** to add them again ******************)

declare split_paired_All [simp add]
declare split_paired_Ex  [simp add]

end
