           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                July 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_proof_fp = Csp_relation_def + Csp_proc_rec:

(*****************************************************************

         1. fixed point theory in Csp-Prover
         2.
         3.
         4. 

 *****************************************************************)

(*******************************
            check eq
 *******************************)

consts
  CHECKeq :: "('n,'a) procRec => ('m,'a) procRec =>
                 ('n => ('m,'a) proc) => bool"
                    ("/(2CHECK /_ )/(2=F /_ )/(2BY /_)" [50,50,50] 50)

defs
  CHECKeq_def : 
    "CHECK R1 =F R2 BY f12 ==
                    (ALL C. (nohide ((LetD R1) C))) &
                    (ALL C. (guard  ((LetD R1) C))) &
                    (ALL C. (nohide ((LetD R2) C))) &
                    (ALL C. (guard  ((LetD R2) C))) &
                    (LET (LetD R2) IN (Rewrite (InP R1) By f12) 
                  =F LET (LetD R2) IN (InP R2)) &
            (ALL C. LET (LetD R2) IN (Rewrite (LetD R1 C) By f12)
                 =F LET (LetD R2) IN (f12 C))"

(*******************************
           check ref
 *******************************)

consts
  CHECKref :: "('n,'a) procRec => ('m,'a) procRec => 
                 ('n => ('m,'a) proc) => bool"
                   ("/(2CHECK /_ )/(2<=F /_ )/(2BY /_)" [50,50,50] 50)

defs
  CHECKref_def : 
    "CHECK R1 <=F R2 BY f12 ==
                    (ALL C. (nohide ((LetD R1) C))) &
                    (ALL C. (guard  ((LetD R1) C))) &
                    (ALL C. (nohide ((LetD R2) C))) &
                    (ALL C. (guard  ((LetD R2) C))) &
                    (LET (LetD R2) IN (Rewrite (InP R1) By f12)
                 <=F LET (LetD R2) IN (InP R2))&
            (ALL C. LET (LetD R2) IN (Rewrite (LetD R1 C) By f12)
                <=F LET (LetD R2) IN (f12 C))"

(*******************************
           check ref
 *******************************)

consts
  CHECKrev :: "('m,'a) procRec => ('n,'a) procRec => 
                 ('m => ('n,'a) proc) => bool"
                   ("/(2CHECK /_ )/(2=>F /_ )/(2BY /_)" [50,50,50] 50)

defs
  CHECKrev_def : 
    "CHECK R1 =>F R2 BY f12 ==
                    (ALL C. (nohide ((LetD R1) C))) &
                    (ALL C. (guard  ((LetD R1) C))) &
                    (ALL C. (nohide ((LetD R2) C))) &
                    (ALL C. (guard  ((LetD R2) C))) &
                    (LET (LetD R2) IN (InP R2)
                 <=F LET (LetD R2) IN (Rewrite (InP R1) By f12)) &
            (ALL C. LET (LetD R2) IN (f12 C)
                <=F LET (LetD R2) IN (Rewrite (LetD R1 C) By f12))"

(*********************************************************
                        Lemmas
 *********************************************************)

lemma eval_SF_eval_procDef:
   "[[df C]]SF (ev) = ([[df]]DF (ev)) C"
apply (simp add: eval_procDef_def)
done

(*********************************************************
                  Fixpoint (unwind name)
 *********************************************************)

lemma eq_unwind:
  "[| !! C. nohide (df C) ; !! C. guard  (df C) |]
         ==> LET df IN <C> =F[af] LET df IN df C"
apply (simp add: eqsf_rec_def)
apply (insert eval_SF_eval_procDef[of df C "UFP [[df]]DF"])
apply (simp add: eval_SF_def)
apply (simp add: Rep_dom_SF_inverse)
apply (simp add: Banach_FPdom_SF_fixpoint)
done

(*********************************************************
                 Fixpoint (equaity check)
 *********************************************************)

lemma eq_fp_induct:
  "CHECK (LET df1 IN P1) =F (LET df2 IN P2) BY f12 
       ==> LET df1 IN P1 =F LET df2 IN P2"
apply (unfold CHECKeq_def)
apply (simp add: CHECKeq_def)
apply (erule conjE)+
apply (simp add: eqsf_rec_def)
apply (simp add: compo_proc)
apply (simp add: eval_SF_eval_procDef)

apply (insert Banach_dom_SF)
apply (drule_tac x="df1" in spec)
apply (simp)
apply (elim conjE exE)

apply (drule_tac x="[[f12]]DF (UFP [[df2]]DF)" in spec)
apply (drule mp)
apply (simp add: expand_fun_eq)

apply (insert Banach_FPdom_SF_unique)
apply (drule_tac x="df1" in spec)
apply (simp)
done

(*********************************************************
                 Fixpoint (refinment check)
                    f12 : df1 --> df2
 *********************************************************)

lemma ref_fp_induct:
  "CHECK (LET df1 IN P1) <=F (LET df2 IN P2) BY f12
       ==> LET df1 IN P1 <=F LET df2 IN P2"
apply (simp add: CHECKref_def)
apply (erule conjE)+
apply (simp add: refsf_rec_def)
apply (simp add: compo_proc)
apply (simp add: eval_SF_eval_procDef)

apply (insert order_prod_inv)
apply (drule_tac x="[[f12]]DF (UFP [[df2]]DF)" in spec)
apply (drule_tac x="[[df1]]DF ([[f12]]DF (UFP [[df2]]DF))" in spec)
apply (simp)

apply (insert metric_fixpoint_induction_ref)
apply (drule_tac x="[[df1]]DF" in spec)
apply (drule_tac x="[[f12]]DF (UFP [[df2]]DF)" in spec)
apply (drule_tac x="UFP [[df1]]DF" in spec)

apply (simp add: mono_procDef_fun)
apply (insert contract_procDef)
apply (drule_tac x="df1" in spec)
apply (simp add: contra_half_to_contst)
apply (simp add: Banach_FPdom_SF_fixpoint)

apply (insert procsf_mono)
apply (drule_tac x="[[f12]]DF (UFP [[df2]]DF)" in spec)
apply (drule_tac x="UFP [[df1]]DF" in spec)
apply (drule_tac x="P1" in spec)
apply (simp)
done

(*********************************************************
             Fixpoint (reverse refinment check)
                    f12 : df1 --> df2
 *********************************************************)

lemma rev_fp_induct:
  "CHECK (LET df1 IN P1) =>F (LET df2 IN P2) BY f12
       ==> LET df2 IN P2 <=F LET df1 IN P1"
apply (simp add: CHECKrev_def)
apply (erule conjE)+
apply (simp add: refsf_rec_def)
apply (simp add: compo_proc)
apply (simp add: eval_SF_eval_procDef)

apply (insert order_prod_inv)
apply (drule_tac x="[[df1]]DF ([[f12]]DF (UFP [[df2]]DF))" in spec)
apply (drule_tac x="[[f12]]DF (UFP [[df2]]DF)" in spec)
apply (simp)

apply (insert metric_fixpoint_induction_rev)
apply (drule_tac x="[[df1]]DF" in spec)
apply (drule_tac x="[[f12]]DF (UFP [[df2]]DF)" in spec)
apply (drule_tac x="UFP [[df1]]DF" in spec)

apply (simp add: mono_procDef_fun)
apply (insert contract_procDef)
apply (drule_tac x="df1" in spec)
apply (simp add: contra_half_to_contst)
apply (simp add: Banach_FPdom_SF_fixpoint)

apply (insert procsf_mono)
apply (drule_tac x="UFP [[df1]]DF" in spec)
apply (drule_tac x="[[f12]]DF (UFP [[df2]]DF)" in spec)
apply (drule_tac x="P1" in spec)
apply (simp)
done

end
