           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                August 2004                |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_relation_def = Csp_proc:

(*****************************************************************

         1. def  =F
         2. def <=F
         3. decomposion SF --> T and F
         4. 

 *****************************************************************)

consts
  refsf :: "('n,'a) proc => ('n,'a) dom_SF_prod
            => ('n,'a) proc => bool"
                                  ("(0_ /<=F<_> /_)" [50,0,50] 50)

  eqsf  :: "('n,'a) proc => ('n,'a) dom_SF_prod
            => ('n,'a) proc => bool"
                                  ("(0_ /=F<_> /_)" [50,0,50] 50)

defs
  refsf_def : "P <=F<ev> Q == [[Q]]SF ev <= [[P]]SF ev"
  eqsf_def  : "P  =F<ev> Q == [[P]]SF ev  = [[Q]]SF ev"


consts
  refsf_rec :: "('n,'a) procRec => bool => ('m,'a) procRec => bool"
                                            ("(0_ /<=F[_] /_)" [50,0,50] 50)
  eqsf_rec  :: "('n,'a) procRec => bool => ('m,'a) procRec => bool"
                                            ("(0_ /=F[_] /_)" [50,0,50] 50)

defs
  refsf_rec_def : "R1 <=F[af] R2 == [[R2]]RC <= [[R1]]RC"
  eqsf_rec_def  : "R1  =F[af] R2 == [[R1]]RC  = [[R2]]RC"

syntax
  "_eqsf_rec_inact"   :: "('n,'a) procRec => ('m,'a) procRec => bool"
                                            ("(0_ /=F /_)" [50,50] 50)
  "_eqsf_rec_act"     :: "('n,'a) procRec => ('m,'a) procRec => bool"
                                            ("(0_ /=F:active /_)" [50,50] 50)
  "_refsf_rec_inact"  :: "('n,'a) procRec => ('m,'a) procRec => bool"
                                            ("(0_ /<=F /_)" [50,50] 50)
  "_refsf_rec_act"    :: "('n,'a) procRec => ('m,'a) procRec => bool"
                                            ("(0_ /<=F:active /_)" [50,50] 50)

translations
  "R1 =F R2"         == "R1 =F[False] R2"
  "R1 =F:active R2"  == "R1 =F[True] R2"
  "R1 <=F R2"        == "R1 <=F[False] R2"
  "R1 <=F:active R2" == "R1 <=F[True] R2"

(*======================================================================*

   What is "af" in "=F[af]"?

   "af" is an active flag for controlling automatic proofs in tacticals and
   it do not affect CSP-theory at all.

   For instance, if "af" is True, then expressions can be decomposed,
   else they cannot. It avoid infinite decomposition.

 *=======================================================================*)

(*-------------------------------------------------------*
 |                                                       |
 |                  Basic properties                     |
 |                                                       |
 *-------------------------------------------------------*)

(*********************************************************
                    decomposition
 *********************************************************)

(*** eqsf ***)

lemma eqsf_decompo:
  "(P =F<ev> Q) 
         = (([[P]]T ev = [[Q]]T ev) & ([[P]]F ev = [[Q]]F ev))"
apply (simp add: eqsf_def)
apply (simp add: eval_SF_def)
apply (simp add: procsf_in Abs_dom_SF_inject)
apply (simp add: eval_TF_decompo)
apply (simp add: eval_T_in Abs_dom_T_inject)
apply (simp add: eval_F_in Abs_dom_F_inject)
done

(*** refsf ***)

lemma refsf_decompo:
  "(P <=F<ev> Q) 
         = (([[Q]]T ev <= [[P]]T ev) & ([[Q]]F ev <= [[P]]F ev))"
apply (simp add: refsf_def)
apply (simp add: eval_SF_def)
apply (simp add: subset_SF_def)
apply (simp add: procsf_in Abs_dom_SF_inverse)
apply (simp add: eval_TF_decompo)
apply (simp add: order_pair_def)
apply (simp add: subset_T_def eval_T_in Abs_dom_T_inverse)
apply (simp add: subset_F_def eval_F_in Abs_dom_F_inverse)
done

(*********************************************************
                      eq <==> ref
 *********************************************************)

lemma eqsf_refsq:
  "(P =F<ev> Q) = (P <=F<ev> Q & Q <=F<ev> P)"
apply (simp add: refsf_def eqsf_def)
apply (auto intro: order_antisym)
done

(*********************************************************
                     Reflexivity
 *********************************************************)

lemma eqsf_reflex: "P =F<ev> P"
apply (simp add: eqsf_def)
done

lemma refsf_reflex: "P <=F<ev> P"
apply (simp add: refsf_def)
done

(*********************************************************
                    Transitivity
 *********************************************************)

lemma eqsf_trans: "[| P1 =F<ev> P2 ; P2 =F<ev> P3 |] ==> P1 =F<ev> P3"
apply (simp add: eqsf_def)
done

lemma refsf_trans: "[| P1 <=F<ev> P2 ; P2 <=F<ev> P3 |] ==> P1 <=F<ev> P3"
apply (simp add: refsf_def)
done

(*********************************************************
                         symmetry
 *********************************************************)

lemma eqsf_sym:
  "P =F<ev> Q ==> Q =F<ev> P"
apply (simp add: eqsf_def)
done

(*********************************************************
                      relation iff
 *********************************************************)

(*** eq ***)

lemma eqsf_iff:
  "(LET df IN P =F[af] LET df IN Q) = (P =F<UFP [[df]]DF> Q)"
apply (simp add: eqsf_rec_def)
apply (simp add: eqsf_def)
done

(*** ref ***)

lemma refsf_iff:
  "(LET df IN P <=F[af] LET df IN Q) = (P <=F<UFP [[df]]DF> Q)"
apply (simp add: refsf_rec_def)
apply (simp add: refsf_def)
done

end
