           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                July 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_semantics = Csp_syntax + Domain_SF_prod_cpo
                     + Trace_par + Trace_hide + Trace_ren + Trace_seq:

(*****************************************************************

         1. 
         2. 
         3. 
         4. 

 *****************************************************************)

(*** SKIP ***)

consts 
  SKIPt  :: "'a trace set"
  SKIPf  :: "'a failure set"
  SKIPsf :: "'a dom_T_F"

defs
  SKIPt_def  : "SKIPt  == {[]t, [Tick]t}"
  SKIPf_def  : "SKIPf  == {([]t, X) |X. X <= Evset} Un
                          {([Tick]t, X) |X. X <= EvsetTick}"
  SKIPsf_def : "SKIPsf == ((Abs_dom_T SKIPt), (Abs_dom_F SKIPf))"

(* "X <= EvsetTick" is True *)

(*** STOP ***)

consts 
  STOPt  :: "'a trace set"
  STOPf  :: "'a failure set"
  STOPsf :: "'a dom_T_F"

defs
  STOPt_def  : "STOPt  == {[]t}"
  STOPf_def  : "STOPf  == {([]t, X) |X. (X <= EvsetTick)}"
  STOPsf_def : "STOPsf == ((Abs_dom_T STOPt), (Abs_dom_F STOPf))"

(*** DIV ***)

consts 
  DIVsf :: "'a dom_T_F"

defs
  DIVsf_def : "DIVsf == ((Abs_dom_T STOPt), (Abs_dom_F {}))"

(*** Prefix ***)

consts 
  Prefixt  :: "'a => 'a trace set => 'a trace set"    ("(_ ->t _)" [90,80] 80)
  Prefixf  :: "'a => 'a failure set => 'a failure set"("(_ ->f _)" [90,80] 80)
  Prefixsf :: "'a => 'a dom_T_F => 'a dom_T_F"        ("(_ ->sf _)" [90,80] 80)

defs
  Prefixt_def  : "a ->t T   == insert []t {[Ev a]t @t s |s. s: T}"
  Prefixf_def  : "a ->f F   == {([]t,X) |X. Ev a ~:X} Un
                               {([Ev a]t @t s,X) |s X. (s,X): F}"
  Prefixsf_def : "a ->sf TF == (Abs_dom_T (a ->t Rep_dom_T (fst TF)),
                                Abs_dom_F (a ->f Rep_dom_F (snd TF)))"

(*** External Choice ***)

consts 
  Ext_choicef  :: "'a trace set set => 'a failure set set => 'a failure set"
                                                     ("EXTCHf _ _" [72,72] 72) 
  Ext_choicesf :: "'a dom_T_F set => 'a dom_T_F"     ("EXTCHsf _" [72] 72) 

defs
  Ext_choicef_def  : 
    "EXTCHf Ts Fs == {([]t,X) |X. ([]t,X) : Inter Fs} Un
                     {(s,X) |s X. s ~= []t & (s,X) : Union Fs} Un
                     {([]t,X) |X. X <= Evset & [Tick]t : Union Ts}"

  Ext_choicesf_def : 
    "EXTCHsf TFs == (Abs_dom_T (Union  (Rep_dom_T ` fst ` TFs)),
                     Abs_dom_F (EXTCHf (Rep_dom_T ` fst ` TFs)
                                       (Rep_dom_F ` snd ` TFs)))"

(*** Internal Choice ***)

consts 
  Int_choicesf :: "'a dom_T_F set => 'a dom_T_F"     ("INTCHsf _" [68] 68) 

defs
  Int_choicesf_def : 
    "INTCHsf TFs == (Abs_dom_T (Union (Rep_dom_T ` fst ` TFs)),
                     Abs_dom_F (Union (Rep_dom_F ` snd ` TFs)))"

(*** Parallel ***)

consts 
  Parallelt  :: "'a trace set => 'a set => 'a trace set 
                  => 'a trace set"              ("_ |[_]|t _" [76,0,77] 76)
  Parallelf  :: "'a failure set => 'a set => 'a failure set
                  => 'a failure set"            ("_ |[_]|f _" [76,0,77] 76)
  Parallelsf :: "'a dom_T_F => 'a set => 'a dom_T_F
                  => 'a dom_T_F"                ("_ |[_]|sf _" [76,0,77] 76)

defs
  Parallelt_def : 
    "S |[X]|t T == Union {s |[X]|set t |s t. s : S & t : T}"
  Parallelf_def : 
    "E |[X]|f F == {(u, Y Un Z) 
                      |u Y Z. Y-((Ev ` X) Un {Tick})= Z-((Ev ` X) Un {Tick}) &
                     (EX s t. u: s |[X]|set t & (s,Y):E & (t,Z):F)}"
  Parallelsf_def : 
    "SE |[X]|sf TF == 
           (Abs_dom_T (Rep_dom_T (fst SE) |[X]|t Rep_dom_T (fst TF)),
            Abs_dom_F (Rep_dom_F (snd SE) |[X]|f Rep_dom_F (snd TF)))"

(*** Hide ***)

consts 
  Hidet  :: "'a trace set => 'a set => 'a trace set"     ("_ --t _" [84,85] 84)
  Hidef  :: "'a failure set => 'a set => 'a failure set" ("_ --f _" [84,85] 84)
  Hidesf :: "'a dom_T_F => 'a set => 'a dom_T_F"         ("_ --sf _"  [84,85] 84)

defs
  Hidet_def : "T --t X == {t --tr X |t. t : T}"
  Hidef_def : "F --f X == {(t --tr X,Y) |t Y. (t,(Ev ` X) Un Y) : F}"
  Hidesf_def: "TF --sf X == (Abs_dom_T (Rep_dom_T (fst TF) --t X),
                               Abs_dom_F (Rep_dom_F (snd TF) --f X))"

(*** Rename ***)

consts 
  Renamet  :: "'a trace set => ('a*'b) set => 'b trace set"     ("_ [[_]]t" [84,0] 84)
  Renamef  :: "'a failure set => ('a*'b) set => 'b failure set" ("_ [[_]]f" [84,0] 84)
  Renamesf :: "'a dom_T_F => ('a*'b) set => 'b dom_T_F"         ("_ [[_]]sf"[84,0] 84)

defs
  Renamet_def : "T [[r]]t == {t. EX s:T. ren_check (s,r,t)}"
  Renamef_def : "F [[r]]f == {(t,X)|t X. EX s. ren_check (s,r,t) &
                                              (s, <invEv r> X) : F}"
  Renamesf_def: "TF [[r]]sf == (Abs_dom_T (Rep_dom_T (fst TF) [[r]]t),
                                Abs_dom_F (Rep_dom_F (snd TF) [[r]]f))"

(*** Sequence ***)

consts 
  Sequencet :: "'a trace set => 'a trace set => 'a trace set"  ("_ ;;t _" [79,78] 78)
  Sequencef :: "('a trace set * 'a failure set) => 'a failure set =>'a failure set"
                                                              ("_ ;;f _" [79,78] 78)
  Sequencesf:: "'a dom_T_F => 'a dom_T_F => 'a dom_T_F"       ("_ ;;sf _" [79,78] 78)

defs
  Sequencet_def : "S ;;t T == {rmtick s|s. s:S} Un
                              {s @t t |s t. notick s & s @t [Tick]t : S & t : T}"
  Sequencef_def : 
    "SE ;;f F == {(s,X) |s X. notick s & (s, X Un {Tick}) : snd SE} Un
                 {(s @t t,X) |s t X. notick s & s @t [Tick]t : fst SE & (t,X) : F}"
defs
  Sequencesf_def: 
    "SE ;;sf TF == (Abs_dom_T (Rep_dom_T (fst SE) ;;t Rep_dom_T (fst TF)),
                    Abs_dom_F ((Rep_dom_T (fst SE), Rep_dom_F (snd SE))
                               ;;f Rep_dom_F (snd TF)))"

(*********************************************************
                        process
 *********************************************************)

consts
  eval_TF :: "('n,'a) proc => ('n,'a) dom_SF_prod => 'a dom_T_F" ("[[_]]TF")

primrec
  "[[STOP]]TF         = (%ev. STOPsf)"
  "[[SKIP]]TF         = (%ev. SKIPsf)"
  "[[DIV]]TF          = (%ev. DIVsf)"
  "[[a -> P]]TF       = (%ev. (a ->sf ([[P]]TF ev)))"
  "[[? :X -> Pf]]TF   = (%ev. EXTCHsf ({a ->sf [[Pf a]]TF ev |a. a:X} Un {STOPsf}))"
  "[[P [+] Q]]TF      = (%ev. EXTCHsf {[[P]]TF ev, [[Q]]TF ev})"
  "[[P |~| Q]]TF      = (%ev. INTCHsf {[[P]]TF ev, [[Q]]TF ev})"
  "[[! :X .. Pf]]TF   = (%ev. INTCHsf ({ [[Pf a]]TF ev |a. a:X} Un {DIVsf}))"
  "[[IF b THEN P ELSE Q]]TF = (%ev. (if b then ([[P]]TF ev) else ([[Q]]TF ev)))"
  "[[P |[X]| Q]]TF    = (%ev. (([[P]]TF ev) |[X]|sf ([[Q]]TF ev)))"
  "[[P -- X]]TF       = (%ev. ([[P]]TF ev) --sf X)"
  "[[P [[r]]]]TF      = (%ev. ([[P]]TF ev) [[r]]sf)"
  "[[P ;; Q]]TF       = (%ev. (([[P]]TF ev) ;;sf ([[Q]]TF ev)))"
  "[[<C>]]TF          = (%ev. Rep_dom_SF (ev C))"

consts
  eval_SF :: "('n,'a) proc => ('n,'a) dom_SF_prod => 'a dom_SF"  ("[[_]]SF")
  eval_T  :: "('n,'a) proc => ('n,'a) dom_SF_prod => 'a trace set" ("[[_]]T")
  eval_F  :: "('n,'a) proc => ('n,'a) dom_SF_prod => 'a failure set" ("[[_]]F")

defs
  eval_SF_def : "[[P]]SF == (%ev. Abs_dom_SF ([[P]]TF ev))"
  eval_T_def  : "[[P]]T  == (%ev. Rep_dom_T  (fst ([[P]]TF ev)))"
  eval_F_def  : "[[P]]F  == (%ev. Rep_dom_F  (snd ([[P]]TF ev)))"

consts
  Rep_Tof :: "'a dom_SF => 'a trace set"
  Rep_Fof :: "'a dom_SF => 'a failure set"

defs
  Rep_Tof_def  : "Rep_Tof SF == Rep_dom_T (fst (Rep_dom_SF SF))"
  Rep_Fof_def  : "Rep_Fof SF == Rep_dom_F (snd (Rep_dom_SF SF))"

(*********************************************************
             process definition & recursion
 *********************************************************)

consts
  eval_procDef :: 
    "('n => ('m,'a) proc) => ('m,'a) dom_SF_prod 
                          => ('n,'a) dom_SF_prod"        ("[[_]]DF")
defs
  eval_procDef_def :  "[[f]]DF == (%ev. (%C. [[f C]]SF ev))"

consts
  eval_procRec ::
    "('n,'a) procRec => 'a dom_SF"                   ("[[_]]RC")

primrec
  "[[LET df IN P]]RC = [[P]]SF (UFP [[df]]DF)"

(*********************************************************
                    eval_T and eval_F
 *********************************************************)

lemma eval_T_in: "ALL P ev. [[P]]T ev : dom_T"
apply (simp add: eval_T_def Rep_dom_T)
done

lemma eval_F_in: "ALL P ev. [[P]]F ev : dom_F"
apply (simp add: eval_F_def Rep_dom_F)
done

(*********************************************************
                    decomposition
 *********************************************************)

lemma eval_TF_decompo:
  "ALL P ev. [[P]]TF ev = ((Abs_dom_T ([[P]]T ev)), (Abs_dom_F ([[P]]F ev)))"
apply (intro allI)
apply (simp add: eval_T_def eval_F_def)
apply (simp add: Rep_dom_T_inverse Rep_dom_F_inverse)
done

lemma eval_SF_decompo:
  "ALL P ev. [[P]]SF ev =
      Abs_dom_SF ((Abs_dom_T ([[P]]T ev)), (Abs_dom_F ([[P]]F ev)))"
apply (simp add: eval_TF_decompo eval_SF_def)
done

end
