           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                July 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Csp_syntax = Complex_Main:

(*-----------------------------------------------------------*
 |                                                           |
 |    Process Type Definitions                               |
 |                                                           |
 |          ('n,'a) proc : type of process expressions       |
 |                       'n : process name                   |
 |                       'a : event                          |
 |                                                           |
 *-----------------------------------------------------------*)

(*********************************************************************
                       process expression
 *********************************************************************)

datatype 
 ('n,'a) proc
    = STOP

    | SKIP

    | DIV

    | Prefix      "'a" "('n,'a) proc"   ("(1_ /-> _)" [150,80] 80)

    | Pre_choice  "'a set" "'a => ('n,'a) proc"
                                               ("(1? :_ /-> _)" [90,80] 80)
    | Ext_choice  "('n,'a) proc" "('n,'a) proc"  
                                               ("(1_ /[+] _)" [72,73] 72)
    | Int_choice  "('n,'a) proc" "('n,'a) proc"  
                                               ("(1_ /|~| _)" [64,65] 64)
    | Int_bind    "'a set" "'a => ('n,'a) proc"
                                               ("(1! :_ .. /_)" [0,68] 68) 
    | Conditon    "bool" "('n,'a) proc" "('n,'a) proc"
                                 ("(0IF _ /THEN _ /ELSE _)" [900,60,60] 88)
    | Parallel    "('n,'a) proc" "'a set" "('n,'a) proc"  
                                           ("(1_ /|[_]| _)" [76,0,77] 76)
    | Hiding      "('n,'a) proc" "'a set"  ("(1_ /-- _)" [84,85] 84)

    | Rename      "('n,'a) proc" "('a * 'a) set"  ("(1_ /[[_]])" [84,0] 84)

    | Sequence    "('n,'a) proc" "('n,'a) proc"  
                                               ("(1_ /;; _)" [79,78] 78)
    | Name        "'n"                     ("<_>" [0] 90)

(*--------------------------------------------------------------------*
 |                                                                    |
 | (1) binding power:                                                 |
 |      Name > Conditional > {Hiding, Rename} >                       |
 |      {Prefix, Prefix choice} > Sequence > Parallel >               |
 |      External choice > Internal bind >  Internal choice            |
 |                                                                    |
 |   The binding power 90 is higher than ones of processes ops.       |
 |                                                                    |
 | (2) DIV is a short notation as follows: DIV = ! {} .. (%x. STOP)   |
 |                                                                    |
 | (3) About ("(1_ /-> _)" [150,80] 80)                               |
 |      The nth operator ! has the power 101.                         |
 |      So, 150 > 101 is needed.                                      |
 |                                                                    |
 *--------------------------------------------------------------------*)

(*** external prefix ***)

syntax
  "@Ext_prefix"  :: "pttrn => 'a set => ('n,'a) proc 
                => ('n,'a) proc"  ("(1? _:_ /-> _)" [900,900,80] 80)

translations
  "? x:X -> P"  == "? :X -> (%x. P)"

(*** internal prefix ***)

consts
  Int_prefix :: "'a set => ('a => ('n,'a) proc) => ('n,'a) proc"  
                                      ("(1! :_ /-> _)" [900,80] 80)

defs
  Int_prefix_def : "! :X -> Pf == ! :X .. (%x. x -> (Pf x))"

syntax
  "@Int_prefix"  :: "pttrn => 'a set => ('n,'a) proc 
                => ('n,'a) proc"  ("(1! _:_ /-> _)" [900,900,80] 80)

translations
  "! x:X -> P"  == "! :X -> (%x. P)"

(*** sending and receiving prefixes ***)

consts
  S_Prefix :: "('x => 'a) => 'x => ('n,'a) proc => ('n,'a) proc" 
                                     ("(1_ ! _ /-> _)" [900,1000,80] 80)
  I_Prefix :: "('x => 'a) => 'x set => 
               ('x => ('n,'a) proc) => ('n,'a) proc" 
  R_Prefix :: "('x => 'a) => 'x set => 
               ('x => ('n,'a) proc) => ('n,'a) proc" 

defs
  S_Prefix_def : "a ! x -> P    == a x -> P"
  I_Prefix_def : "I_Prefix f X Pf == ! :(f ` X) -> (%x. (Pf ((inv f) x)))"
  R_Prefix_def : "R_Prefix f X Pf == ? :(f ` X) -> (%x. (Pf ((inv f) x)))"

syntax
  "@I_Prefix" :: "('x => 'a) => pttrn => 'x set => 
                  ('n,'a) proc => ('n,'a) proc"
                               ("(1_ !! _:_ /-> _)" [900,900,1000,80] 80)

  "@IU_Prefix" :: "('x => 'a) => pttrn => ('n,'a) proc => ('n,'a) proc"
                               ("(1_ !! _ /-> _)" [900,900,80] 80)

  "@R_Prefix" :: "('x => 'a) => pttrn => 'x set => 
                  ('n,'a) proc => ('n,'a) proc"
                               ("(1_ ? _:_ /-> _)" [900,900,1000,80] 80)
  "@RU_Prefix" :: "('x => 'a) => pttrn => ('n,'a) proc => ('n,'a) proc"
                               ("(1_ ? _ /-> _)" [900,900,80] 80)

translations
  "a !! x:X -> P" == "I_Prefix a X (%x. P)"
  "a !! x -> P"   == "a !! x:UNIV -> P"

  "a ? x:X -> P"  == "R_Prefix a X (%x. P)"
  "a ? x -> P"    == "a? x:UNIV -> P"

(*** internal choice ***)

consts
  R_Int_bind :: "('x => 'a) => 'x set => 
                 ('x => ('n,'a) proc) => ('n,'a) proc" 
defs
  R_Int_bind_def : 
    "R_Int_bind f X Pf == ! :(f ` X) .. (%x. (Pf ((inv f) x)))"

syntax
  "@Int_bind"    :: "pttrn => 'a set => ('n,'a) proc 
                => ('n,'a) proc"  ("(1! _:_ .. /_)" [900,900,68] 68)
  "@R_Int_bind"  :: "('x => 'a) => pttrn => 'x set => 
                      ('n,'a) proc => ('n,'a) proc"
                               ("(1_ !! _:_ .. /_)" [900,900,1000,80] 80)
  "@RU_Int_bind" :: "('x => 'a) => pttrn => 
                       ('n,'a) proc => ('n,'a) proc"
                               ("(1_ !! _ .. /_)" [900,900,80] 80)

translations
  "! x:X .. P"    == "! :X .. (%x. P)"
  "a !! x:X .. P" == "R_Int_bind a X (%x. P)"
  "a !! x .. P"   == "a !! x:UNIV .. P"

(*** parallel ***)

syntax
  "_Interleave"    :: "('n,'a) proc => ('n,'a) proc
                    => ('n,'a) proc"  ("(1_ ||| _)" [76,77] 76)
  "_Synchro"       :: "('n,'a) proc => ('n,'a) proc
                    => ('n,'a) proc"  ("(1_ || _)" [76,77] 76)
  "_alpha_Parallel":: "('n,'a) proc => 'a set => 'a set => ('n,'a) proc
                    => ('n,'a) proc"  ("(1_ |[_,_]| _)" [76,0,0,77] 76)
translations
  "P ||| Q"     == "P |[{}]| Q"
  "P || Q"      == "P |[UNIV]| Q"
  "P |[X,Y]| Q" == "P |[X Int Y]| Q"

(*** timeout ***)

syntax
  "_Timeout"  :: "('n,'a) proc => ('n,'a) proc 
               => ('n,'a) proc"  ("(1_ /[> _)" [73,74] 73)
translations
  "P [> Q"      == "(P |~| STOP) [+] Q"

(*********************************************************************
                       recursive process
 *********************************************************************)

types ('n,'a) procDef = "'n => ('n,'a) proc"           (* synonym *)

datatype 
 ('n,'a) procRec
    = Letin "('n,'a) procDef" "('n,'a) proc"
          ("//(2LET /_ )/(2IN /_)/" [53,53] 53)

(*********************************************************************
                    name and proc from Letin
 *********************************************************************)

consts
  LetD :: "('n,'a) procRec => ('n,'a) procDef"
  InP  :: "('n,'a) procRec => ('n,'a) proc"

primrec "LetD (LET df IN P) = df"
primrec "InP  (LET df IN P) = P"

(****************** lemma *****************)

lemma LetD_InP: "ALL R. LET LetD R IN InP R = R"
apply (intro allI)
apply (induct_tac R)
apply (simp)
done

  (*================================================*
   |   Short notation for non-recursive processes   |
   *================================================*)

datatype NoRec = NoUse

consts 
  NoDef :: "(NoRec, 'a) procDef"

primrec
  "NoDef  (NoUse) = DIV"

syntax
  "_LETIN"  :: "(NoRec,'a) proc => (NoRec,'a) procRec"
            ("(2LET-IN /_)" [53] 53)

translations
  "LET-IN P" == "LET NoDef IN P"

(*************************************************************
                   Syntactical functions
 *************************************************************)

(*** guarded (proc) ***)

consts
  guard :: "('n,'a) proc => bool"

primrec
  "guard STOP                 = True"
  "guard SKIP                 = True"
  "guard DIV                  = True"
  "guard (a -> P)             = True"
  "guard (? :X -> Pf)         = True"
  "guard (P [+] Q)            = (guard P & guard Q)"
  "guard (P |~| Q)            = (guard P & guard Q)"
  "guard (! :X .. Pf)         = (ALL x. guard (Pf x))"
  "guard (IF b THEN P ELSE Q) = (guard P & guard Q)"
  "guard (P |[X]| Q)          = (guard P & guard Q)"
  "guard (P -- X)             = guard P"
  "guard (P [[r]])            = guard P"
  "guard (P ;; Q)             = (guard P & guard Q)"
  "guard (<C>)                = False"

(*** nohide (proc) ***)

consts
  nohide :: "('n,'a) proc => bool"

primrec
  "nohide STOP                 = True"
  "nohide SKIP                 = True"
  "nohide DIV                  = True"
  "nohide (a -> P)             = nohide P"
  "nohide (? :X -> Pf)         = (ALL x. nohide (Pf x))"
  "nohide (P [+] Q)            = (nohide P & nohide Q)"
  "nohide (P |~| Q)            = (nohide P & nohide Q)"
  "nohide (! :X .. Pf)         = (ALL x. nohide (Pf x))"
  "nohide (IF b THEN P ELSE Q) = (nohide P & nohide Q)"
  "nohide (P |[X]| Q)          = (nohide P & nohide Q)"
  "nohide (P -- X)             = False"
  "nohide (P [[r]])            = nohide P"
  "nohide (P ;; Q)             = (nohide P & nohide Q)"
  "nohide (<C>)                = True"

(*** extending function over processes from process-names ***)

consts
  rewrite_by :: "('n,'a) proc => ('n => ('m,'a) proc) => ('m,'a) proc"
                 ("(Rewrite _ By _)" [1000,1000] 100)

primrec
  "Rewrite STOP         By f = STOP"
  "Rewrite SKIP         By f = SKIP"
  "Rewrite DIV          By f = DIV"
  "Rewrite (a -> P)     By f = a -> (Rewrite P By f)"
  "Rewrite (? :X -> Pf) By f = ? :X -> (%x. (Rewrite (Pf x) By f))"
  "Rewrite (P [+] Q)    By f = (Rewrite P By f) [+] (Rewrite Q By f)"
  "Rewrite (P |~| Q)    By f = (Rewrite P By f) |~| (Rewrite Q By f)"
  "Rewrite (! :X .. Pf) By f = ! :X .. (%x. (Rewrite (Pf x) By f))"
  "Rewrite (IF b THEN P ELSE Q) By f
          = IF b THEN (Rewrite P By f) ELSE (Rewrite Q By f)"
  "Rewrite (P |[X]| Q)  By f = (Rewrite P By f) |[X]| (Rewrite Q By f)"
  "Rewrite (P -- X)     By f = (Rewrite P By f) -- X"
  "Rewrite (P [[r]])    By f = (Rewrite P By f) [[r]]"
  "Rewrite (P ;; Q)     By f = (Rewrite P By f) ;; (Rewrite Q By f)"
  "Rewrite (<C>)        By f = f C"

(*********************************************************
             termination relation for proc
 *********************************************************)

consts
  procterm :: "(('n,'a) proc * ('n,'a) proc) set"

inductive procterm
intros
  "(P, a -> P)             : procterm"
  "(Pf a, ? :X -> Pf)      : procterm"
  "(P, P [+] Q)            : procterm"
  "(Q, P [+] Q)            : procterm"
  "(P, P |~| Q)            : procterm"
  "(Q, P |~| Q)            : procterm"
  "(Pf a, ! :X .. Pf)      : procterm"
  "(P, IF b THEN P ELSE Q) : procterm"
  "(Q, IF b THEN P ELSE Q) : procterm"
  "(P, P |[X]| Q)          : procterm"
  "(Q, P |[X]| Q)          : procterm"
  "(P, P -- X)             : procterm"
  "(P, P [[r]])            : procterm"
  "(P, P ;; Q)              : procterm"
  "(Q, P ;; Q)              : procterm"

lemma wf_procterm: "wf procterm"
  apply (unfold wf_def)
  apply (intro strip)
  apply (induct_tac x)

  apply (erule allE, erule mp, intro strip,
         erule procterm.elims,
         simp, simp, simp, simp, simp, simp,
         simp, simp, simp, simp, simp, simp,
         simp, simp, simp)+
done

(****************************************************************
                 procterm Test
   The following guardrec is easily defined by primrec.
   This is an example for showing how to use procterm for recdef.
   An advantage for using recdef is that matching can be used.

 ****************************************************************)
(*
consts
  guardrec :: "('n,'a) proc => bool"

recdef guardrec "procterm"
  "guardrec STOP                 = True"
  "guardrec SKIP                 = True"
  "guardrec DIV                  = True"
  "guardrec (a -> P)             = True"
  "guardrec (? :X -> Pf)         = True"
  "guardrec (P [+] Q)            = (guardrec P & guardrec Q)"
  "guardrec (P |~| Q)            = (guardrec P & guardrec Q)"
  "guardrec (! :X .. Pf)         = (ALL x. guardrec (Pf x))"
  "guardrec (IF b THEN P ELSE Q) = (guardrec P & guardrec Q)"
  "guardrec (P |[X]| Q)          = (guardrec P & guardrec Q)"
  "guardrec (P -- X)             = guardrec P"
  "guardrec (P [[r]])            = guardrec P"
  "guardrec (P ;; Q)             = (guardrec P & guardrec Q)"
  "guardrec (<C>)                = False"
  (hints recdef_wf add: wf_procterm intro: procterm.intros)
*)
(*********************************************************
                       Guared Test
 *********************************************************)
(*
datatype abcType = a | b |c

datatype testprocNameType = A nat | B nat | C

consts
  testSYSdef :: "(testprocNameType, abcType) procDef"

primrec
  "testSYSdef  (A n) = a -> <B (n+1)>"
  "testSYSdef  (B n) = b -> <A (n+1)> [+] c -> <C>"
  "testSYSdef    (C) = (b -> SKIP) ;; a -> <A 0>"

lemma "!! C. guardrec(testSYSdef C)"
apply (induct_tac C)
apply (simp_all)
done
*)

end
