           (*-------------------------------------------*
            |                Csp-Prover                 |
            |               October 2004                |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Domain_F_cpo = Domain_F + CPO:

(*****************************************************************

         1. Domain_F is a pointed cpo.
         2. 
         3. 
         4. 

 *****************************************************************)

(*  The following simplification rules are deleted in this theory file *)
(*  because they unexpectly rewrite UnionF and InterF.                 *)
(*                  Union (B ` A) = (UN x:A. B x)                      *)
(*                  Inter (B ` A) = (INT x:A. B x)                     *)

declare Union_image_eq [simp del]
declare Inter_image_eq [simp del]

(*********************************************************
                      Bottom in Dom_F
 *********************************************************)

instance dom_F :: (type) bot0
apply (intro_classes)
done

defs (overloaded)
  bottom_F_def   :  "Bot == {}f"

lemma bottom_F : "ALL (F::'a dom_F). Bot <= F"
apply (intro allI)
apply (simp add: bottom_F_def)
apply (simp add: subset_F_def)
apply (simp add: emptyset_in_dom_F Abs_dom_F_inverse)
done

instance dom_F :: (type) bot
apply (intro_classes)
apply (simp add: bottom_F)
done

(********************************************************** 
      lemmas used in a proof that domain_F is a cpo.
 **********************************************************)

(* UnionF Fs is an upper bound of Fs *)

lemma Union_F_is_ub : "ALL Fs. Fs <| UnionF Fs"
apply (intro allI)
apply (simp add: is_ub_def)
apply (intro allI, intro impI)
apply (simp add: subset_F_def)
apply (simp add: dom_F_Union_in_dom_F Abs_dom_F_inverse)
apply (rule subsetI)
apply (simp add: UNION_def)
apply (rule_tac x=y in bexI)
apply (simp)
apply (simp)
done

(* UnionF Fs is the least upper bound of Fs *)

lemma Union_F_lub : "ALL Fs. Fs <<| UnionF Fs"
apply (intro allI impI)
apply (simp add: is_lub_def Union_F_is_ub)
apply (simp add: is_ub_def subset_F_def)
apply (simp add: dom_F_Union_in_dom_F Abs_dom_F_inverse)
apply (intro allI impI)

apply (rule subsetI)
apply (simp add: UNION_def)
apply (erule bexE)
apply (drule_tac x="xa" in spec)
apply (auto)
done

(* the least upper bound of Fs is UnionF Fs *)

lemma lub_Union_F : "ALL Fs F. Fs <<| F --> F = UnionF Fs"
apply (intro allI impI)
apply (insert Union_F_lub)
apply (drule_tac x="Fs" in spec)
apply (insert lub_unique)
apply (drule_tac x="Fs" in spec)
apply (drule_tac x="F" in spec)
apply (drule_tac x="UnionF Fs" in spec)
apply (simp)
done

(* rewrite *)

lemma lub_to_Union_F : "ALL Fs F. (Fs <<| F) = (F = UnionF Fs)"
apply (intro allI impI)
apply (rule iffI)
apply (simp add: lub_Union_F)
apply (simp add: Union_F_lub)
done

(* LUB is UnionF Fs *)

lemma LUB_Union_F : "ALL Fs. LUB Fs = UnionF Fs"
apply (intro allI impI)
apply (simp add: Union_F_lub LUB_lub_imp)
done

(********************************************************** 
                 ( dom_F, <= ) is a CPO
 **********************************************************)

instance dom_F :: (type) cpo
apply (intro_classes)
apply (rule_tac x="UnionF X" in exI)
apply (simp add: Union_F_lub)
done

(********************************************************** 
              ( dom_F, <= ) is a pointed CPO
 **********************************************************)

instance dom_F :: (type) cpo_bot
apply (intro_classes)
done

(***********************************************************
                          lemmas
 ***********************************************************)


(****************** to add them again ******************)

declare Union_image_eq [simp]
declare Inter_image_eq [simp]

end


