           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                June 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Domain_SF = Domain_T + Domain_F + RS_pair:

(*****************************************************************

         1. 
         2. 
         3. 
         4. 

 *****************************************************************)

(***********************************************************
                 type def (Stable Failure)
 ***********************************************************)

types 'a dom_T_F = "'a dom_T * 'a dom_F"       (* synonym *)

consts
  CT2    :: "'a dom_T_F => bool" 
  CF2    :: "'a dom_T_F => bool" 
  CF3    :: "'a dom_T_F => bool" 

defs
  CT2_def : 
    "CT2 TF == ALL s. s @t [Tick]t :t (fst TF) & notick s
                --> (s, Evset) :f (snd TF) & 
                    (s @t [Tick]t, EvsetTick) :f (snd TF)"

  CF2_def : 
    "CF2 TF == ALL s X Y. (s, X) :f (snd TF) & notick s &
                          (ALL a. a : Y --> s @t [a]t ~:t fst TF)
                --> (s, X Un Y) :f (snd TF)"

  CF3_def : 
    "CF3 TF == ALL s X. (s, X) :f (snd TF)
                --> s :t (fst TF)"


(*** BOT in SF ***)

lemma BOT_SF: "CT2({[]t}t , {}f) & CF2({[]t}t , {}f) & CF3({[]t}t , {}f)"
apply (rule conjI)

apply (simp add: CT2_def)
apply (intro allI impI)
apply (simp add: nil_t_set_in Abs_dom_T_inverse)

apply (erule conjE)
apply (simp add: appt_to_app_or_sym)
apply (simp add: trace_rep_nil_t)
apply (simp add: tick_in_trace Abs_trace_inverse)

apply (simp add: CF2_def CF3_def)
apply (simp add: emptyset_in_dom_F Abs_dom_F_inverse)
done

(*******************************
           Type SF
 *******************************)

typedef 'a dom_SF = "{SF::('a dom_T_F). CT2(SF) & CF2(SF) & CF3(SF)}"
apply (rule_tac x ="({[]t}t , {}f)" in exI)
apply (simp add: BOT_SF)
done

(*******************************
         Rep lemma
 *******************************)

lemma Rep_dom_SF_inject_inv:
  "ALL (SF::'a dom_SF) (SE::'a dom_SF).
      (SF = SE) = (Rep_dom_SF SF = Rep_dom_SF SE)"
apply (simp add: Rep_dom_SF_inject)
done

(*********************************************************
          The relation (<=) is defined over dom_SF
 *********************************************************)

instance dom_SF :: (type) ord 
apply (intro_classes)
done

defs (overloaded)
  subset_SF_def   :  "SF <= SE  ==  (Rep_dom_SF SF) <= (Rep_dom_SF SE)"
  psubset_SF_def  :  "SF <  SE  ==  (Rep_dom_SF SF) <  (Rep_dom_SF SE)"

(*********************************************************
          The relation (<=) is a partial order
 *********************************************************)

instance dom_SF :: (type) order
apply (intro_classes)
apply (unfold subset_SF_def psubset_SF_def)
apply (simp)
apply (erule order_trans, simp)
apply (drule order_antisym, simp)
apply (simp add: Rep_dom_SF_inject)
apply (simp only: order_less_le Rep_dom_SF_inject)
done

(***********************************************************
                          lemmas
 ***********************************************************)

(*******************************
              basic
 *******************************)

(*** T2 ***)

lemma dom_TF_T2:
  "ALL TF s. TF: dom_SF & s @t [Tick]t :t fst TF & notick s 
       --> (s, Evset) :f snd TF & (s @t [Tick]t, EvsetTick) :f snd TF"
apply (simp add: dom_SF_def CT2_def)
done

(*** F2 ***)

lemma dom_TF_F2:
    "ALL TF s X Y. TF: dom_SF & (s, X) :f (snd TF) & notick s &
                   (ALL a. a : Y --> s @t [a]t ~:t fst TF)
               --> (s, X Un Y) :f (snd TF)"
apply (simp add: dom_SF_def CF2_def)
done

(*** F3 ***)

lemma dom_TF_F3:
    "ALL TF s X. TF: dom_SF & (s, X) :f (snd TF)
                --> s :t (fst TF)"
apply (simp add: dom_SF_def CF3_def)
done

(*******************************
         check in dom_SF
 *******************************)

(*** ({[]t}t, {}f) ***)

lemma BOT_in_dom_SF: "({[]t}t, {}f) : dom_SF"
apply (simp add: dom_SF_def BOT_SF)
done

(*******************************
       BOT is the bottom
 *******************************)

lemma BOT_is_bottom_dom_SF: 
   "ALL SF. ({[]t}t , {}f) <= SF"
apply (intro allI)
apply (simp add: order_pair_def)

apply (simp add: subset_T_def subset_F_def)
apply (simp add: nil_t_set_in Abs_dom_T_inverse)
apply (simp add: emptyset_in_dom_F Abs_dom_F_inverse)
apply (simp add: nil_t_in_all_dom Rep_dom_T)
done

(*********************************************************
                 Restriction in Dom_F
 *********************************************************)

instance dom_SF :: (type) ms0
apply (intro_classes)
done

consts
 restSFpair  :: "'a dom_SF => nat => 'a dom_T_F" ("_ restSFpair _" [55,56] 55)
 limitSFpair :: "'a dom_SF infinite_seq => 'a dom_T_F"

defs
  restSFpair_def  : 
   "SF restSFpair n == (Rep_dom_SF SF) rest n"
  limitSFpair_def  : 
   "limitSFpair SFs == pair_limit (%n. Rep_dom_SF (SFs n))"

defs (overloaded)
  rest_SF_def : "SF rest n == Abs_dom_SF (SF restSFpair n)"

syntax
  "_limitSF"  :: "'a dom_SF infinite_seq => 'a dom_SF"   ("limitSF")

translations
  "limitSF SFs" == "Abs_dom_SF (limitSFpair SFs)"

(***********************************************************
                   operators on dom_SF
 ***********************************************************)

consts
  Tof_fun  :: "('x => 'a dom_SF) => ('x => 'a dom_T)"
  Fof_fun  :: "('x => 'a dom_SF) => ('x => 'a dom_F)"


defs
  Tof_fun_def  : "Tof_fun SFf == fst_fun (%n. (Rep_dom_SF (SFf n)))"
  Fof_fun_def  : "Fof_fun SFf == snd_fun (%n. (Rep_dom_SF (SFf n)))"

(*********************************************************
              Lemmas (Restriction in Dom_SF)
 *********************************************************)

(*** restSFpair_def (T2) ***)

lemma restSFpair_def_T2 : 
  "ALL SF n. CT2 ((Rep_dom_SF SF) rest n)"
apply (intro allI)
apply (simp add: CT2_def)
apply (intro allI impI)
apply (simp add: pair_restriction_def)

apply (simp add: rest_T_def restTset_in Abs_dom_T_inverse)
apply (simp add: rest_F_def restFset_in Abs_dom_F_inverse)

apply (simp add: restTset_def)

apply (erule conjE)+
apply (rule conjI)

apply (simp add: restFset_def)
apply (insert dom_TF_T2)
apply (drule_tac x="Rep_dom_SF SF" in spec)
apply (drule_tac x="s" in spec)
apply (simp add: Rep_dom_SF)
apply (simp add: trace_app_in_trace_if Abs_trace_inverse)
apply (simp add: tick_in_trace Abs_trace_inverse)

apply (simp add: restFset_def)
apply (drule_tac x="fst (Rep_dom_SF SF)" in spec)
apply (drule_tac x="snd (Rep_dom_SF SF)" in spec)
apply (drule_tac x="s" in spec)
apply (auto simp add: Rep_dom_SF)
done

(*** restSFpair_def (F2) ***)

lemma restSFpair_def_F2 : 
  "ALL SF n. CF2 ((Rep_dom_SF SF) rest n)"
apply (intro allI)
apply (simp add: CF2_def)
apply (intro allI impI)
apply (simp add: pair_restriction_def)

apply (simp add: rest_T_def restTset_in Abs_dom_T_inverse)
apply (simp add: rest_F_def restFset_in Abs_dom_F_inverse)

apply (simp add: restFset_def)

apply (erule conjE)+
apply (erule disjE)

apply (insert dom_TF_F2)
apply (drule_tac x="Rep_dom_SF SF" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Y" in spec)
apply (drule mp)

apply (simp add: Rep_dom_SF)
apply (intro allI impI)
apply (drule_tac x="a" in spec)
apply (simp)

apply (simp add: restTset_def del: disj_not1)
apply (erule disjE)
apply (simp)

apply (simp add: trace_app_in_trace_if Abs_trace_inverse)
apply (simp add: event_in_trace Abs_trace_inverse)
apply (simp)

(* *)

apply (erule conjE)
apply (erule exE)
apply (simp)
apply (simp add: trace_app_in_trace_if Abs_trace_inverse)
apply (simp add: tick_in_trace Abs_trace_inverse)
done

(*** restSFpair_def (F3) ***)

lemma restSFpair_def_F3 : 
  "ALL SF n. CF3 ((Rep_dom_SF SF) rest n)"
apply (intro allI)
apply (simp add: CF3_def)
apply (intro allI impI)
apply (simp add: pair_restriction_def)

apply (simp add: rest_T_def restTset_in Abs_dom_T_inverse)
apply (simp add: rest_F_def restFset_in Abs_dom_F_inverse)

apply (simp add: restFset_def)
apply (simp add: restTset_def)

apply (erule conjE)+
apply (erule exE)

apply (insert dom_TF_F3)
apply (drule_tac x="Rep_dom_SF SF" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="X" in spec)
apply (auto simp add: Rep_dom_SF)
done

(*** restSFpair_def in dom_SF ***)

lemma restSFpair_def_in : 
  "ALL SF n. (Rep_dom_SF SF) rest n : dom_SF"
apply (intro allI)
apply (simp add: dom_SF_def)
apply (simp add: restSFpair_def_T2)
apply (simp add: restSFpair_def_F2)
apply (simp add: restSFpair_def_F3)
done

(*** restSFpair in dom_SF ***)

lemma restSFpair_in : 
  "ALL SF n. SF restSFpair n : dom_SF"
apply (simp add: restSFpair_def restSFpair_def_in)
done

(*********************************************************
                     Dom_SF --> RS
 *********************************************************)

(*** zero_eq_rs_dom_SF ***)

lemma zero_eq_rs_dom_SF: "ALL (SF::'a dom_SF) SE. SF rest 0 = SE rest 0"
apply (simp add: rest_SF_def)
apply (simp add: restSFpair_in Abs_dom_SF_inject)
apply (simp add: restSFpair_def)
apply (simp add: zero_eq_rs)
done

(*** min_rs_dom_SF ***)

lemma min_rs_dom_SF:
  "ALL (SF::'a dom_SF) (m::nat) (n::nat). 
     (SF rest m) rest n = SF rest (min m n)"
apply (simp add: rest_SF_def)
apply (simp add: restSFpair_in Abs_dom_SF_inject)
apply (simp add: restSFpair_def)
apply (simp add: restSFpair_def_in Abs_dom_SF_inverse)
apply (simp add: min_rs)
done

(*** diff_rs_dom_SF ***)

lemma diff_rs_dom_SF: 
  "ALL (SF::'a dom_SF) (SE::'a dom_SF).
    SF ~= SE --> (EX (n::nat). SF rest n ~= SE rest n)"
apply (simp add: rest_SF_def)
apply (simp add: restSFpair_in Abs_dom_SF_inject)
apply (simp add: restSFpair_def)
apply (simp add: Rep_dom_SF_inject_inv)
apply (simp add: diff_rs)
done

(***************************************************************
                        dom_SF ==> RS
 ***************************************************************)

instance dom_SF :: (type) rs
apply (intro_classes)
apply (simp add: zero_eq_rs_dom_SF)
apply (simp add: min_rs_dom_SF)
apply (simp add: diff_rs_dom_SF)
done

(************************************************************
                        dom_SF ==> MS
 ************************************************************)

instance dom_SF :: (type) ms
apply (intro_classes)
apply (simp add: positive_rs)
apply (simp add: diagonal_rs)
apply (simp add: symmetry_rs)
apply (simp add: triangle_inequality_rs)
done

(************************************************************
                 i.e.  dom_SF ==> MS & RS 
 ************************************************************)

instance dom_SF :: (type) ms_rs
apply (intro_classes)
done


(***********************************************************
                    lemmas (distance)
 ***********************************************************)

(*** restriction ***)

lemma rest_rep_dom_SF:
  "ALL (SF::'a dom_SF) (SE::'a dom_SF) n.
     (SF rest n = SE rest n) = (Rep_dom_SF SF rest n = Rep_dom_SF SE rest n)"
apply (simp add: rest_SF_def restSFpair_in Abs_dom_SF_inject)
apply (simp add: restSFpair_def)
done

(*** distance ***)

lemma distance_dom_SF:
  "ALL (SF::'a dom_SF) (SE::'a dom_SF).
      distance(SF, SE) = distance(Rep_dom_SF SF, Rep_dom_SF SE)"
apply (simp add: rest_rep_dom_SF distance_diff_type)
done

(*** normal ***)

lemma normal_dom_SF:
  "ALL (SFs::'a dom_SF infinite_seq). 
       normal SFs = normal (%n. (Rep_dom_SF (SFs n)))"
apply (simp add: normal_def distance_dom_SF)
done

(* T and F *)

lemma normal_of_dom_SF:
  "ALL (SFs::'a dom_SF infinite_seq). 
       normal SFs --> (normal (Tof_fun SFs) & normal (Fof_fun SFs))"
apply (simp add: normal_dom_SF)
apply (simp add: Tof_fun_def Fof_fun_def)
apply (simp add: pair_normal_seq)
done

(*** cauchy ***)

lemma cauchy_dom_SF:
  "ALL (SFs::'a dom_SF infinite_seq). 
       cauchy SFs = cauchy (%n. Rep_dom_SF (SFs n))"
apply (simp add: cauchy_def distance_dom_SF)
done


(* T and F *)

lemma cauchy_of_dom_SF:
  "ALL (SFs::'a dom_SF infinite_seq). 
       cauchy SFs --> (cauchy (Tof_fun SFs) & cauchy (Fof_fun SFs))"
apply (simp add: cauchy_dom_SF)
apply (simp add: Tof_fun_def Fof_fun_def)
apply (simp add: pair_cauchy_seq)
done


(***********************************************************
                      lemmas (limit)
 ***********************************************************)

(*** convergeTo dom_SF ***)

lemma convergeTo_dom_SF:
  "ALL (SFs::'a dom_SF infinite_seq) TF.
       (%n. Rep_dom_SF (SFs n)) convergeTo TF & TF : dom_SF
            --> SFs convergeTo Abs_dom_SF TF"
apply (intro allI impI)
apply (erule conjE)
apply (simp add: convergeTo_def)
apply (intro allI impI)
apply (drule_tac x="eps" in spec)
apply (simp)
apply (erule exE)
apply (rule_tac x="n" in exI)
apply (intro allI impI)
apply (drule_tac x="m" in spec)

apply (simp add: distance_dom_SF)
apply (simp add: Abs_dom_SF_inverse)
done

(*** limitSFpair ***)

lemma limitSFpair_iff:
  "ALL (SFs::'a dom_SF infinite_seq). normal (%n. Rep_dom_SF (SFs n)) 
      --> pair_limit (%n. Rep_dom_SF (SFs n)) 
             = (limitT (Tof_fun SFs) , limitF (Fof_fun SFs))"
apply (simp add: pair_limit_def)
apply (simp add: Tof_fun_def Fof_fun_def)
apply (simp add: pair_normal_seq limitT_limit_eq limitF_limit_eq)
done

(*******************************
      limitSFpair in dom_F
 *******************************)

(*** T2 ***)

lemma limitSFpair_T2:
  "ALL (SFs::'a dom_SF infinite_seq). normal SFs -->
      CT2 (limitT (Tof_fun SFs) , limitF (Fof_fun SFs))"
apply (intro allI impI)
apply (simp add: CT2_def)
apply (intro allI impI)

apply (simp add: normal_of_dom_SF limitTset_in Abs_dom_T_inverse)
apply (simp add: normal_of_dom_SF limitFset_in Abs_dom_F_inverse)

apply (simp add: limitTset_def)
apply (simp add: Tof_fun_def fst_fun_def)
 
apply (insert dom_TF_T2)
apply (drule_tac x="Rep_dom_SF (SFs (lengtht (s @t [Tick]t)))" in spec)
apply (drule_tac x="s" in spec)
apply (simp add: Rep_dom_SF)

apply (simp add: limitFset_def)
apply (simp add: Fof_fun_def snd_fun_def)
apply (simp add: length_app_event_Suc)
apply (auto)
done

(*** F2 ***)

lemma limitSFpair_F2:
  "ALL (SFs::'a dom_SF infinite_seq). normal SFs -->
      CF2 (limitT (Tof_fun SFs) , limitF (Fof_fun SFs))"
apply (intro allI impI)
apply (simp add: CF2_def)
apply (intro allI impI)

apply (simp add: normal_of_dom_SF limitTset_in Abs_dom_T_inverse)
apply (simp add: normal_of_dom_SF limitFset_in Abs_dom_F_inverse)

apply (simp add: limitFset_def)
apply (erule conjE)+
apply (erule disjE)

apply (insert dom_TF_F2)
apply (drule_tac x="Rep_dom_SF (SFs (Suc (lengtht s)))" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Y" in spec)
apply (drule mp)

apply (simp add: Rep_dom_SF)
apply (simp add: Fof_fun_def snd_fun_def)

apply (intro allI impI)
apply (drule_tac x="a" in spec)
apply (simp)

apply (simp add: limitTset_def)
apply (simp add: length_app_event_Suc)
apply (simp add: Tof_fun_def fst_fun_def)

apply (simp add: Fof_fun_def snd_fun_def)

(* *)

apply (erule conjE)
apply (erule exE)
apply (simp add: trace_app_in_trace_if Abs_trace_inverse)
apply (simp add: tick_in_trace Abs_trace_inverse)
done

(*** F3 ***)

lemma limitSFpair_F3:
  "ALL (SFs::'a dom_SF infinite_seq). normal SFs -->
      CF3 (limitT (Tof_fun SFs) , limitF (Fof_fun SFs))"
apply (intro allI impI)
apply (simp add: CF3_def)
apply (intro allI impI)

apply (simp add: normal_of_dom_SF limitTset_in Abs_dom_T_inverse)
apply (simp add: normal_of_dom_SF limitFset_in Abs_dom_F_inverse)

apply (simp add: limitFset_def)
apply (erule exE)
apply (erule disjE)

apply (insert dom_TF_F3)
apply (drule_tac x="Rep_dom_SF (SFs (Suc (lengtht s)))" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="X" in spec)

apply (simp add: Rep_dom_SF)
apply (simp add: Fof_fun_def snd_fun_def)

apply (simp add: limitTset_def)

apply (insert normal_seq_dom_T)
apply (drule_tac x="Tof_fun SFs" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="Suc (lengtht s)" in spec)
apply (simp add: normal_of_dom_SF)
apply (simp add: Tof_fun_def fst_fun_def)

(* *)

apply (drule_tac x="Rep_dom_SF (SFs (lengtht s))" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="X" in spec)
apply (simp add: Rep_dom_SF Fof_fun_def snd_fun_def)
apply (simp add: limitTset_def)
apply (simp add: Tof_fun_def fst_fun_def)
done

(*** limitSFpari_in ***)

lemma limitSFpair_in:
  "ALL (SFs::'a dom_SF infinite_seq). normal SFs -->
      (limitT (Tof_fun SFs) , limitF (Fof_fun SFs)) : dom_SF"
apply (simp add: dom_SF_def)
apply (simp add: limitSFpair_T2)
apply (simp add: limitSFpair_F2)
apply (simp add: limitSFpair_F3)
done

(*** (normal) SFs converges to (limitSF SFs) ***)

lemma limitSF_limit:
  "ALL (SFs::'a dom_SF infinite_seq). 
     normal SFs --> SFs convergeTo (limitSF SFs)"
apply (intro allI impI)

apply (insert pair_cms_cauchy_limit)
apply (drule_tac x="(%n. Rep_dom_SF (SFs n))" in spec)
apply (simp add: normal_dom_SF normal_cauchy)

apply (insert convergeTo_dom_SF)
apply (drule_tac x="SFs" in spec)
apply (drule_tac x="pair_limit (%n. Rep_dom_SF (SFs n))" in spec)
apply (simp add: limitSFpair_def normal_dom_SF 
                 limitSFpair_iff limitSFpair_in)
done

(*** (cauchy) Fs converges to (limitSF NF SFs) ***)

lemma cauchy_limitSF_limit:
  "ALL (SFs::'a dom_SF infinite_seq). 
      cauchy SFs --> SFs convergeTo (limitSF (NF SFs))"
apply (simp add: limitSF_limit normal_form_seq_same_limit 
                 normal_form_seq_normal)
done

(**************************************
    Dom_SF --> Complete Metric Space
 **************************************)

lemma dom_SF_cms:
  "ALL (SFs::'a dom_SF infinite_seq). 
      cauchy SFs --> (EX SF. SFs convergeTo SF)"
apply (intro allI impI)
apply (rule_tac x="limitSF (NF SFs)" in exI)
apply (simp add: cauchy_limitSF_limit)
done

(************************************************************
                   dom_SF ==> CMS and RS
 ************************************************************)

instance dom_SF :: (type) cms
apply (intro_classes)
apply (simp add: dom_SF_cms)
done

instance dom_SF :: (type) cms_rs
apply (intro_classes)
done

(*** (normal) limit SFs = limitSF SFs ***)

lemma limitSF_limit_eq:
  "ALL (SFs::'a dom_SF infinite_seq). normal SFs --> limit SFs = limitSF SFs"
apply (intro allI impI)
apply (insert unique_convergence)
apply (drule_tac x="SFs" in spec)
apply (drule_tac x="limit SFs" in spec)
apply (drule_tac x="limitSF SFs" in spec)
apply (simp add: limitSF_limit the_limit normal_cauchy)
done

(*----------------------------------------------------------*
 |                                                          |
 |         Fixed point induction for refinement             |
 |                                                          |
 *----------------------------------------------------------*)

consts
  Ref_SF :: "'a dom_SF => 'a dom_SF => bool"
  Rev_SF :: "'a dom_SF => 'a dom_SF => bool"

defs
  Ref_SF_def : "Ref_SF SE == (%SF. (SE <= SF))"
  Rev_SF_def : "Rev_SF SE == (%SF. (SF <= SE))"

(************************************************************
             continuity lemma for refinement
 ************************************************************)

lemma Ref_SF_cont: "ALL SE. continuous (Ref_SF SE)"
apply (intro allI)
apply (simp add: Ref_SF_def subset_SF_def)
apply (simp add: order_pair_def)

apply (insert pair_continuous)
apply (drule_tac x="Ref_T (fst (Rep_dom_SF SE))" in spec)
apply (drule_tac x="Ref_F (snd (Rep_dom_SF SE))" in spec)
apply (simp add: Ref_T_cont Ref_F_cont)
apply (simp add: Ref_T_def Ref_F_def)

apply (simp add: continuous_def)
apply (intro allI impI)
apply (drule_tac x="fst (Rep_dom_SF x)" in spec)
apply (drule_tac x="snd (Rep_dom_SF x)" in spec)
apply (simp)

apply (elim conjE exE)
apply (rule_tac x="n" in exI)
apply (intro allI impI)
apply (drule_tac x="fst (Rep_dom_SF y)" in spec)
apply (drule_tac x="snd (Rep_dom_SF y)" in spec)

apply (simp add: rest_SF_def)
apply (simp add: restSFpair_in Abs_dom_SF_inject)
apply (simp add: restSFpair_def)
done

(************************************************************
         continuity lemma for (Reverse) refinement
 ************************************************************)

lemma Rev_SF_cont: "ALL SE. continuous (Rev_SF SE)"
apply (intro allI)
apply (simp add: Rev_SF_def subset_SF_def)
apply (simp add: order_pair_def)

apply (insert pair_continuous)
apply (drule_tac x="Rev_T (fst (Rep_dom_SF SE))" in spec)
apply (drule_tac x="Rev_F (snd (Rep_dom_SF SE))" in spec)
apply (simp add: Rev_T_cont Rev_F_cont)
apply (simp add: Rev_T_def Rev_F_def)

apply (simp add: continuous_def)
apply (intro allI impI)
apply (drule_tac x="fst (Rep_dom_SF x)" in spec)
apply (drule_tac x="snd (Rep_dom_SF x)" in spec)
apply (simp)

apply (elim conjE exE)
apply (rule_tac x="n" in exI)
apply (intro allI impI)
apply (drule_tac x="fst (Rep_dom_SF y)" in spec)
apply (drule_tac x="snd (Rep_dom_SF y)" in spec)

apply (simp add: rest_SF_def)
apply (simp add: restSFpair_in Abs_dom_SF_inject)
apply (simp add: restSFpair_def)
done

end
