           (*-------------------------------------------*
            |                Csp-Prover                 |
            |               October 2004                |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Domain_SF_cpo = Domain_SF + Domain_T_cpo + Domain_F_cpo + CPO_pair:

(*****************************************************************

         1. 
         2. 
         3. 
         4. 

 *****************************************************************)


(*  The following simplification rules are deleted in this theory file *)
(*  because they unexpectly rewrite UnionT and InterT.                 *)
(*                  Union (B ` A) = (UN x:A. B x)                      *)
(*                  Inter (B ` A) = (INT x:A. B x)                     *)

declare Union_image_eq [simp del]
declare Inter_image_eq [simp del]

(*********************************************************
                      Bottom in Dom_SF
 *********************************************************)

instance dom_SF :: (type) bot0
apply (intro_classes)
done

defs (overloaded)
  bottom_SF_def   :  "Bot == Abs_dom_SF ({[]t}t , {}f)"

lemma bottom_SF: "ALL (SF::'a dom_SF). Bot <= SF"
apply (simp add: subset_SF_def)
apply (simp add: bottom_SF_def)
apply (simp add: BOT_in_dom_SF Abs_dom_SF_inverse)
apply (simp add: BOT_is_bottom_dom_SF Rep_dom_SF)
done

instance dom_SF :: (type) bot
apply (intro_classes)
apply (simp add: bottom_SF)
done

(********************************************************** 
      lemmas used in a proof that domain_SF is a cpo.
 **********************************************************)

(* lub_TF TFs is an upper bound of TFs *)

consts
  lub_TF :: "'a dom_T_F set => 'a dom_T_F"
  lub_SF :: "'a dom_SF set => 'a dom_SF"

defs
  lub_TF_def : "lub_TF TFs == (UnionT (fst ` TFs), UnionF (snd ` TFs))"
  lub_SF_def : "lub_SF SFs == Abs_dom_SF (lub_TF (Rep_dom_SF ` SFs))"

(************* lub_TF *************)

(*** lub_TF --> lub ***)

lemma lub_TF_is_lub: 
  "ALL TFs. TFs ~= {} --> TFs <<| lub_TF TFs"
apply (intro allI)
apply (simp add: fst_snd_lub lub_TF_def)
apply (simp add: Union_T_lub Union_F_lub)
done

(*** lub --> lub_TF ***)

lemma lub_is_lub_TF:
  "ALL TFs TF. TFs ~= {} & TFs <<| TF --> TF = lub_TF TFs"
apply (intro allI impI)
apply (insert lub_TF_is_lub)
apply (drule_tac x="TFs" in spec)
apply (simp)
apply (insert lub_unique)
apply (drule_tac x="TFs" in spec)
apply (drule_tac x="TF" in spec)
apply (drule_tac x="lub_TF TFs" in spec)
apply (simp)
done

(* rewrite *)

lemma lub_to_lub_TF : "ALL TFs TF. TFs ~= {} --> (TFs <<| TF) = (TF = lub_TF TFs)"
apply (intro allI impI)
apply (rule iffI)
apply (simp add: lub_is_lub_TF)
apply (simp add: lub_TF_is_lub)
done

(*** LUB TF = lub_TF ***)

lemma LUB_is_lub_TF:
  "ALL TFs. TFs ~= {} --> LUB TFs = lub_TF TFs"
apply (simp add: lub_TF_is_lub LUB_lub_imp)
done

(****** lub_TF TFs in dom_SF ******)

(*** T2 ***)

lemma lub_TF_in_T2: 
  "ALL TFs.  TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> CT2 (lub_TF TFs)"
apply (intro allI)
apply (simp (no_asm) add: CT2_def)
apply (intro allI impI)

apply (simp add: lub_TF_def)
apply (simp add: dom_T_Union_in_dom_T Abs_dom_T_inverse)
apply (simp add: dom_F_Union_in_dom_F Abs_dom_F_inverse)

apply (elim bexE conjE)
apply (rename_tac TFs s TF)

apply (simp add: dom_SF_def CT2_def)
apply (drule_tac x="TF" in bspec)
apply (simp)

apply (erule conjE)+
apply (drule_tac x="s" in spec)
apply (rule conjI)

  (* Evset *)

apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)

  (* EvsetTick *)

apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)
done

(* F2 *)

lemma lub_TF_in_F2: 
  "ALL TFs. TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> CF2 (lub_TF TFs)"
apply (simp add: lub_TF_def CF2_def)
apply (intro allI impI)

apply (simp add: dom_T_Union_in_dom_T Abs_dom_T_inverse)
apply (simp add: dom_F_Union_in_dom_F Abs_dom_F_inverse)
apply (elim bexE conjE)

apply (rename_tac TFs s X Y TF)

apply (simp add: dom_SF_def CF2_def)
apply (drule_tac x="TF" in bspec)
apply (simp)
apply (erule conjE)+
apply (drule_tac x="s" in spec)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Y" in spec)

apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)
done

(* F3 *)

lemma lub_TF_in_F3: 
  "ALL TFs. TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> CF3 (lub_TF TFs)"
apply (simp add: lub_TF_def CF3_def)
apply (intro allI impI)

apply (simp add: dom_T_Union_in_dom_T Abs_dom_T_inverse)
apply (simp add: dom_F_Union_in_dom_F Abs_dom_F_inverse)
apply (elim bexE conjE)

apply (elim exE bexE)
apply (rename_tac TFs s X TF)

apply (simp add: dom_SF_def CF3_def)

apply (drule_tac x="TF" in bspec)
apply (simp)

apply (erule conjE)+
apply (drule_tac x="s" in spec)
apply (drule mp)
apply (rule_tac x="X" in exI)
apply (simp)

apply (rule_tac x="TF" in bexI)
apply (simp)
apply (simp)
done

(*** lub_TF TFs in dom_SF ***)

lemma lub_TF_in: 
  "ALL TFs. TFs ~= {} & (ALL TF:TFs. TF:dom_SF) --> (lub_TF TFs): dom_SF"
apply (intro allI impI)
apply (simp (no_asm) add: dom_SF_def)
apply (simp add: lub_TF_in_T2)
apply (simp add: lub_TF_in_F2)
apply (simp add: lub_TF_in_F3)
done

(************* lub_SF *************)

(*** lub_SF --> lub ***)

lemma lub_SF_is_lub: 
  "ALL SFs. SFs ~= {} --> SFs <<| lub_SF SFs"
apply (intro allI impI)
apply (simp add: lub_SF_def)
apply (insert lub_TF_in)
apply (drule_tac x="Rep_dom_SF ` SFs" in spec)
apply (drule mp)

apply (simp add: directed_def)
apply (intro ballI impI)
apply (simp add: Rep_dom_SF)

apply (insert lub_TF_is_lub)
apply (drule_tac x="Rep_dom_SF ` SFs" in spec)
apply (simp add: is_lub_def is_ub_def)
apply (rule conjI)

(* ub *)
apply (intro allI impI)
apply (elim conjE)
apply (drule_tac x="fst (Rep_dom_SF y)" in spec)
apply (drule_tac x="snd (Rep_dom_SF y)" in spec)
apply (simp add: subset_SF_def)
apply (simp add: Abs_dom_SF_inverse)

(* lub *)
apply (intro allI impI)
apply (elim conjE)
apply (rotate_tac 4)
apply (drule_tac x="fst (Rep_dom_SF y)" in spec)
apply (drule_tac x="snd (Rep_dom_SF y)" in spec)
apply (simp)

apply (drule mp)
apply (intro allI impI)
apply (simp add: image_def)
apply (elim bexE)
apply (drule_tac x="x" in spec)
apply (simp add: subset_SF_def)
apply (simp add: subset_SF_def Abs_dom_SF_inverse)
done

(*** lub --> lub_SF ***)

lemma lub_is_lub_SF:
  "ALL SFs SF. SFs ~= {} & SFs <<| SF --> SF = lub_SF SFs"
apply (intro allI impI)
apply (insert lub_SF_is_lub)
apply (drule_tac x="SFs" in spec)
apply (simp)
apply (insert lub_unique)
apply (drule_tac x="SFs" in spec)
apply (drule_tac x="SF" in spec)
apply (drule_tac x="lub_SF SFs" in spec)
apply (simp)
done

(* rewrite *)

lemma lub_to_lub_SF : "ALL SFs SF. SFs ~= {} 
       --> (SFs <<| SF) = (SF = lub_SF SFs)"
apply (intro allI impI)
apply (rule iffI)
apply (simp add: lub_is_lub_SF)
apply (simp add: lub_SF_is_lub)
done

(*** LUB SF = lub_SF ***)

lemma LUB_is_lub_SF:
  "ALL SFs. SFs ~= {} --> LUB SFs = lub_SF SFs"
apply (simp add: lub_SF_is_lub LUB_lub_imp)
done

(********************************************************** 
                ( dom_SF, <= ) is a CPO
 **********************************************************)

instance dom_SF :: (type) cpo
apply (intro_classes)
apply (rule_tac x="lub_SF X" in exI)
apply (simp add: directed_def lub_SF_is_lub)
done

(********************************************************** 
             ( dom_SF, <= ) is a pointed CPO
 **********************************************************)

instance dom_SF :: (type) cpo_bot
apply (intro_classes)
done

(********************************************************** 
                 continuity of Abs_dom_SF
 **********************************************************)

(*** Abs_dom_SF ***)

lemma cont_Abs_dom_SF: 
 "ALL f. (ALL x. f x: dom_SF) & cont f --> cont (%x. Abs_dom_SF (f x))"
apply (intro allI impI)
apply (rule cont_iffI)
apply (intro allI impI)
apply (elim conjE)
apply (erule lub_cont_funE, simp, simp)
apply (simp add: directed_def)
apply (elim conjE)
apply (rotate_tac -1)
apply (erule rem_asmE)

apply (simp add: lub_to_lub_TF)
apply (simp add: lub_to_lub_SF)
apply (simp add: lub_SF_def)

apply (simp add: lub_TF_in Abs_dom_SF_inject Rep_dom_SF)
apply (simp add: lub_TF_def)
apply (rule conjI)

(* T *)

apply (simp add: dom_T_Union_in_dom_T Abs_dom_T_inject)
apply (rule equalityI)

 (* <= *)

apply (rule subsetI)
apply (simp)
apply (elim bexE)
apply (rule_tac x="xb" in bexI)
apply (simp add: Abs_dom_SF_inverse)
apply (simp)

 (* => *)

apply (rule subsetI)
apply (simp)
apply (elim bexE)
apply (rule_tac x="xb" in bexI)
apply (simp add: Abs_dom_SF_inverse)
apply (simp)

(* F *)

apply (simp add: dom_F_Union_in_dom_F Abs_dom_F_inject)
apply (rule equalityI)

 (* <= *)

apply (rule subsetI)
apply (simp)
apply (elim bexE)
apply (rule_tac x="xb" in bexI)
apply (simp add: Abs_dom_SF_inverse)
apply (simp)

 (* => *)

apply (rule subsetI)
apply (simp)
apply (elim bexE)
apply (rule_tac x="xb" in bexI)
apply (simp add: Abs_dom_SF_inverse)
apply (simp)
done

(*** Rep_dom_SF ***)

lemma cont_Rep_dom_SF: "cont Rep_dom_SF"
apply (rule cont_iffI)
apply (intro allI impI)
apply (elim conjE)
apply (simp add: directed_def)
apply (elim conjE)
apply (rotate_tac -1)
apply (erule rem_asmE)

apply (simp add: lub_to_lub_TF)
apply (simp add: lub_to_lub_SF)
apply (erule rem_asmE)
apply (simp add: lub_SF_def)

apply (simp add: lub_TF_in Abs_dom_SF_inverse Rep_dom_SF)
done

(***********************************************************
                          lemmas
 ***********************************************************)


(****************** to add them again ******************)

declare Union_image_eq [simp]
declare Inter_image_eq [simp]

end
