           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                June 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Domain_T = Prefix + RS:

(*****************************************************************

         1. 
         2. 
         3. 
         4. 

 *****************************************************************)

(*  The following simplification rules are deleted in this theory file *)
(*  because they unexpectly rewrite UnionT and InterT.                 *)
(*                  Union (B ` A) = (UN x:A. B x)                      *)
(*                  Inter (B ` A) = (INT x:A. B x)                     *)

declare Union_image_eq [simp del]
declare Inter_image_eq [simp del]

(***********************************************************
                 type def (Trace Part)
 ***********************************************************)

consts
  CT1    :: "'a trace set => bool" 

defs
  CT1_def : "CT1 T == (T ~= {} & prefix_closed T)"

typedef 'a dom_T = "{T::('a trace set). CT1(T)}"
apply (rule_tac x ="{[]t}" in exI)
apply (simp add: CT1_def prefix_closed_def)
apply (simp add: prefix_of_nil_t)
done

(***********************************************************
                   operators on dom_T
 ***********************************************************)

syntax
  "_mem_T"     :: "'a trace => 'a dom_T => bool"  ("(_/ :t _)" [50, 51] 50)
  "_nonmem_T"  :: "'a trace => 'a dom_T => bool"  ("(_/ ~:t _)" [50, 51] 50)
  "_Collect_T" :: "('a trace => bool) => 'a dom_T"("CollectT")
  "_Union_T"   :: "'a dom_T set => 'a dom_T"      ("UnionT _" [90] 90)
  "_Un_T"      :: "'a dom_T => 'a dom_T => 'a dom_T" ("_ UnT _" [65,66] 65)
  "_Inter_T"   :: "'a dom_T set => 'a dom_T"      ("InterT _" [90] 90)
  "_Int_T"     :: "'a dom_T => 'a dom_T => 'a dom_T" ("_ IntT _" [70,71] 70)

  "@Coll_T"    :: "pttrn => bool => 'a dom_T"     ("(1{_./ _}t)")
  "@Finset_T"  :: "args => 'a dom_T"              ("{(_)}t")

  "_emp_T"     :: "'a dom_T"                      ("{}t")
  "_UNIV_T"    :: "'a dom_T"                      ("UNIVt")


translations
  "x :t T"     == "x : (Rep_dom_T T)"
  "x ~:t T"    == "x ~: (Rep_dom_T T)"
  "CollectT P" == "Abs_dom_T (Collect P)"
  "UnionT Ts"  == "Abs_dom_T (Union (Rep_dom_T ` Ts))"
  "T UnT S"    == "UnionT {T,S}"
  "InterT Ts"  == "Abs_dom_T (Inter (Rep_dom_T ` Ts))"
  "T IntT S"   == "InterT {T,S}"

  "{x. P}t"    == "Abs_dom_T {x. P}"
  "{X}t"       == "Abs_dom_T {X}"

  "{}t"        == "Abs_dom_T {}"
  "UNIVt"      == "Abs_dom_T UNIV"


(*********************************************************
          The relation (<=) is defined over dom_T
 *********************************************************)

instance dom_T :: (type) ord 
apply (intro_classes)
done

defs (overloaded)
  subset_T_def   :  "T <= S  ==  (Rep_dom_T T) <= (Rep_dom_T S)"
  psubset_T_def  :  "T <  S  ==  (Rep_dom_T T) <  (Rep_dom_T S)"

(*********************************************************
          The relation (<=) is a partial order
 *********************************************************)

instance dom_T :: (type) order
apply (intro_classes)
apply (unfold subset_T_def psubset_T_def)
apply (simp)
apply (erule order_trans, simp)
apply (drule order_antisym, simp)
apply (simp add: Rep_dom_T_inject)
apply (simp only: order_less_le Rep_dom_T_inject)
done


(***********************************************************
                          lemmas
 ***********************************************************)

lemma Rep_dom_T_injectI: "Rep_dom_T S = Rep_dom_T T ==> S = T"
apply (simp add: Rep_dom_T_inject)
done

(*******************************
             basic
 *******************************)

lemma dom_T_is_non_empty: "ALL T. T:dom_T --> T ~= {}"
apply (simp add: dom_T_def CT1_def)
done

lemma dom_T_is_prefix_closed: 
  "ALL s t T. T:dom_T & t : T & prefix s t --> s : T"
apply (intro allI impI)
apply (simp add: dom_T_def CT1_def)
apply (erule conjE)+
apply (erule prefix_closedE)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (simp)
done

(*** {[]t} in dom_T ***)

lemma nil_t_set_in: "{[]t} : dom_T"
apply (simp add: dom_T_def CT1_def)
apply (simp add: prefix_closed_def prefix_of_nil_t)
done

(*** {[]t, [a]t} in dom_T ***)

lemma one_t_set_in: "ALL a. {[]t,  [a]t} : dom_T"
apply (intro allI)
apply (simp add: dom_T_def CT1_def)
apply (simp add: prefix_closed_def)
apply (intro allI impI)
apply (erule exE)
apply (erule conjE)
apply (erule disjE)
apply (simp add: prefix_of_nil_t)

apply (simp add: prefix_of_one)
done

(* []t is contained in all dom_T *)

lemma nil_t_in_all_dom: "ALL T. T : dom_T --> []t : T"
apply (intro allI impI)
apply (simp add: dom_T_def CT1_def)
apply (erule conjE)
apply (erule nonempty_exE)
apply (erule exE)
apply (erule prefix_closedE)
apply (drule_tac x="[]t" in spec)
apply (drule_tac x="x" in spec)
apply (simp add: nil_t_is_prefix)
done

(*******************************
        check in dom_T 
 *******************************)

(*** Union ***)

lemma dom_T_Union_in_dom_T:
  "ALL Ts. Ts ~= {} --> (Union (Rep_dom_T ` Ts)) : dom_T"
apply (intro allI impI)
apply (simp add: dom_T_def CT1_def)
apply (simp add: prefix_closed_def)
apply (rule conjI)

apply (erule nonempty_exE)
apply (erule exE)
apply (simp add: Bex_def)
apply (rule_tac x="x" in exI)
apply (simp add: Rep_dom_T dom_T_is_non_empty)

apply (intro allI impI)
apply (erule exE)
apply (simp add: Bex_def)
apply (erule conjE)
apply (erule exE)
apply (rename_tac Ts s t T)
apply (rule_tac x="T" in exI)

apply (insert dom_T_is_prefix_closed)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule_tac x="Rep_dom_T T" in spec)
apply (simp add: Rep_dom_T)
done

(*** Un ***)

lemma dom_T_Un_in_dom_T:
  "ALL T S. (Rep_dom_T T Un Rep_dom_T S) : dom_T"
apply (intro allI)
apply (insert dom_T_Union_in_dom_T)
apply (drule_tac x="{T,S}" in spec)
apply (simp)
done

(*** Inter ***)

lemma dom_T_Inter_in_dom_T:
  "ALL Ts. (Inter (Rep_dom_T ` Ts)) : dom_T"
apply (intro allI)
apply (simp add: dom_T_def CT1_def)
apply (simp add: prefix_closed_def)
apply (rule conjI)

apply (rule ex_nonemptyI)
apply (rule_tac x="[]t" in exI)
apply (simp)

apply (simp add: Ball_def)
apply (intro allI impI)
apply (simp add: nil_t_in_all_dom Rep_dom_T)

apply (intro allI impI)
apply (simp add: Ball_def)
apply (intro allI impI)
apply (rename_tac Ts s T)

apply (erule exE)
apply (erule conjE)
apply (drule_tac x="T" in spec)

apply (insert dom_T_is_prefix_closed)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule_tac x="Rep_dom_T T" in spec)
apply (simp add: Rep_dom_T)
done

(*** Int ***)

lemma dom_T_Int_in_dom_T:
  "ALL T S. (Rep_dom_T T Int Rep_dom_T S) : dom_T"
apply (intro allI)
apply (insert dom_T_Inter_in_dom_T)
apply (drule_tac x="{T,S}" in spec)
apply (simp)
done

(*******************************
    dom_T type --> set type
 *******************************)

(*** UnionT ***)

lemma dom_T_UnionT_Rep:
  "ALL Ts. Ts ~= {} --> Rep_dom_T (UnionT Ts) = Union (Rep_dom_T ` Ts)"
apply (simp add: Abs_dom_T_inverse dom_T_Union_in_dom_T)
done

(*** UnT ***)

lemma dom_T_UnT_Rep:
  "ALL T S. Rep_dom_T (T UnT S) = (Rep_dom_T T) Un (Rep_dom_T S)"
apply (simp add: Abs_dom_T_inverse dom_T_Un_in_dom_T)
done

(*** InterT ***)

lemma dom_T_InterT_Rep:
  "ALL Ts. Rep_dom_T (InterT Ts) = Inter (Rep_dom_T ` Ts)"
apply (simp add: Abs_dom_T_inverse dom_T_Inter_in_dom_T)
done

(*** IntT ***)

lemma dom_T_IntT_Rep:
  "ALL T S. Rep_dom_T (T IntT S) = (Rep_dom_T T) Int (Rep_dom_T S)"
apply (simp add: Abs_dom_T_inverse dom_T_Int_in_dom_T)
done

(*********************************************************
                 Restriction in Dom_T
 *********************************************************)

instance dom_T :: (type) ms0
apply (intro_classes)
done

consts
  restTset :: "'a dom_T => nat => 'a trace set" ("_ restTset _" [55,56] 55)
  limitTset:: "'a dom_T infinite_seq => 'a trace set"

defs
 restTset_def  : "T restTset n == {s. s :t T & (lengtht s) <= n}"
 limitTset_def : "limitTset Ts == {s. s :t Ts (lengtht s)}"

defs (overloaded)
  rest_T_def : "T rest n == Abs_dom_T (T restTset n)"

syntax
  "_limitT"  :: "'a dom_T infinite_seq => 'a dom_T"   ("limitT")

translations
  "limitT Ts" == "Abs_dom_T (limitTset Ts)"

(**********************************************************
              Lemmas (Restriction in Dom_T)
 **********************************************************)

(*** restTset_def in dom_T ***)

lemma restTset_def_in : "ALL T n. {s. s :t T & (lengtht s) <= n}: dom_T"
apply (intro allI)
apply (simp add: dom_T_def CT1_def)
apply (rule conjI)

apply (rule_tac x="[]t" in exI)
apply (simp add: nil_t_in_all_dom Rep_dom_T)
apply (simp add: trace_rep_nil_t)

apply (simp add: prefix_closed_def)
apply (intro allI)
apply (intro impI)
apply (erule exE)

apply (insert dom_T_is_prefix_closed)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule_tac x="Rep_dom_T T" in spec)
apply (simp add: Rep_dom_T)

apply (insert prefix_trace_length)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (blast intro: order_trans)
done

(*** restTset in dom_T ***)

lemma restTset_in : "ALL T n. T restTset n : dom_T"
apply (simp add: restTset_def restTset_def_in)
done

(*********************************************************
                     Dom_T --> RS
 *********************************************************)

(*******************************
        zero_eq_rs_dom_T
 *******************************)

(*** restTset 0 ***)

lemma zero_dom_T: "ALL (T::'a dom_T). T restTset 0 = {[]t}"
apply (intro allI)
apply (simp add: restTset_def)
apply (rule order_antisym)

apply (rule subsetI)
apply (simp add: trace_rep_nil_t_inv)
apply (rule subsetI)
apply (simp add: Rep_dom_T nil_t_in_all_dom trace_rep_nil_t)
done

(*** zero_eq_rs_dom_T ***)

lemma zero_eq_rs_dom_T: "ALL (T::'a dom_T) S. T rest 0 = S rest 0"
apply (simp add: rest_T_def)
apply (simp add: restTset_in Abs_dom_T_inject)
apply (simp add: zero_dom_T)
done

(*******************************
         min_rs_dom_T
 *******************************)

lemma min_rs_dom_T:
  "ALL (T::'a dom_T) (m::nat) (n::nat). 
     (T rest m) rest n = T rest (min m n)"
apply (intro allI)
apply (simp add: rest_T_def)
apply (simp add: restTset_in Abs_dom_T_inject)

apply (simp add: restTset_def)
apply (simp add: restTset_def_in Abs_dom_T_inverse)
done

(*******************************
         diff_rs_dom_T
 *******************************)

(*** contra <= ***)

lemma contra_diff_rs_dom_T_left: 
  "ALL (T::'a dom_T) (S::'a dom_T).
    (ALL (n::nat). T rest n = S rest n) --> T <= S"
apply (intro allI)
apply (intro impI)
apply (simp add: subset_T_def)

apply (rule subsetI)
apply (drule_tac x="lengtht x" in spec)
apply (simp add: rest_T_def)
apply (simp add: restTset_in Abs_dom_T_inject)
apply (auto simp add: restTset_def) 
done

(*** contra = ***)

lemma contra_diff_rs_dom_T: 
  "ALL (T::'a dom_T) (S::'a dom_T).
    (ALL (n::nat). T rest n = S rest n) --> T = S"
apply (intro allI)
apply (intro impI)
apply (rule order_antisym)
apply (simp_all add: contra_diff_rs_dom_T_left)
done

(*** diff_rs_dom_T ***)

lemma diff_rs_dom_T: 
  "ALL (T::'a dom_T) (S::'a dom_T).
    T ~= S --> (EX (n::nat). T rest n ~= S rest n)"
apply (intro allI)
apply (intro impI)
apply (erule contrapos_pp)
apply (insert contra_diff_rs_dom_T)
apply (drule_tac x="T" in spec)
apply (drule_tac x="S" in spec)
apply (simp)
done

(***************************************************************
                        dom_T ==> RS
 ***************************************************************)

instance dom_T :: (type) rs
apply (intro_classes)
apply (simp add: zero_eq_rs_dom_T)
apply (simp add: min_rs_dom_T)
apply (simp add: diff_rs_dom_T)
done

(************************************************************
                        dom_T ==> MS
 ************************************************************)

instance dom_T :: (type) ms
apply (intro_classes)
apply (simp add: positive_rs)
apply (simp add: diagonal_rs)
apply (simp add: symmetry_rs)
apply (simp add: triangle_inequality_rs)
done

(************************************************************
                 i.e.  dom_T ==> MS & RS 
 ************************************************************)

instance dom_T :: (type) ms_rs
apply (intro_classes)
done

(***********************************************************
                      lemmas (Limit)
 ***********************************************************)

(*** normal_seq lemma ***)

lemma normal_seq_dom_T:
  "ALL (Ts::'a dom_T infinite_seq) s n. normal Ts & lengtht s <= n
      --> (s :t Ts (lengtht s)) = (s :t Ts n)"
apply (intro allI)
apply (intro impI)
apply (erule conjE)+

apply (simp add: normal_def)
apply (drule_tac x="lengtht s" in spec)
apply (drule_tac x="n" in spec)
apply (simp add: min_x_y)

apply (insert rest_nat_le_1_if)
apply (drule_tac x="Ts (lengtht s)" in spec)
apply (drule_tac x="Ts n" in spec)
apply (drule_tac x="lengtht s" in spec)
apply (simp add: rest_T_def restTset_in Abs_dom_T_inject)
apply (simp add: restTset_def)

apply (simp add: Collect_open)
apply (drule_tac x="s" in spec)
apply (simp)
done

(*** limitTset_def in dom_T ***)

lemma limitTset_def_in:
  "ALL (Ts::'a dom_T infinite_seq). normal Ts 
       --> {s. s :t Ts (lengtht s)} : dom_T"
apply (intro allI impI)
apply (simp add: dom_T_def CT1_def)
apply (rule conjI)

apply (rule_tac x="[]t" in exI)
apply (simp add: nil_t_in_all_dom Rep_dom_T)

apply (simp add: prefix_closed_def)
apply (intro allI impI)
apply (erule exE)

apply (insert dom_T_is_prefix_closed)
apply (drule_tac x="s" in spec)
apply (drule_tac x="t" in spec)
apply (drule_tac x="Rep_dom_T (Ts (lengtht t))" in spec)
apply (simp add: Rep_dom_T)

apply (insert normal_seq_dom_T)
apply (drule_tac x="Ts" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="lengtht t" in spec)

apply (simp add: prefix_trace_length)
done

(*** limitTset in dom_T ***)

lemma limitTset_in:
  "ALL (Ts::'a dom_T infinite_seq). normal Ts --> limitTset Ts : dom_T"
apply (simp add: limitTset_def_in limitTset_def)
done

(*** limitT lemma ***)

lemma limitT_limit_lm:
  "ALL (Ts::'a dom_T infinite_seq). 
     normal Ts --> (ALL n. (limitT Ts) rest n = (Ts n) rest n)"
apply (intro allI impI)
apply (simp add: rest_T_def restTset_in Abs_dom_T_inject)

apply (simp add: limitTset_def restTset_def)
apply (simp add: limitTset_def_in Abs_dom_T_inverse)
apply (auto simp add: normal_seq_dom_T)
done

(*** (normal) Ts converges to (limitT Ts) ***)

lemma limitT_limit:
  "ALL (Ts::'a dom_T infinite_seq). normal Ts --> Ts convergeTo (limitT Ts)"
apply (simp add: limitT_limit_lm rest_limit)
done

(*** (cauchy) Ts converges to (limitT NF Ts) ***)

lemma cauchy_limitT_limit:
  "ALL (Ts::'a dom_T infinite_seq). 
      cauchy Ts --> Ts convergeTo (limitT (NF Ts))"
apply (simp add: limitT_limit normal_form_seq_same_limit 
                 normal_form_seq_normal)
done

(***************************************
     Dom_T --> Complete Metric Space
 ***************************************)

lemma dom_T_cms:
  "ALL (Ts::'a dom_T infinite_seq). 
      cauchy Ts --> (EX T. Ts convergeTo T)"
apply (intro allI impI)
apply (rule_tac x="limitT (NF Ts)" in exI)
apply (simp add: cauchy_limitT_limit)
done

(************************************************************
                   dom_T ==> CMS and RS
 ************************************************************)

instance dom_T :: (type) cms
apply (intro_classes)
apply (simp add: dom_T_cms)
done

instance dom_T :: (type) cms_rs
apply (intro_classes)
done

(*** (normal) limit Ts = limitT Ts ***)

lemma limitT_limit_eq:
  "ALL (Ts::'a dom_T infinite_seq). normal Ts --> limit Ts = limitT Ts"
apply (intro allI impI)
apply (insert unique_convergence)
apply (drule_tac x="Ts" in spec)
apply (drule_tac x="limit Ts" in spec)
apply (drule_tac x="limitT Ts" in spec)
apply (simp add:  limitT_limit the_limit normal_cauchy)
done

(*----------------------------------------------------------*
 |                                                          |
 |         Fixed point induction for refinement             |
 |                                                          |
 *----------------------------------------------------------*)

consts
  Ref_T :: "'a dom_T => 'a dom_T => bool"
  Rev_T :: "'a dom_T => 'a dom_T => bool"

defs
  Ref_T_def : "Ref_T S == (%T. (S <= T))"
  Rev_T_def : "Rev_T S == (%T. (T <= S))"


(************************************************************
             continuity lemma for refinement
 ************************************************************)

lemma Ref_T_cont: "ALL S. continuous (Ref_T S)"
apply (intro allI)
apply (simp add: continuous_def)
apply (intro allI impI)
apply (simp add: Ref_T_def subset_T_def)
apply (simp add: subset_def Bex_def)
apply (elim conjE exE)
apply (rename_tac S T t)

apply (rule_tac x="lengtht t" in exI)
apply (intro allI impI)
apply (rule_tac x="t" in exI)

apply (simp add: rest_T_def)
apply (simp add: restTset_in Abs_dom_T_inject)
apply (simp add: restTset_def)
apply (simp add: Collect_open)
apply (drule_tac x="t" in spec)
apply (simp)
done

(************************************************************
         continuity lemma for (Reverse) refinement
 ************************************************************)

lemma Rev_T_cont: "ALL S. continuous (Rev_T S)"
apply (intro allI)
apply (simp add: continuous_def)
apply (intro allI impI)
apply (simp add: Rev_T_def subset_T_def)
apply (simp add: subset_def Bex_def)
apply (elim conjE exE)
apply (rename_tac S T t)

apply (rule_tac x="lengtht t" in exI)
apply (intro allI impI)
apply (rule_tac x="t" in exI)

apply (simp add: rest_T_def)
apply (simp add: restTset_in Abs_dom_T_inject)
apply (simp add: restTset_def)
apply (simp add: Collect_open)
apply (drule_tac x="t" in spec)
apply (simp)
done

(****************** to add them again ******************)

declare Union_image_eq [simp]
declare Inter_image_eq [simp]

end

