           (*-------------------------------------------*
            |                Csp-Prover                 |
            |              September 2004               |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Domain_T_cpo = Domain_T + CPO:

(*****************************************************************

         1. Domain_T is a pointed cpo.
         2. 
         3. 
         4. 

 *****************************************************************)

(*  The following simplification rules are deleted in this theory file *)
(*  because they unexpectly rewrite UnionT and InterT.                 *)
(*                  Union (B ` A) = (UN x:A. B x)                      *)
(*                  Inter (B ` A) = (INT x:A. B x)                     *)

declare Union_image_eq [simp del]
declare Inter_image_eq [simp del]

(*********************************************************
                      Bottom in Dom_T
 *********************************************************)

instance dom_T :: (type) bot0
apply (intro_classes)
done

defs (overloaded)
  bottom_T_def   :  "Bot == {[]t}t"

lemma bottom_T : "ALL (T::'a dom_T). Bot <= T"
apply (intro allI)
apply (simp add: bottom_T_def)
apply (simp add: subset_T_def)
apply (simp add: nil_t_set_in Abs_dom_T_inverse)
apply (simp add: Rep_dom_T nil_t_in_all_dom)
done

instance dom_T :: (type) bot
apply (intro_classes)
apply (simp add: bottom_T)
done

(********************************************************** 
      lemmas used in a proof that domain_T is a cpo.
 **********************************************************)

(* UnionT Ts is an upper bound of Ts *)

lemma Union_T_is_ub : "ALL Ts. Ts <| UnionT Ts"
apply (intro allI)
apply (simp add: is_ub_def)
apply (intro allI, intro impI)
apply (simp add: subset_T_def)
apply (erule ex_nonemptyE)
apply (simp add: dom_T_Union_in_dom_T Abs_dom_T_inverse)
apply (rule subsetI)
apply (simp add: UNION_def)
apply (rule_tac x=y in bexI)
apply (simp)
apply (simp)
done

(* UnionT Ts is the least upper bound of Ts *)

lemma Union_T_lub : "ALL Ts. (Ts ~= {} --> Ts <<| UnionT Ts)"
apply (intro allI impI)
apply (simp add: is_lub_def Union_T_is_ub)
apply (simp add: is_ub_def subset_T_def)
apply (simp add: dom_T_Union_in_dom_T Abs_dom_T_inverse)
apply (intro allI impI)

apply (rule subsetI)
apply (simp add: UNION_def)
apply (erule bexE)
apply (drule_tac x="xa" in spec)
apply (auto)
done

(* the least upper bound of Ts is UnionT Ts *)

lemma lub_Union_T : "ALL Ts T. (Ts ~= {} & Ts <<| T --> T = UnionT Ts)"
apply (intro allI impI)
apply (insert Union_T_lub)
apply (drule_tac x="Ts" in spec)
apply (simp)
apply (elim conjE)
apply (insert lub_unique)
apply (drule_tac x="Ts" in spec)
apply (drule_tac x="T" in spec)
apply (drule_tac x="UnionT Ts" in spec)
apply (simp)
done

(* rewrite *)

lemma lub_to_Union_T : "ALL Ts T. Ts ~= {} --> (Ts <<| T) = (T = UnionT Ts)"
apply (intro allI impI)
apply (rule iffI)
apply (simp add: lub_Union_T)
apply (simp add: Union_T_lub)
done

(* LUB is UnionT Ts *)

lemma LUB_Union_T : "ALL Ts. Ts ~= {} --> LUB Ts = UnionT Ts"
apply (intro allI impI)
apply (simp add: Union_T_lub LUB_lub_imp)
done

(********************************************************** 
                 ( dom_T, <= ) is a CPO
 **********************************************************)

instance dom_T :: (type) cpo
apply (intro_classes)
apply (rule_tac x="UnionT X" in exI)
apply (simp add: directed_def Union_T_lub)
done

(********************************************************** 
              ( dom_T, <= ) is a pointed CPO
 **********************************************************)

instance dom_T :: (type) cpo_bot
apply (intro_classes)
done

(***********************************************************
                          lemmas
 ***********************************************************)


(****************** to add them again ******************)

declare Union_image_eq [simp]
declare Inter_image_eq [simp]

end


