           (*-------------------------------------------*
            |                Csp-Prover                 |
            |                June 2004                  |
            |        Yoshinao Isobe (AIST JAPAN)        |
            *-------------------------------------------*)

theory Trace_hide = Prefix:

(*****************************************************************

         1. 
         2. 
         3. 
         4. 

 *****************************************************************)

consts
  hide_list :: "'a list => 'a set => 'a list"   ("_ --list _" [84,85] 84)

primrec
  "[] --list X    = []"
  "(a#t) --list X = (if a:X then (t --list X) 
                              else a#(t --list X))"

consts
  hide_rep  :: "'a trace => 'a set => 'a event list"
                                    ("_ --rep _" [84,85] 84)

defs
  hide_rep_def : 
    "s --rep X == (Rep_trace s) --list (Ev ` X)"

syntax
  "_hide_tr"  :: "'a trace => 'a set => 'a event list"
                                    ("_ --tr _" [84,85] 84)

translations
  "s --tr X" == "Abs_trace (s --rep X)"

(*************************************************************
                      hide list middle
 *************************************************************)

(*** in ***)

lemma hide_list_middle_in:
  "ALL X s x t. x:X --> 
      (s @ [x] @ t) --list X = (s --list X) @ (t --list X)"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

(*** notin ***)

lemma hide_list_middle_notin:
  "ALL X s x t. x~:X --> 
      (s @ [x] @ t) --list X = (s --list X) @ [x] @ (t --list X)"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

(*************************************************************
                      hide list last
 *************************************************************)

(*** in ***)

lemma hide_list_last_in:
  "ALL X s x. x:X --> 
      (s @ [x]) --list X = s --list X"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

(*** notin ***)

lemma hide_list_last_notin:
  "ALL X s x t. x~:X --> 
      (s @ [x]) --list X = (s --list X) @ [x]"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

(*************************************************************
                    hide list reconst HIDE
 *************************************************************)

lemma hide_list_reconst:
  "ALL X t s1 x s2. t --list X = s1 @ [x] @ s2 
      --> (EX t1 t2. t = t1 @ [x] @ t2 &
                     s1 = t1 --list X & s2 = t2 --list X) &
                     x ~: X"
apply (rule allI)
apply (rule allI)
apply (induct_tac t)
apply (simp)

apply (intro allI impI)
apply (case_tac "a:X")

(* a:X *)

apply (simp)
apply (drule_tac x="s1" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="s2" in spec)
apply (simp)
apply (elim conjE exE)
apply (rule_tac x="a#t1" in exI)
apply (rule_tac x="t2" in exI)
apply (simp)

(* a~:X *)

apply (simp)
apply (insert list_nil_or_unnil)
apply (rotate_tac -1)
apply (drule_tac x="s1" in spec)
apply (erule disjE)

(* s1 = [] *)

apply (simp)
apply (rule_tac x="[]" in exI)
apply (rule_tac x="list" in exI)
apply (simp)

(* s1 ~= [] *)

apply (erule exE)+
apply (simp)
apply (rename_tac X y t' s1 x s2 z s1')

apply (drule_tac x="s1'" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="s2" in spec)
apply (simp)

apply (elim conjE exE)
apply (rule_tac x="y#t1" in exI)
apply (rule_tac x="t2" in exI)
apply (simp)
done

(*************************************************************
                hide list reconst HIDE (last)
 *************************************************************)

lemma hide_list_reconst_last:
  "ALL X t s1 x. t --list X = s1 @ [x]
      --> (EX t' t1. t' --list X = t --list X &
                     t' = t1 @ [x] & s1 = t1 --list X)"
apply (intro allI impI)
apply (insert hide_list_reconst)
apply (drule_tac x="X" in spec)
apply (drule_tac x="t" in spec)
apply (drule_tac x="s1" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="[]" in spec)
apply (simp)
apply (elim conjE exE)

apply (rule_tac x="t1 @ [x]" in exI)
apply (simp)

apply (insert hide_list_middle_notin)
apply (drule_tac x="X" in spec)
apply (drule_tac x="t1" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="[]" in spec)
apply (simp)
done

(*************************************************************
           a new event is not introduced by HIDE
 *************************************************************)

lemma hide_list_in_event:
  "ALL X a s. a : set(s --list X) --> a~:X & a : set s"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

lemma contra_hide_list_in_event:
  "ALL X a s. (a:X | a ~: set s) --> a ~: set(s --list X)"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

lemma hide_list_notick:
  "ALL X s. Tick ~: set s  --> Tick ~: set(s --list X)"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

lemma hide_list_subseteq:
  "ALL X s. set(s --list X) <= set s"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

(*************************************************************
                events notin X is not hidden
 *************************************************************)

lemma hide_list_in_event_inv:
  "ALL X a s. a : set s & a~:X --> a : set(s --list X)"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done

lemma contra_hide_list_in_event_inv:
  "ALL X a s. a ~: set(s --list X) & a ~: X --> a ~: set s"
apply (intro allI)
apply (induct_tac s)
apply (auto)
done


(*************************************************************
                    hide rep in trace
 *************************************************************)

(*** hide list in trace ***)

lemma hide_list_in_trace:
 "ALL X (s::'a event list). s:trace --> (s --list (Ev ` X)) :trace"
apply (intro allI impI)

apply (insert list_last_nil_or_unnil)
apply (drule_tac x="s" in spec)
apply (erule disjE)

apply (simp add: trace_def)

apply (erule exE)+
apply (simp add: trace_def)

apply (insert hide_list_subseteq)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="sa" in spec)

apply (case_tac "a:Ev ` X")

      (* a : Ev ` X *)
apply (simp add: hide_list_last_in)
apply (insert butlast_subseteq)
apply (drule_tac x="(sa --list Ev ` X)" in spec)
apply (blast)

      (* a ~: Ev ` X *)
apply (simp add: hide_list_last_notin)
apply (blast)
done

(*** hide rep in trace ***)

lemma hide_rep_in_trace:
 "ALL X (s::'a trace). s --rep X :trace"
apply (intro allI)
apply (simp add: hide_rep_def)

apply (insert hide_list_in_trace)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Rep_trace s" in spec)
apply (simp add: Rep_trace)
done

(*************************************************************
                   hide list prefix_closed
 *************************************************************)

lemma hide_list_prefix:
 "ALL X s u. s:trace & prefix_list u (s --list X)
     --> (EX t. u = t --list X & prefix_list t s)"

apply (rule allI)
apply (rule allI)
apply (induct_tac s)

 (* base case *)

apply (intro allI impI)
apply (rule_tac x="[]" in exI)
apply (simp add: prefix_list_def)

 (* step case *)

apply (intro allI impI)

apply (case_tac "a:X")

      (* a:X *)
apply (simp)
apply (drule_tac x="u" in spec)
apply (drule mp)
apply (blast intro: event_app_in_trace_invI)

apply (erule exE)
apply (rule_tac x="a#t" in exI)
apply (simp add: prefix_list_def)

      (* a~:X *)
apply (simp)

apply (insert prefix_list_head_same_inv)
apply (rotate_tac -1)
apply (drule_tac x="u" in spec)
apply (drule_tac x="a" in spec)
apply (rotate_tac -1)
apply (drule_tac x="(list --list X)" in spec)
apply (simp)
apply (erule disjE)

        (* u=[] *)

apply (rule_tac x="[]" in exI)
apply (simp add: prefix_list_def)

        (* u = a # v' *)

apply (erule exE)
apply (drule_tac x="v'" in spec)
apply (drule mp)
apply (blast intro: event_app_in_trace_invI)

apply (erule exE)
apply (rule_tac x="a#t" in exI)
apply (simp add: prefix_list_def)
done

(*                                                  *)
(*                   list --> trace                 *)
(*                                                  *)

(*************************************************************
        a new event is not introduced by HIDE (trace)
 *************************************************************)

lemma hide_trace_in_event:
  "ALL X a s. a : sett(s --tr X) --> a~:Ev ` X & a : sett s"
apply (intro allI impI)
apply (simp add: hide_rep_def)
apply (simp add: hide_list_in_trace Rep_trace Abs_trace_inverse)
apply (insert hide_list_in_event)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="a" in spec)
apply (drule_tac x="Rep_trace s" in spec)
apply (simp)
done

(* contraposition *)

lemma contra_hide_trace_in_event:
  "ALL X a s. (a : Ev ` X | a ~: sett s) --> a ~: sett (s --tr X)"
apply (intro allI impI)
apply (erule contrapos_pp)
apply (insert hide_trace_in_event)
apply (drule_tac x="X" in spec)
apply (drule_tac x="a" in spec)
apply (drule_tac x="s" in spec)
apply (simp)
done

(* notick *)

lemma hide_trace_notick:
  "ALL X s. notick s --> notick (s --tr X)"
apply (intro allI impI)
apply (insert contra_hide_trace_in_event)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Tick" in spec)
apply (drule_tac x="s" in spec)
apply (simp)
done

(*************************************************************
              events notin X is not hidden (trace)
 *************************************************************)

lemma contra_hide_trace_in_event_inv:
  "ALL X a s. a ~: sett (s --tr X) & a ~: Ev ` X --> a ~: sett s"
apply (intro allI impI)
apply (simp add: hide_rep_def)
apply (simp add: hide_list_in_trace Rep_trace Abs_trace_inverse)
apply (insert contra_hide_list_in_event_inv)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="a" in spec)
apply (drule_tac x="Rep_trace s" in spec)
apply (simp)
done

(* notick *)

lemma hide_trace_notick_inv:
  "ALL X s. notick (s --tr X) --> notick s"
apply (intro allI impI)
apply (insert contra_hide_trace_in_event_inv)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Tick" in spec)
apply (drule_tac x="s" in spec)
apply (simp add: image_def)
done

(*************************************************************
                      hide trace nil
 *************************************************************)

lemma hide_trace_nil: "ALL X. []t --tr X = []t"
apply (intro allI)
apply (simp add: hide_rep_def)
apply (simp add: nil_in_trace Abs_trace_inverse)
done

(*************************************************************
                      hide trace middle
 *************************************************************)

(*** in ***)

lemma hide_trace_middle_in:
  "ALL X s x t. notick s & x: Ev ` X --> 
      (s @t [x]t @t t) --tr X = (s --tr X) @t (t --tr X)"
apply (intro allI impI)
apply (insert hide_list_middle_in)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="Rep_trace s" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="Rep_trace t" in spec)
apply (simp add: hide_rep_def)
apply (simp add: hide_list_in_trace Rep_trace Abs_trace_inverse)

apply (simp add: app_trace_notick Rep_trace hide_list_notick
                 hide_list_in_trace Abs_trace_inject)

apply (simp add: event_in_trace Abs_trace_inverse)
apply (case_tac "x = Tick")

apply (simp add: image_def)

apply (simp add: notick_app_in_trace Rep_trace Abs_trace_inverse
                 app_trace_notick hide_list_in_trace)
done

(*** notin ***)

lemma hide_trace_middle_notin:
  "ALL X s x t. notick s & [x]t @rep t: trace & x ~: Ev ` X --> 
      (s @t [x]t @t t) --tr X = (s --tr X) @t [x]t @t (t --tr X)"
apply (intro allI impI)
apply (insert hide_list_middle_notin)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="Rep_trace s" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="Rep_trace t" in spec)
apply (simp add: hide_rep_def)
apply (simp add: hide_list_in_trace Rep_trace Abs_trace_inverse)
apply (simp add: event_in_trace Abs_trace_inverse)

apply (insert hide_list_in_trace)
apply (drule_tac x="X" in spec)
apply (drule_tac x="x # Rep_trace t" in spec)
apply (simp add: Abs_trace_inverse)

apply (simp add: app_trace_notick Rep_trace hide_list_notick
                 hide_list_in_trace Abs_trace_inject)

apply (simp add: notick_app_in_trace Rep_trace Abs_trace_inverse
                 app_trace_notick hide_list_in_trace)
done

(*************************************************************
                      hide trace head
 *************************************************************)

(*** in ***)

lemma hide_trace_head_in:
  "ALL X s x. x: Ev ` X -->  ([x]t @t s) --tr X = (s --tr X)"
apply (intro allI impI)
apply (insert hide_trace_middle_in)
apply (drule_tac x="X" in spec)
apply (drule_tac x="[]t" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="s" in spec)
apply (simp add: nil_in_trace Abs_trace_inverse hide_trace_nil)
apply (simp add: Rep_trace_inverse)
done

(*** notin ***)

lemma hide_trace_head_notin:
  "ALL X s x. [x]t @rep s : trace & x ~: Ev ` X 
        --> ([x]t @t s) --tr X = [x]t @t (s --tr X)"
apply (intro allI impI)
apply (insert hide_trace_middle_notin)
apply (drule_tac x="X" in spec)
apply (drule_tac x="[]t" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="s" in spec)
apply (simp add: nil_in_trace Abs_trace_inverse hide_trace_nil)
apply (simp add: Rep_trace_inverse)
done

(*************************************************************
                      hide trace last
 *************************************************************)

(*** in ***)

lemma hide_trace_last_in:
  "ALL X s x. notick s & x: Ev ` X --> 
      (s @t [x]t) --tr X = (s --tr X)"
apply (intro allI impI)
apply (insert hide_trace_middle_in)
apply (drule_tac x="X" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="[]t" in spec)
apply (simp add: nil_in_trace Abs_trace_inverse hide_trace_nil)
apply (simp add: Rep_trace_inverse)
done

(*** notin ***)

lemma hide_trace_last_notin:
  "ALL X s x. notick s & x ~: Ev ` X --> 
      (s @t [x]t) --tr X = (s --tr X) @t [x]t"
apply (intro allI impI)
apply (insert hide_trace_middle_notin)
apply (drule_tac x="X" in spec)
apply (drule_tac x="s" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="[]t" in spec)
apply (simp add: nil_in_trace event_in_trace Abs_trace_inverse
                 hide_trace_nil)
done

(*************************************************************
                    hide trace reconst HIDE
 *************************************************************)

lemma hide_trace_reconst:
  "ALL X t s1 x s2.  
      notick s1 & [x]t @rep s2: trace & t --tr X = s1 @t [x]t @t s2 
      --> (EX t1 t2. t = t1 @t [x]t @t t2 &
                     s1 = t1 --tr X & s2 = t2 --tr X) &
                     x ~: Ev ` X"
apply (intro allI impI)
apply (insert hide_list_reconst)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="Rep_trace t" in spec)
apply (drule_tac x="Rep_trace s1" in spec)
apply (drule_tac x="x" in spec)
apply (drule_tac x="Rep_trace s2" in spec)
apply (simp add: hide_rep_def)

apply (erule conjE)+
apply (simp add: event_in_trace Abs_trace_inverse)
apply (simp add: app_trace_notick Rep_trace 
                 hide_list_in_trace Abs_trace_inject)

apply (elim conjE exE)
apply (rule_tac x="Abs_trace t1" in exI)
apply (rule_tac x="Abs_trace t2" in exI)

      (* for t1:trace & t2:trace *)

apply (insert app_trace_in_trace_inv2)
apply (drule_tac x="t1" in spec)
apply (drule_tac x="x # t2" in spec)
apply (simp add: Rep_trace_eq)
apply (erule conjE)
apply (insert event_app_in_trace_inv)
apply (drule_tac x="x" in spec)
apply (drule_tac x="t2" in spec)
apply (simp)

apply (simp add: Abs_trace_inverse)
apply (simp add: Abs_Rep_t_inv_imp)
done

(*************************************************************
                 hide trace reconst HIDE (Tick)
 *************************************************************)

lemma hide_trace_reconst_tick:
  "ALL X t s1.  
      notick s1 & t --tr X = s1 @t [Tick]t
      --> (EX t1. t = t1 @t [Tick]t & s1 = t1 --tr X)"
apply (intro allI impI)
apply (insert hide_list_reconst)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="Rep_trace t" in spec)
apply (drule_tac x="Rep_trace s1" in spec)
apply (drule_tac x="Tick" in spec)
apply (drule_tac x="[]" in spec)
apply (simp add: hide_rep_def)
apply (simp add: event_in_trace Abs_trace_inverse)

apply (insert hide_list_in_trace)
apply (drule_tac x="X" in spec)
apply (drule_tac x="Rep_trace t" in spec)
apply (simp add: Rep_trace)
apply (insert app_trace_notick)
apply (drule_tac x="Rep_trace s1" in spec)
apply (drule_tac x="[Tick]" in spec)
apply (simp add: Rep_trace event_in_trace)

apply (simp add: Abs_trace_inject)
apply (elim conjE exE)

apply (rule_tac x="Abs_trace t1" in exI)

      (* for t1:trace *)

apply (insert app_trace_in_trace_inv2)
apply (drule_tac x="t1" in spec)
apply (drule_tac x="Tick # t2" in spec)
apply (simp add: Rep_trace_eq)
apply (erule conjE)

      (* for t2 = [] *)

apply (insert Tick_app_nil_in_trace)
apply (drule_tac x="t2" in spec)
apply (simp)

apply (simp add: Abs_trace_inverse)
apply (simp add: Abs_Rep_t_inv_imp)
done


(*************************************************************
                  hide trace prefix_closed
 *************************************************************)

lemma hide_trace_prefix:
 "ALL X s u. prefix u (s --tr X) --> (EX t. u = t --tr X & prefix t s)"
apply (intro allI impI)
apply (insert hide_list_prefix)
apply (drule_tac x="Ev ` X" in spec)
apply (drule_tac x="Rep_trace s" in spec)
apply (drule_tac x="Rep_trace u" in spec)
apply (simp add: Rep_trace hide_rep_def hide_list_in_trace Abs_trace_inverse)
apply (erule exE)
apply (rule_tac x="Abs_trace t" in exI)

apply (insert prefix_list_s_trace)
apply (drule_tac x="t" in spec)
apply (drule_tac x="Rep_trace s" in spec)
apply (simp add: Abs_trace_inverse)
apply (simp add: Abs_Rep_t_inv_imp)
done

(*************************************************************
                      hide trace Tick
 *************************************************************)

lemma hide_trace_Tick: "ALL X. [Tick]t --tr X = [Tick]t"
apply (intro allI)
apply (simp add: hide_rep_def)
apply (simp add: event_in_trace Abs_trace_inverse)
apply (simp add: image_iff)
done

end
