           (*-------------------------------------------------*
            |                 (a part of) ep2                 |
            |                  September 2004                 |
            |  Markus Roggenbach (Univ of Wales Swansea, UK)  |
            |  Yoshinao Isobe    (AIST, Japan)                |
            *-------------------------------------------------*)

theory ep2_acl = Csp_Prover:

(*********************************************************
          automatic unfolding syntactic-sugar
 *********************************************************)

declare csp_ss_def [simp]

(*********************************************************
              data type passed on channels
 *********************************************************)

typedecl D_SI_Init_SessionStart
typedecl D_SI_Init_SessionEnd
typedecl D_SI_Init_ConfigDataRequest
typedecl D_SI_Init_ConfigDataResponse
typedecl D_SI_Init_ConfigDataNotification
typedecl D_SI_Init_ConfigDataAcknowledge
typedecl D_SI_Init_RemoveConfigDataNotification
typedecl D_SI_Init_RemoveConfigDataAcknowledge
typedecl D_SI_Init_ActivateConfigDataNotification
typedecl D_SI_Init_ActivateConfigDataAcknowledge

datatype D_SI_Init = 
     SStart D_SI_Init_SessionStart
   | SEnd   D_SI_Init_SessionEnd
   | CDReq  D_SI_Init_ConfigDataRequest
   | CDRes  D_SI_Init_ConfigDataResponse
   | CDN    D_SI_Init_ConfigDataNotification
   | CDA    D_SI_Init_ConfigDataAcknowledge
   | RCDN   D_SI_Init_RemoveConfigDataNotification
   | RCDA   D_SI_Init_RemoveConfigDataAcknowledge
   | ACDN   D_SI_Init_ActivateConfigDataNotification
   | ACDA   D_SI_Init_ActivateConfigDataAcknowledge

typedecl TerminalState
typedecl Trigger
typedecl Message

(*********************************************************
                     event (channel)
 *********************************************************)

datatype Event = C_SI_Init D_SI_Init
               | C_TerminalDisplay Message
               | PairTT "TerminalState * Trigger"

(*********************************************************
         abstract component description level
 *********************************************************)

datatype ACName = TInit
                | TConfigurationManagement
                | AcquirerInit
                | ConfigurationManagement

consts 
  ACDef :: "(ACName, Event) procDef"

primrec
  "ACDef (TInit) 
    = C_SI_Init !! x: (range SStart)
      -> <TConfigurationManagement>"

  "ACDef (TConfigurationManagement)
    = C_SI_Init ? x ->
       IF (x:range CDReq)
       THEN C_SI_Init !! y: (range CDRes)
            -> <TConfigurationManagement>

       ELSE IF (x:range CDN)
       THEN C_SI_Init !! y: (range CDA)
            -> <TConfigurationManagement>

       ELSE IF (x:range RCDN)
       THEN C_SI_Init !! y: (range RCDA)
            -> <TConfigurationManagement>

       ELSE IF (x:range ACDN)
       THEN C_SI_Init !! y: (range ACDA)
            -> <TConfigurationManagement> 

       ELSE IF (x:range SEnd)
       THEN SKIP
       ELSE STOP"

  "ACDef (AcquirerInit)
    = C_SI_Init ? sessionStart: (range SStart)
      -> <ConfigurationManagement>"

  "ACDef (ConfigurationManagement)
    =     C_SI_Init !! sessionEnd:(range SEnd) -> SKIP
      |~| C_SI_Init !! request:(range CDReq)
          -> C_SI_Init ? response:(range CDRes)
          -> <ConfigurationManagement>
      |~| C_SI_Init !! notification:(range CDN)
          -> C_SI_Init ? acknowledge:(range CDA)
          -> <ConfigurationManagement>
      |~| C_SI_Init !! notification:(range RCDN)
          -> C_SI_Init ? acknowledge:(range RCDA)
          -> <ConfigurationManagement>
      |~| C_SI_Init !! notification:(range ACDN)
          -> C_SI_Init ? acknowledge:(range ACDA)
          -> <ConfigurationManagement>"

consts
  AC :: "(ACName, Event) procRec"

defs
  AC_def:
   "AC == 
     LET ACDef
     IN (<TInit> |[range C_SI_Init]| <AcquirerInit>)"

(*********************************************************
        nohide and guarded lemmas (routine work)
 *********************************************************)

lemma guardACDef[simp]: "!!C. guard ((ACDef) C)"
apply (induct_tac C, simp_all)
done

lemma nohideACDef[simp]: "!!C. nohide ((ACDef) C)"
apply (induct_tac C, simp_all)
done

(*********************************************************
               abstract level (deadlock free)
 *********************************************************)

syntax
  "_REQs" :: "D_SI_Init" ("REQs")
  "_RESs" :: "D_SI_Init" ("RESs")

translations
  "REQs" == "(range CDReq) Un (range CDN) Un 
             (range RCDN)  Un (range ACDN)"
  "RESs" == "(range CDRes) Un (range CDA) Un 
             (range RCDA)  Un (range ACDA)"

datatype AbsName = Abstract | Loop

consts 
  AbsDef :: "(AbsName, Event) procDef"

primrec
  "AbsDef (Abstract)
          = C_SI_Init !! init:(range SStart) -> <Loop>"

  "AbsDef (Loop)
          = C_SI_Init !! exit:(range SEnd) -> SKIP
            |~| 
            C_SI_Init !! request:REQs
              -> C_SI_Init !! response:RESs
                -> <Loop>"

consts
  Abs :: "(AbsName, Event) procRec"

defs
  Abs_def:
   "Abs == 
     LET AbsDef IN <Abstract>"

(*********************************************************
        nohide and guarded lemmas (routine work)
 *********************************************************)

lemma guardAbsDef[simp]: "!!C. guard ((AbsDef) C)"
apply (induct_tac C, simp_all)
done

lemma nohideAbsDef[simp]: "!!C. nohide ((AbsDef) C)"
apply (induct_tac C, simp_all)
done

(*********************************************************
        relating function between AbsName and ACName
 *********************************************************)

consts 
  Abs_to_AC :: "AbsName => (ACName, Event) proc"

primrec
  "Abs_to_AC (Abstract)
          = (<TInit> |[range C_SI_Init]| <AcquirerInit>)"

  "Abs_to_AC (Loop)
          = (<TConfigurationManagement> |[range C_SI_Init]|
             <ConfigurationManagement>)"

(*********************************************************
           a theorem for verifying Abs <=F AC
               (i.e. AC is deadlock-free)
 *********************************************************)

(*** initializing ***)

lemma ep2_step1:
   "LET ACDef IN Rewrite (AbsDef Abstract) By Abs_to_AC <=F 
    LET ACDef IN Abs_to_AC Abstract"
apply (simp)
apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_dist_tac 1 *})
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (tactic {* csp_simp_tac 1 *})
done

(*** main loop ***)

lemma ep2_step2:
   "LET ACDef IN Rewrite (AbsDef Loop) By Abs_to_AC <=F 
    LET ACDef IN Abs_to_AC Loop"
apply (simp)
apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_dist_tac 1 *})
apply (auto)

 (*** skip ***)
apply (tactic {* csp_step_tac 1 *})
apply (rule csp_Int_choice_left1)
apply (rule csp_decompo_subset)
apply (auto)
apply (simp add: image_iff)
apply (tactic {* csp_step_tac 1 *})

 (*** C_SI_Init ? ***)
apply (rule csp_Int_choice_left2)
apply (rule csp_decompo_subset)
apply (auto)
apply (simp add: image_iff)
apply (tactic {* csp_simp_tac 1 *})

apply (tactic {* csp_dist_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (auto)
apply (tactic {* csp_simp_tac 1 *})

 (*** CD req res ***)
apply (rule csp_Int_choice_left2)
apply (tactic {* csp_step_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (simp add: image_iff)
apply (tactic {* csp_simp_tac 1 *})

apply (tactic {* csp_dist_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (auto)
apply (tactic {* csp_simp_tac 1 *})

 (*** RCD ***)
apply (rule csp_Int_choice_left2)
apply (tactic {* csp_step_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (simp add: image_iff)
apply (tactic {* csp_simp_tac 1 *})

apply (tactic {* csp_dist_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (auto)
apply (tactic {* csp_simp_tac 1 *})

 (*** ACD ***)
apply (rule csp_Int_choice_left2)
apply (tactic {* csp_step_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (simp add: image_iff)
apply (tactic {* csp_simp_tac 1 *})

apply (tactic {* csp_dist_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (auto)
apply (tactic {* csp_simp_tac 1 *})
done

(**********************
      Abs <=F AC
 **********************)

theorem ep2_abs: "Abs <=F AC"

  (***** by fixed point induction *****)
apply (rule csp_fp_induct[of _ _ "Abs_to_AC"])
apply (rule)

  (***** check guarded and no hiding operators *****)
apply (simp_all add: Abs_def AC_def)

   (*** recursion ***)
apply (induct_tac C)
apply (simp add: ep2_step1 del: AbsDef.simps Abs_to_AC.simps)
apply (simp add: ep2_step2 del: AbsDef.simps Abs_to_AC.simps)
done

end
