           (*-------------------------------------------------*
            |                 (a part of) ep2                 |
            |                  September 2004                 |
            |  Markus Roggenbach (Univ of Wales Swansea, UK)  |
            |  Yoshinao Isobe    (AIST, Japan)                |
            *-------------------------------------------------*)

theory ep2_ccl = ep2_acl:

(*********************************************************
                    functions
 *********************************************************)

(* pair *)
consts
  state  :: "TerminalState * Trigger => TerminalState"
  trigger:: "TerminalState * Trigger => Trigger"

defs
  state_def  : "state p == fst p"
  trigger_def: "trigger p == snd p"

(* Session start *)
consts
  sessionStart:: "Trigger => D_SI_Init_SessionStart"

(* ConfigDataRequest *)
consts
  configDataResponse:: 
    "D_SI_Init_ConfigDataRequest * TerminalState => 
     D_SI_Init_ConfigDataResponse"

(* ConfigDataNotification *)
consts
  configDataAcknowledge:: D_SI_Init_ConfigDataAcknowledge            (* !! *)
  configData::                                                        (* !! *)
    "D_SI_Init_ConfigDataNotification * TerminalState =>
     TerminalState"

(* ConfigDataRemove *)
consts
  removeDataAcknowledge:: D_SI_Init_RemoveConfigDataAcknowledge   (* ?? *)
  removeData:: "D_SI_Init_RemoveConfigDataNotification * TerminalState =>
                TerminalState"

(* ActivateConfigDataNotification *)
consts
  activateDataAcknowledge:: D_SI_Init_ActivateConfigDataAcknowledge   (* ?? *)
  activateData:: "D_SI_Init_ActivateConfigDataNotification * TerminalState =>  (* !! *)
                  TerminalState"

(* Message *)
consts
  AcqConnectionFailed    :: Message
  InitialisationFinished :: Message
  InitialisationFailed   :: Message

(*********************************************************
         concrete component description level
 *********************************************************)

datatype CCName = CTInit "TerminalState * Trigger"
                | CTConfigurationManagement "TerminalState * Trigger"
                | CAcquirerInit
                | CConfigurationManagement

consts 
  CCDef :: "(CCName, Event) procDef"

primrec
  "CCDef (CTInit p) 
    = C_SI_Init ! (SStart (sessionStart (trigger p)))
      -> <CTConfigurationManagement p>"

  "CCDef (CTConfigurationManagement p)
    = C_SI_Init ? x ->
       IF (x:range CDReq)
       THEN C_SI_Init ! 
               (CDRes (configDataResponse 
               (((inv CDReq) x),(state p)))) 
            -> <CTConfigurationManagement p>

       ELSE IF (x:range CDN)
       THEN C_SI_Init ! (CDA configDataAcknowledge)
            -> <CTConfigurationManagement 
                ((configData (((inv CDN) x),(state p))), (trigger p))>

       ELSE IF (x:range RCDN)
       THEN C_SI_Init ! (RCDA removeDataAcknowledge)             (* !! *)
            -> <CTConfigurationManagement 
                (removeData((inv RCDN) x,(state p)), (trigger p))>

       ELSE IF (x:range ACDN)
       THEN C_SI_Init ! (ACDA activateDataAcknowledge)           (* !! *)
            -> <CTConfigurationManagement 
                (activateData((inv ACDN) x,(state p)), (trigger p))>

       ELSE IF (x:range SEnd)
       THEN C_TerminalDisplay ! InitialisationFinished -> SKIP
       ELSE STOP"

  "CCDef (CAcquirerInit)
    = C_SI_Init ? sessionStart: (range SStart)
      -> <CConfigurationManagement>"

  "CCDef (CConfigurationManagement)
    =     C_SI_Init !! sessionEnd:(range SEnd) -> SKIP
      |~| C_SI_Init !! request:(range CDReq)
          -> C_SI_Init ? response:(range CDRes)
          -> <CConfigurationManagement>
      |~| C_SI_Init !! notification:(range CDN)
          -> C_SI_Init ? acknowledge:(range CDA)
          -> <CConfigurationManagement>
      |~| C_SI_Init !! notification:(range RCDN)
          -> C_SI_Init ? acknowledge:(range RCDA)
          -> <CConfigurationManagement>
      |~| C_SI_Init !! notification:(range ACDN)
          -> C_SI_Init ? acknowledge:(range ACDA)
          -> <CConfigurationManagement>"

consts
  CC :: "(TerminalState * Trigger) => (CCName, Event) procRec"

defs
  CC_def:
   "CC p == 
     LET CCDef
     IN (<CTInit p> -- (range C_TerminalDisplay) |[range C_SI_Init]| 
         <CAcquirerInit>)"

(*********************************************************
        nohide and guarded lemmas (routine work)
 *********************************************************)

lemma guardCCDef[simp]: "!!C. guard ((CCDef) C)"
apply (induct_tac C, simp_all)
done

lemma nohideCCDef[simp]: "!!C. nohide ((CCDef) C)"
apply (induct_tac C, simp_all)
done

(*********************************************************
        relating function between AbsName and ACName
 *********************************************************)

consts 
  AC_to_CC :: "ACName => (CCName, Event) proc"

primrec
  "AC_to_CC (TInit)                    =  
     PairTT!!p .. <CTInit p> -- range C_TerminalDisplay"

  "AC_to_CC (TConfigurationManagement) = 
     PairTT!!p ..  <CTConfigurationManagement p> -- range C_TerminalDisplay"

  "AC_to_CC (AcquirerInit)             = <CAcquirerInit>"
  "AC_to_CC (ConfigurationManagement)  = <CConfigurationManagement>"

(*********************************************************
           a theorem for verifying !!p. AC <=F CC p
 *********************************************************)

(*** initializing (terminal) ***)

lemma ep2_ccl_terminal_step1:
    "LET CCDef IN Rewrite (ACDef TInit) By AC_to_CC <=F 
     LET CCDef IN AC_to_CC TInit"
apply (auto)
apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_step_tac 1 *})
apply (simp add: image_iff)
apply (tactic {* csp_simp_tac 1 *})
apply (rule csp_decompo_subset)
apply (auto)
apply (rule csp_Int_bind_left)
apply (rule_tac x="PairTT(aa, b)" in exI)
apply (simp)
done

(*** loop (terminal) ***)

lemma ep2_ccl_terminal_step2:
    "LET CCDef IN Rewrite (ACDef TConfigurationManagement) By AC_to_CC <=F 
     LET CCDef IN AC_to_CC TConfigurationManagement"
apply (auto)
apply (tactic {* csp_unwind_tac 1 *})
apply (tactic {* csp_step_tac 1 *})
apply (simp add: image_iff)
apply (subgoal_tac "range C_SI_Init Int range C_TerminalDisplay = {}")
apply (tactic {* csp_simp_tac 1 *})
apply (auto)

apply (case_tac "x : range CDReq")
apply (subgoal_tac "(EX xa. x = CDReq xa)", erule exE)
apply (tactic {* csp_step_tac 1 *})
apply (subgoal_tac "(C_SI_Init (CDRes (configDataResponse (xa, state (aa, b)))) ~: range C_TerminalDisplay)")
apply (tactic {* csp_simp_tac 1 *})
apply (auto)

apply (rule csp_decompo_subset)
apply (auto)
apply (rule csp_Int_bind_left)
apply (rule_tac x="PairTT(aa, b)" in exI)
apply (simp)

apply (case_tac "(EX xa. x = CDReq xa)")
apply (auto)
apply (tactic {* csp_step_tac 1 *})

apply (case_tac "(EX xa. x = CDN xa)")
apply (auto)
apply (tactic {* csp_step_tac 1 *})

apply (subgoal_tac "(C_SI_Init (CDA configDataAcknowledge) ~: range C_TerminalDisplay)")
apply (tactic {* csp_simp_tac 1 *})
apply (auto)

apply (rule csp_decompo_subset)
apply (auto)
apply (rule csp_Int_bind_left)
apply (rule_tac x="PairTT(configData (xa, state (aa, b)), trigger (aa, b))" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (case_tac "(EX xa. x = RCDN xa) ")
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (subgoal_tac "(C_SI_Init (RCDA removeDataAcknowledge) ~: range C_TerminalDisplay)")
apply (auto)
apply (tactic {* csp_simp_tac 1 *})

apply (rule csp_decompo_subset)
apply (auto)
apply (rule csp_Int_bind_left)
apply (rule_tac x="PairTT(removeData (xa, state (aa, b)), trigger (aa, b))" in exI)
apply (simp)

apply (tactic {* csp_simp_tac 1 *})
apply (case_tac "(EX xa. x = ACDN xa)")
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (subgoal_tac "(C_SI_Init (ACDA activateDataAcknowledge) ~: range C_TerminalDisplay)")
apply (auto)
apply (tactic {* csp_simp_tac 1 *})

apply (rule csp_decompo_subset)
apply (auto)
apply (rule csp_Int_bind_left)
apply (rule_tac x="PairTT(activateData (xa, state (aa, b)), trigger (aa, b))" in exI)
apply (simp)

 (*** C_TerminalDisplay ***)

apply (tactic {* csp_simp_tac 1 *})
apply (case_tac "(EX xa. x = SEnd xa)")
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (auto)
apply (tactic {* csp_step_tac 1 *})
apply (auto)
done

(****************************
      !!p. AC p <=F CC p
 ****************************)

theorem ep2_ccl: "!!p. AC <=F CC p"

  (***** by fixed point induction *****)
apply (rule csp_fp_induct[of _ _ "AC_to_CC"])
apply (rule)

  (***** check guarded and no hiding operators *****)
apply (simp_all add: AC_def CC_def)

   (*** body ***)
apply (rule csp_decompo, auto)
apply (rule csp_Int_bind_left)
apply (rule_tac x="PairTT (a, b)" in exI)
apply (simp)

apply (induct_tac C)
apply (simp add: ep2_ccl_terminal_step1 del: ACDef.simps AC_to_CC.simps)
apply (simp add: ep2_ccl_terminal_step2 del: ACDef.simps AC_to_CC.simps)
apply (tactic {* csp_unwind_tac 1 *})+
done

(****************************
      !!p. Abs <=F CC p
 ****************************)

theorem ep2_ccl: "!!p. Abs <=F CC p"

apply (rule csp_trans)
apply (rule ep2_abs)
apply (rule ep2_ccl)
done

end
